@@ -9,12 +9,14 @@
 
 #include <memory>
 
-#include "core/fxcrt/fx_basic.h"
+#include "core/fxcrt/retain_ptr.h"
+#include "core/fxcrt/unowned_ptr.h"
 #include "core/fxge/fx_dib.h"
 
 class CPDF_ImageObject;
 class CPDF_PageRenderCache;
 class CPDF_RenderStatus;
+class IFX_PauseIndicator;
 
 class CPDF_ImageLoader {
  public:
@@ -26,23 +28,19 @@ class CPDF_ImageLoader {
              bool bStdCS,
              uint32_t GroupFamily,
              bool bLoadMask,
-             CPDF_RenderStatus* pRenderStatus,
-             int32_t nDownsampleWidth,
-             int32_t nDownsampleHeight);
-  bool Continue(IFX_Pause* pPause);
+             CPDF_RenderStatus* pRenderStatus);
+  bool Continue(IFX_PauseIndicator* pPause, CPDF_RenderStatus* pRenderStatus);
 
-  CFX_RetainPtr<CFX_DIBSource> m_pBitmap;
-  CFX_RetainPtr<CFX_DIBSource> m_pMask;
+  RetainPtr<CFX_DIBSource> m_pBitmap;
+  RetainPtr<CFX_DIBSource> m_pMask;
   uint32_t m_MatteColor;
   bool m_bCached;
 
  private:
   void HandleFailure();
 
-  int32_t m_nDownsampleWidth;
-  int32_t m_nDownsampleHeight;
-  CPDF_PageRenderCache* m_pCache;
-  CPDF_ImageObject* m_pImage;
+  UnownedPtr<CPDF_PageRenderCache> m_pCache;
+  UnownedPtr<CPDF_ImageObject> m_pImageObject;
 };
 
 #endif  // CORE_FPDFAPI_RENDER_CPDF_IMAGELOADER_H_
