@@ -25,9 +25,9 @@
 #include "core/fpdfapi/render/cpdf_renderstatus.h"
 #include "core/fpdfapi/render/cpdf_transferfunc.h"
 #include "core/fpdfdoc/cpdf_occontext.h"
-#include "core/fxcrt/cfx_maybe_owned.h"
 #include "core/fxcrt/fx_safe_types.h"
-#include "core/fxge/cfx_fxgedevice.h"
+#include "core/fxcrt/maybe_owned.h"
+#include "core/fxge/cfx_defaultrenderdevice.h"
 #include "core/fxge/cfx_pathdata.h"
 #include "core/fxge/dib/cfx_dibitmap.h"
 #include "core/fxge/dib/cfx_dibsource.h"
@@ -59,14 +59,10 @@ bool CPDF_ImageRenderer::StartLoadDIBSource() {
   if (!image_rect.Valid())
     return false;
 
-  int dest_width =
-      m_ImageMatrix.a >= 0 ? image_rect.Width() : -image_rect.Width();
-  int dest_height =
-      m_ImageMatrix.d <= 0 ? image_rect.Height() : -image_rect.Height();
-  if (m_Loader.Start(
-          m_pImageObject, m_pRenderStatus->m_pContext->GetPageCache(), m_bStdCS,
-          m_pRenderStatus->m_GroupFamily, m_pRenderStatus->m_bLoadMask,
-          m_pRenderStatus, dest_width, dest_height)) {
+  if (m_Loader.Start(m_pImageObject.Get(),
+                     m_pRenderStatus->GetContext()->GetPageCache(), m_bStdCS,
+                     m_pRenderStatus->GetGroupFamily(),
+                     m_pRenderStatus->GetLoadMask(), m_pRenderStatus.Get())) {
     m_Status = 4;
     return true;
   }
@@ -77,26 +73,23 @@ bool CPDF_ImageRenderer::StartRenderDIBSource() {
   if (!m_Loader.m_pBitmap)
     return false;
 
-  m_BitmapAlpha =
-      FXSYS_round(255 * m_pImageObject->m_GeneralState.GetFillAlpha());
+  CPDF_GeneralState& state = m_pImageObject->m_GeneralState;
+  m_BitmapAlpha = FXSYS_round(255 * state.GetFillAlpha());
   m_pDIBSource = m_Loader.m_pBitmap;
-  if (m_pRenderStatus->m_Options.m_ColorMode == RENDER_COLOR_ALPHA &&
+  if (m_pRenderStatus->GetRenderOptions()->ColorModeIs(
+          CPDF_RenderOptions::kAlpha) &&
       !m_Loader.m_pMask) {
     return StartBitmapAlpha();
   }
-  if (m_pImageObject->m_GeneralState.GetTR()) {
-    if (!m_pImageObject->m_GeneralState.GetTransferFunc()) {
-      m_pImageObject->m_GeneralState.SetTransferFunc(
-          m_pRenderStatus->GetTransferFunc(
-              m_pImageObject->m_GeneralState.GetTR()));
-    }
-    if (m_pImageObject->m_GeneralState.GetTransferFunc() &&
-        !m_pImageObject->m_GeneralState.GetTransferFunc()->m_bIdentity) {
+  if (state.GetTR()) {
+    if (!state.GetTransferFunc())
+      state.SetTransferFunc(m_pRenderStatus->GetTransferFunc(state.GetTR()));
+
+    if (state.GetTransferFunc() && !state.GetTransferFunc()->GetIdentity()) {
       m_pDIBSource = m_Loader.m_pBitmap =
-          m_pImageObject->m_GeneralState.GetTransferFunc()->TranslateImage(
-              m_Loader.m_pBitmap);
+          state.GetTransferFunc()->TranslateImage(m_Loader.m_pBitmap);
       if (m_Loader.m_bCached && m_Loader.m_pMask)
-        m_Loader.m_pMask = m_Loader.m_pMask->Clone();
+        m_Loader.m_pMask = m_Loader.m_pMask->Clone(nullptr);
       m_Loader.m_bCached = false;
     }
   }
@@ -110,65 +103,51 @@ bool CPDF_ImageRenderer::StartRenderDIBSource() {
       if (m_pPattern)
         m_bPatternColor = true;
     }
-    m_FillArgb = m_pRenderStatus->GetFillArgb(m_pImageObject);
-  } else if (m_pRenderStatus->m_Options.m_ColorMode == RENDER_COLOR_GRAY) {
-    m_pClone = m_pDIBSource->Clone();
-    m_pClone->ConvertColorScale(m_pRenderStatus->m_Options.m_BackColor,
-                                m_pRenderStatus->m_Options.m_ForeColor);
-    m_pDIBSource = m_pClone;
+    m_FillArgb = m_pRenderStatus->GetFillArgb(m_pImageObject.Get());
+  } else if (m_pRenderStatus->GetRenderOptions()->ColorModeIs(
+                 CPDF_RenderOptions::kGray)) {
+    RetainPtr<CFX_DIBitmap> pClone = m_pDIBSource->Clone(nullptr);
+    if (!pClone)
+      return false;
+
+    pClone->ConvertColorScale(0xffffff, 0);
+    m_pDIBSource = pClone;
   }
   m_Flags = 0;
-  if (m_pRenderStatus->m_Options.m_Flags & RENDER_FORCE_DOWNSAMPLE)
+  if (m_pRenderStatus->GetRenderOptions()->HasFlag(RENDER_FORCE_DOWNSAMPLE))
     m_Flags |= RENDER_FORCE_DOWNSAMPLE;
-  else if (m_pRenderStatus->m_Options.m_Flags & RENDER_FORCE_HALFTONE)
+  else if (m_pRenderStatus->GetRenderOptions()->HasFlag(RENDER_FORCE_HALFTONE))
     m_Flags |= RENDER_FORCE_HALFTONE;
-  if (m_pRenderStatus->m_pDevice->GetDeviceClass() != FXDC_DISPLAY) {
-    CPDF_Object* pFilters =
-        m_pImageObject->GetImage()->GetStream()->GetDict()->GetDirectObjectFor(
-            "Filter");
-    if (pFilters) {
-      if (pFilters->IsName()) {
-        CFX_ByteString bsDecodeType = pFilters->GetString();
-        if (bsDecodeType == "DCTDecode" || bsDecodeType == "JPXDecode")
-          m_Flags |= FXRENDER_IMAGE_LOSSY;
-      } else if (CPDF_Array* pArray = pFilters->AsArray()) {
-        for (size_t i = 0; i < pArray->GetCount(); i++) {
-          CFX_ByteString bsDecodeType = pArray->GetStringAt(i);
-          if (bsDecodeType == "DCTDecode" || bsDecodeType == "JPXDecode") {
-            m_Flags |= FXRENDER_IMAGE_LOSSY;
-            break;
-          }
-        }
-      }
-    }
-  }
-  if (m_pRenderStatus->m_Options.m_Flags & RENDER_NOIMAGESMOOTH)
+
+  if (m_pRenderStatus->GetRenderDevice()->GetDeviceClass() != FXDC_DISPLAY)
+    HandleFilters();
+
+  if (m_pRenderStatus->GetRenderOptions()->HasFlag(RENDER_NOIMAGESMOOTH))
     m_Flags |= FXDIB_NOSMOOTH;
   else if (m_pImageObject->GetImage()->IsInterpol())
     m_Flags |= FXDIB_INTERPOL;
+
   if (m_Loader.m_pMask)
     return DrawMaskedImage();
 
   if (m_bPatternColor)
-    return DrawPatternImage(m_pObj2Device);
+    return DrawPatternImage(m_pObj2Device.Get());
 
-  if (m_BitmapAlpha != 255 || !m_pImageObject->m_GeneralState ||
-      !m_pImageObject->m_GeneralState.GetFillOP() ||
-      m_pImageObject->m_GeneralState.GetOPMode() != 0 ||
-      m_pImageObject->m_GeneralState.GetBlendType() != FXDIB_BLEND_NORMAL ||
-      m_pImageObject->m_GeneralState.GetStrokeAlpha() != 1.0f ||
-      m_pImageObject->m_GeneralState.GetFillAlpha() != 1.0f) {
+  if (m_BitmapAlpha != 255 || !state.HasRef() || !state.GetFillOP() ||
+      state.GetOPMode() != 0 || state.GetBlendType() != FXDIB_BLEND_NORMAL ||
+      state.GetStrokeAlpha() != 1.0f || state.GetFillAlpha() != 1.0f) {
     return StartDIBSource();
   }
   CPDF_Document* pDocument = nullptr;
   CPDF_Page* pPage = nullptr;
-  if (m_pRenderStatus->m_pContext->GetPageCache()) {
-    pPage = m_pRenderStatus->m_pContext->GetPageCache()->GetPage();
-    pDocument = pPage->m_pDocument;
+  if (auto* pPageCache = m_pRenderStatus->GetContext()->GetPageCache()) {
+    pPage = pPageCache->GetPage();
+    pDocument = pPage->m_pDocument.Get();
   } else {
     pDocument = m_pImageObject->GetImage()->GetDocument();
   }
-  CPDF_Dictionary* pPageResources = pPage ? pPage->m_pPageResources : nullptr;
+  CPDF_Dictionary* pPageResources =
+      pPage ? pPage->m_pPageResources.Get() : nullptr;
   CPDF_Object* pCSObj =
       m_pImageObject->GetImage()->GetStream()->GetDict()->GetDirectObjectFor(
           "ColorSpace");
@@ -186,18 +165,20 @@ bool CPDF_ImageRenderer::StartRenderDIBSource() {
 }
 
 bool CPDF_ImageRenderer::Start(CPDF_RenderStatus* pStatus,
-                               CPDF_PageObject* pObj,
+                               CPDF_ImageObject* pImageObject,
                                const CFX_Matrix* pObj2Device,
                                bool bStdCS,
                                int blendType) {
+  ASSERT(pImageObject);
   m_pRenderStatus = pStatus;
   m_bStdCS = bStdCS;
-  m_pImageObject = pObj->AsImage();
+  m_pImageObject = pImageObject;
   m_BlendType = blendType;
   m_pObj2Device = pObj2Device;
   CPDF_Dictionary* pOC = m_pImageObject->GetImage()->GetOC();
-  if (pOC && m_pRenderStatus->m_Options.m_pOCContext &&
-      !m_pRenderStatus->m_Options.m_pOCContext->CheckOCGVisible(pOC)) {
+  if (pOC && m_pRenderStatus->GetRenderOptions()->GetOCContext() &&
+      !m_pRenderStatus->GetRenderOptions()->GetOCContext()->CheckOCGVisible(
+          pOC)) {
     return false;
   }
   m_ImageMatrix = m_pImageObject->matrix();
@@ -208,7 +189,7 @@ bool CPDF_ImageRenderer::Start(CPDF_RenderStatus* pStatus,
 }
 
 bool CPDF_ImageRenderer::Start(CPDF_RenderStatus* pStatus,
-                               const CFX_RetainPtr<CFX_DIBSource>& pDIBSource,
+                               const RetainPtr<CFX_DIBSource>& pDIBSource,
                                FX_ARGB bitmap_argb,
                                int bitmap_alpha,
                                const CFX_Matrix* pImage2Device,
@@ -227,13 +208,14 @@ bool CPDF_ImageRenderer::Start(CPDF_RenderStatus* pStatus,
 }
 
 bool CPDF_ImageRenderer::NotDrawing() const {
-  return m_pRenderStatus->m_bPrint &&
-         !(m_pRenderStatus->m_pDevice->GetRenderCaps() & FXRC_BLEND_MODE);
+  return m_pRenderStatus->IsPrint() &&
+         !(m_pRenderStatus->GetRenderDevice()->GetRenderCaps() &
+           FXRC_BLEND_MODE);
 }
 
 FX_RECT CPDF_ImageRenderer::GetDrawRect() const {
   FX_RECT rect = m_ImageMatrix.GetUnitRect().GetOuterRect();
-  rect.Intersect(m_pRenderStatus->m_pDevice->GetClipBox());
+  rect.Intersect(m_pRenderStatus->GetRenderDevice()->GetClipBox());
   return rect;
 }
 
@@ -244,15 +226,15 @@ CFX_Matrix CPDF_ImageRenderer::GetDrawMatrix(const FX_RECT& rect) const {
 }
 
 void CPDF_ImageRenderer::CalculateDrawImage(
-    CFX_FxgeDevice* pBitmapDevice1,
-    CFX_FxgeDevice* pBitmapDevice2,
-    const CFX_RetainPtr<CFX_DIBSource>& pDIBSource,
+    CFX_DefaultRenderDevice* pBitmapDevice1,
+    CFX_DefaultRenderDevice* pBitmapDevice2,
+    const RetainPtr<CFX_DIBSource>& pDIBSource,
     CFX_Matrix* pNewMatrix,
     const FX_RECT& rect) const {
   CPDF_RenderStatus bitmap_render;
-  bitmap_render.Initialize(m_pRenderStatus->m_pContext, pBitmapDevice2, nullptr,
-                           nullptr, nullptr, nullptr, nullptr, 0,
-                           m_pRenderStatus->m_bDropObjects, nullptr, true);
+  bitmap_render.Initialize(m_pRenderStatus->GetContext(), pBitmapDevice2,
+                           nullptr, nullptr, nullptr, nullptr, nullptr, 0,
+                           m_pRenderStatus->GetDropObjects(), nullptr, true);
   CPDF_ImageRenderer image_render;
   if (image_render.Start(&bitmap_render, pDIBSource, 0xffffffff, 255,
                          pNewMatrix, m_Flags, true, FXDIB_BLEND_NORMAL)) {
@@ -295,28 +277,29 @@ bool CPDF_ImageRenderer::DrawPatternImage(const CFX_Matrix* pObj2Device) {
     return false;
 
   CFX_Matrix new_matrix = GetDrawMatrix(rect);
-  CFX_FxgeDevice bitmap_device1;
+  CFX_DefaultRenderDevice bitmap_device1;
   if (!bitmap_device1.Create(rect.Width(), rect.Height(), FXDIB_Rgb32, nullptr))
     return true;
 
   bitmap_device1.GetBitmap()->Clear(0xffffff);
   CPDF_RenderStatus bitmap_render;
-  bitmap_render.Initialize(m_pRenderStatus->m_pContext, &bitmap_device1,
+  bitmap_render.Initialize(m_pRenderStatus->GetContext(), &bitmap_device1,
                            nullptr, nullptr, nullptr, nullptr,
-                           &m_pRenderStatus->m_Options, 0,
-                           m_pRenderStatus->m_bDropObjects, nullptr, true);
+                           m_pRenderStatus->GetRenderOptions(), 0,
+                           m_pRenderStatus->GetDropObjects(), nullptr, true);
   CFX_Matrix patternDevice = *pObj2Device;
-  patternDevice.Translate((float)-rect.left, (float)-rect.top);
+  patternDevice.Translate(static_cast<float>(-rect.left),
+                          static_cast<float>(-rect.top));
   if (CPDF_TilingPattern* pTilingPattern = m_pPattern->AsTilingPattern()) {
-    bitmap_render.DrawTilingPattern(pTilingPattern, m_pImageObject,
+    bitmap_render.DrawTilingPattern(pTilingPattern, m_pImageObject.Get(),
                                     &patternDevice, false);
   } else if (CPDF_ShadingPattern* pShadingPattern =
                  m_pPattern->AsShadingPattern()) {
-    bitmap_render.DrawShadingPattern(pShadingPattern, m_pImageObject,
+    bitmap_render.DrawShadingPattern(pShadingPattern, m_pImageObject.Get(),
                                      &patternDevice, false);
   }
 
-  CFX_FxgeDevice bitmap_device2;
+  CFX_DefaultRenderDevice bitmap_device2;
   if (!bitmap_device2.Create(rect.Width(), rect.Height(), FXDIB_8bppRgb,
                              nullptr)) {
     return true;
@@ -327,7 +310,7 @@ bool CPDF_ImageRenderer::DrawPatternImage(const CFX_Matrix* pObj2Device) {
   bitmap_device2.GetBitmap()->ConvertFormat(FXDIB_8bppMask);
   bitmap_device1.GetBitmap()->MultiplyAlpha(bitmap_device2.GetBitmap());
   bitmap_device1.GetBitmap()->MultiplyAlpha(255);
-  m_pRenderStatus->m_pDevice->SetDIBitsWithBlend(
+  m_pRenderStatus->GetRenderDevice()->SetDIBitsWithBlend(
       bitmap_device1.GetBitmap(), rect.left, rect.top, m_BlendType);
   return false;
 }
@@ -343,7 +326,7 @@ bool CPDF_ImageRenderer::DrawMaskedImage() {
     return false;
 
   CFX_Matrix new_matrix = GetDrawMatrix(rect);
-  CFX_FxgeDevice bitmap_device1;
+  CFX_DefaultRenderDevice bitmap_device1;
   if (!bitmap_device1.Create(rect.Width(), rect.Height(), FXDIB_Rgb32, nullptr))
     return true;
 
@@ -353,15 +336,15 @@ bool CPDF_ImageRenderer::DrawMaskedImage() {
   bitmap_device1.GetBitmap()->Clear(0xffffff);
 #endif
   CPDF_RenderStatus bitmap_render;
-  bitmap_render.Initialize(m_pRenderStatus->m_pContext, &bitmap_device1,
+  bitmap_render.Initialize(m_pRenderStatus->GetContext(), &bitmap_device1,
                            nullptr, nullptr, nullptr, nullptr, nullptr, 0,
-                           m_pRenderStatus->m_bDropObjects, nullptr, true);
+                           m_pRenderStatus->GetDropObjects(), nullptr, true);
   CPDF_ImageRenderer image_render;
   if (image_render.Start(&bitmap_render, m_pDIBSource, 0, 255, &new_matrix,
                          m_Flags, true, FXDIB_BLEND_NORMAL)) {
     image_render.Continue(nullptr);
   }
-  CFX_FxgeDevice bitmap_device2;
+  CFX_DefaultRenderDevice bitmap_device2;
   if (!bitmap_device2.Create(rect.Width(), rect.Height(), FXDIB_8bppRgb,
                              nullptr))
     return true;
@@ -374,7 +357,7 @@ bool CPDF_ImageRenderer::DrawMaskedImage() {
   CalculateDrawImage(&bitmap_device1, &bitmap_device2, m_Loader.m_pMask,
                      &new_matrix, rect);
 #ifdef _SKIA_SUPPORT_
-  m_pRenderStatus->m_pDevice->SetBitsWithMask(
+  m_pRenderStatus->GetRenderDevice()->SetBitsWithMask(
       bitmap_device1.GetBitmap(), bitmap_device2.GetBitmap(), rect.left,
       rect.top, m_BitmapAlpha, m_BlendType);
 #else
@@ -382,7 +365,7 @@ bool CPDF_ImageRenderer::DrawMaskedImage() {
   bitmap_device1.GetBitmap()->MultiplyAlpha(bitmap_device2.GetBitmap());
   if (m_BitmapAlpha < 255)
     bitmap_device1.GetBitmap()->MultiplyAlpha(m_BitmapAlpha);
-  m_pRenderStatus->m_pDevice->SetDIBitsWithBlend(
+  m_pRenderStatus->GetRenderDevice()->SetDIBitsWithBlend(
       bitmap_device1.GetBitmap(), rect.left, rect.top, m_BlendType);
 #endif  //  _SKIA_SUPPORT_
   return false;
@@ -403,10 +386,10 @@ bool CPDF_ImageRenderer::StartDIBSource() {
     }
   }
 #ifdef _SKIA_SUPPORT_
-  CFX_RetainPtr<CFX_DIBitmap> premultiplied = m_pDIBSource->Clone();
+  RetainPtr<CFX_DIBitmap> premultiplied = m_pDIBSource->Clone(nullptr);
   if (m_pDIBSource->HasAlpha())
     CFX_SkiaDeviceDriver::PreMultiply(premultiplied);
-  if (m_pRenderStatus->m_pDevice->StartDIBitsWithBlend(
+  if (m_pRenderStatus->GetRenderDevice()->StartDIBitsWithBlend(
           premultiplied, m_BitmapAlpha, m_FillArgb, &m_ImageMatrix, m_Flags,
           &m_DeviceHandle, m_BlendType)) {
     if (m_DeviceHandle) {
@@ -416,7 +399,7 @@ bool CPDF_ImageRenderer::StartDIBSource() {
     return false;
   }
 #else
-  if (m_pRenderStatus->m_pDevice->StartDIBitsWithBlend(
+  if (m_pRenderStatus->GetRenderDevice()->StartDIBitsWithBlend(
           m_pDIBSource, m_BitmapAlpha, m_FillArgb, &m_ImageMatrix, m_Flags,
           &m_DeviceHandle, m_BlendType)) {
     if (m_DeviceHandle) {
@@ -437,12 +420,11 @@ bool CPDF_ImageRenderer::StartDIBSource() {
       return false;
     }
 
-    FX_RECT clip_box = m_pRenderStatus->m_pDevice->GetClipBox();
+    FX_RECT clip_box = m_pRenderStatus->GetRenderDevice()->GetClipBox();
     clip_box.Intersect(image_rect);
     m_Status = 2;
     m_pTransformer = pdfium::MakeUnique<CFX_ImageTransformer>(
         m_pDIBSource, &m_ImageMatrix, m_Flags, &clip_box);
-    m_pTransformer->Start();
     return true;
   }
   if (m_ImageMatrix.a < 0)
@@ -454,7 +436,7 @@ bool CPDF_ImageRenderer::StartDIBSource() {
   int dest_left = dest_width > 0 ? image_rect.left : image_rect.right;
   int dest_top = dest_height > 0 ? image_rect.top : image_rect.bottom;
   if (m_pDIBSource->IsOpaqueImage() && m_BitmapAlpha == 255) {
-    if (m_pRenderStatus->m_pDevice->StretchDIBitsWithFlagsAndBlend(
+    if (m_pRenderStatus->GetRenderDevice()->StretchDIBitsWithFlagsAndBlend(
             m_pDIBSource, dest_left, dest_top, dest_width, dest_height, m_Flags,
             m_BlendType)) {
       return false;
@@ -463,7 +445,7 @@ bool CPDF_ImageRenderer::StartDIBSource() {
   if (m_pDIBSource->IsAlphaMask()) {
     if (m_BitmapAlpha != 255)
       m_FillArgb = FXARGB_MUL_ALPHA(m_FillArgb, m_BitmapAlpha);
-    if (m_pRenderStatus->m_pDevice->StretchBitMaskWithFlags(
+    if (m_pRenderStatus->GetRenderDevice()->StretchBitMaskWithFlags(
             m_pDIBSource, dest_left, dest_top, dest_width, dest_height,
             m_FillArgb, m_Flags)) {
       return false;
@@ -474,13 +456,13 @@ bool CPDF_ImageRenderer::StartDIBSource() {
     return true;
   }
 
-  FX_RECT clip_box = m_pRenderStatus->m_pDevice->GetClipBox();
+  FX_RECT clip_box = m_pRenderStatus->GetRenderDevice()->GetClipBox();
   FX_RECT dest_rect = clip_box;
   dest_rect.Intersect(image_rect);
   FX_RECT dest_clip(
       dest_rect.left - image_rect.left, dest_rect.top - image_rect.top,
       dest_rect.right - image_rect.left, dest_rect.bottom - image_rect.top);
-  CFX_RetainPtr<CFX_DIBitmap> pStretched =
+  RetainPtr<CFX_DIBitmap> pStretched =
       m_pDIBSource->StretchTo(dest_width, dest_height, m_Flags, &dest_clip);
   if (pStretched) {
     m_pRenderStatus->CompositeDIBitmap(pStretched, dest_rect.left,
@@ -497,11 +479,11 @@ bool CPDF_ImageRenderer::StartBitmapAlpha() {
     path.Transform(&m_ImageMatrix);
     uint32_t fill_color =
         ArgbEncode(0xff, m_BitmapAlpha, m_BitmapAlpha, m_BitmapAlpha);
-    m_pRenderStatus->m_pDevice->DrawPath(&path, nullptr, nullptr, fill_color, 0,
-                                         FXFILL_WINDING);
+    m_pRenderStatus->GetRenderDevice()->DrawPath(&path, nullptr, nullptr,
+                                                 fill_color, 0, FXFILL_WINDING);
     return false;
   }
-  CFX_RetainPtr<CFX_DIBSource> pAlphaMask;
+  RetainPtr<CFX_DIBSource> pAlphaMask;
   if (m_pDIBSource->IsAlphaMask())
     pAlphaMask = m_pDIBSource;
   else
@@ -510,12 +492,12 @@ bool CPDF_ImageRenderer::StartBitmapAlpha() {
   if (fabs(m_ImageMatrix.b) >= 0.5f || fabs(m_ImageMatrix.c) >= 0.5f) {
     int left;
     int top;
-    CFX_RetainPtr<CFX_DIBitmap> pTransformed =
-        pAlphaMask->TransformTo(&m_ImageMatrix, left, top);
+    RetainPtr<CFX_DIBitmap> pTransformed =
+        pAlphaMask->TransformTo(&m_ImageMatrix, &left, &top);
     if (!pTransformed)
       return true;
 
-    m_pRenderStatus->m_pDevice->SetBitMask(
+    m_pRenderStatus->GetRenderDevice()->SetBitMask(
         pTransformed, left, top,
         ArgbEncode(0xff, m_BitmapAlpha, m_BitmapAlpha, m_BitmapAlpha));
     return false;
@@ -528,42 +510,43 @@ bool CPDF_ImageRenderer::StartBitmapAlpha() {
       m_ImageMatrix.d > 0 ? -image_rect.Height() : image_rect.Height();
   int left = dest_width > 0 ? image_rect.left : image_rect.right;
   int top = dest_height > 0 ? image_rect.top : image_rect.bottom;
-  m_pRenderStatus->m_pDevice->StretchBitMask(
+  m_pRenderStatus->GetRenderDevice()->StretchBitMask(
       pAlphaMask, left, top, dest_width, dest_height,
       ArgbEncode(0xff, m_BitmapAlpha, m_BitmapAlpha, m_BitmapAlpha));
   return false;
 }
 
-bool CPDF_ImageRenderer::Continue(IFX_Pause* pPause) {
+bool CPDF_ImageRenderer::Continue(IFX_PauseIndicator* pPause) {
   if (m_Status == 2) {
     if (m_pTransformer->Continue(pPause))
       return true;
 
-    CFX_RetainPtr<CFX_DIBitmap> pBitmap = m_pTransformer->DetachBitmap();
+    RetainPtr<CFX_DIBitmap> pBitmap = m_pTransformer->DetachBitmap();
     if (!pBitmap)
       return false;
 
     if (pBitmap->IsAlphaMask()) {
       if (m_BitmapAlpha != 255)
         m_FillArgb = FXARGB_MUL_ALPHA(m_FillArgb, m_BitmapAlpha);
-      m_Result = m_pRenderStatus->m_pDevice->SetBitMask(
+      m_Result = m_pRenderStatus->GetRenderDevice()->SetBitMask(
           pBitmap, m_pTransformer->result().left, m_pTransformer->result().top,
           m_FillArgb);
     } else {
       if (m_BitmapAlpha != 255)
         pBitmap->MultiplyAlpha(m_BitmapAlpha);
-      m_Result = m_pRenderStatus->m_pDevice->SetDIBitsWithBlend(
+      m_Result = m_pRenderStatus->GetRenderDevice()->SetDIBitsWithBlend(
           pBitmap, m_pTransformer->result().left, m_pTransformer->result().top,
           m_BlendType);
     }
     return false;
   }
-  if (m_Status == 3)
-    return m_pRenderStatus->m_pDevice->ContinueDIBits(m_DeviceHandle.get(),
-                                                      pPause);
+  if (m_Status == 3) {
+    return m_pRenderStatus->GetRenderDevice()->ContinueDIBits(
+        m_DeviceHandle.get(), pPause);
+  }
 
   if (m_Status == 4) {
-    if (m_Loader.Continue(pPause))
+    if (m_Loader.Continue(pPause, m_pRenderStatus.Get()))
       return true;
 
     if (StartRenderDIBSource())
@@ -571,3 +554,30 @@ bool CPDF_ImageRenderer::Continue(IFX_Pause* pPause) {
   }
   return false;
 }
+
+void CPDF_ImageRenderer::HandleFilters() {
+  CPDF_Object* pFilters =
+      m_pImageObject->GetImage()->GetStream()->GetDict()->GetDirectObjectFor(
+          "Filter");
+  if (!pFilters)
+    return;
+
+  if (pFilters->IsName()) {
+    ByteString bsDecodeType = pFilters->GetString();
+    if (bsDecodeType == "DCTDecode" || bsDecodeType == "JPXDecode")
+      m_Flags |= FXRENDER_IMAGE_LOSSY;
+    return;
+  }
+
+  CPDF_Array* pArray = pFilters->AsArray();
+  if (!pArray)
+    return;
+
+  for (size_t i = 0; i < pArray->GetCount(); i++) {
+    ByteString bsDecodeType = pArray->GetStringAt(i);
+    if (bsDecodeType == "DCTDecode" || bsDecodeType == "JPXDecode") {
+      m_Flags |= FXRENDER_IMAGE_LOSSY;
+      break;
+    }
+  }
+}
