@@ -9,7 +9,6 @@
 
 #include <memory>
 
-#include "core/fxcrt/fx_basic.h"
 #include "fxbarcode/utils.h"
 
 class CBC_ReedSolomonGF256Poly;
@@ -17,22 +16,22 @@ class CBC_ReedSolomonGF256Poly;
 class CBC_ReedSolomonGF256 {
  public:
   explicit CBC_ReedSolomonGF256(int32_t primitive);
-  virtual ~CBC_ReedSolomonGF256();
+  ~CBC_ReedSolomonGF256();
 
   static void Initialize();
   static void Finalize();
 
   CBC_ReedSolomonGF256Poly* GetZero() const;
   CBC_ReedSolomonGF256Poly* GetOne() const;
-  CBC_ReedSolomonGF256Poly* BuildMonomial(int32_t degree,
-                                          int32_t coefficient,
-                                          int32_t& e);
+  std::unique_ptr<CBC_ReedSolomonGF256Poly> BuildMonomial(int32_t degree,
+                                                          int32_t coefficient,
+                                                          int32_t& e);
   static int32_t AddOrSubtract(int32_t a, int32_t b);
   int32_t Exp(int32_t a);
   int32_t Log(int32_t a, int32_t& e);
   int32_t Inverse(int32_t a, int32_t& e);
   int32_t Multiply(int32_t a, int32_t b);
-  virtual void Init();
+  void Init();
 
   static CBC_ReedSolomonGF256* QRCodeField;
   static CBC_ReedSolomonGF256* DataMatrixField;
