@@ -18,18 +18,14 @@ class CBC_DataMatrixWriter : public CBC_TwoDimWriter {
   CBC_DataMatrixWriter();
   ~CBC_DataMatrixWriter() override;
 
-  virtual uint8_t* Encode(const CFX_WideString& contents,
-                          int32_t& outWidth,
-                          int32_t& outHeight,
-                          int32_t& e);
+  uint8_t* Encode(const WideString& contents,
+                  int32_t& outWidth,
+                  int32_t& outHeight);
 
   // CBC_TwoDimWriter
   bool SetErrorCorrectionLevel(int32_t level) override;
 
  private:
-  static CBC_CommonByteMatrix* encodeLowLevel(CBC_DefaultPlacement* placement,
-                                              CBC_SymbolInfo* symbolInfo,
-                                              int32_t& e);
   int32_t m_iCorrectLevel;
 };
 
