@@ -41,9 +41,7 @@
 #include "fxbarcode/qrcode/BC_QRCoderVersion.h"
 #include "third_party/base/ptr_util.h"
 
-using ModeStringPair = std::pair<CBC_QRCoderMode*, CFX_ByteString>;
-
-const int32_t kMaxVersion = 40;
+using ModeStringPair = std::pair<CBC_QRCoderMode*, ByteString>;
 
 namespace {
 
@@ -63,7 +61,7 @@ int32_t GetAlphaNumericCode(int32_t code) {
   return g_alphaNumericTable[code_index];
 }
 
-void AppendNumericBytes(const CFX_ByteString& content,
+void AppendNumericBytes(const ByteString& content,
                         CBC_QRCoderBitVector* bits,
                         int32_t& e) {
   int32_t length = content.GetLength();
@@ -86,7 +84,7 @@ void AppendNumericBytes(const CFX_ByteString& content,
   }
 }
 
-void AppendAlphaNumericBytes(const CFX_ByteString& content,
+void AppendAlphaNumericBytes(const ByteString& content,
                              CBC_QRCoderBitVector* bits,
                              int32_t& e) {
   int32_t length = content.GetLength();
@@ -112,7 +110,7 @@ void AppendAlphaNumericBytes(const CFX_ByteString& content,
   }
 }
 
-void AppendGBKBytes(const CFX_ByteString& content,
+void AppendGBKBytes(const ByteString& content,
                     CBC_QRCoderBitVector* bits,
                     int32_t& e) {
   int32_t length = content.GetLength();
@@ -132,15 +130,15 @@ void AppendGBKBytes(const CFX_ByteString& content,
   }
 }
 
-void Append8BitBytes(const CFX_ByteString& content,
+void Append8BitBytes(const ByteString& content,
                      CBC_QRCoderBitVector* bits,
-                     CFX_ByteString encoding,
+                     ByteString encoding,
                      int32_t& e) {
-  for (int32_t i = 0; i < content.GetLength(); i++)
+  for (size_t i = 0; i < content.GetLength(); i++)
     bits->AppendBits(content[i], 8);
 }
 
-void AppendKanjiBytes(const CFX_ByteString& content,
+void AppendKanjiBytes(const ByteString& content,
                       CBC_QRCoderBitVector* bits,
                       int32_t& e) {
   std::vector<uint8_t> bytes;
@@ -171,10 +169,10 @@ bool AppendLengthInfo(int32_t numLetters,
                       CBC_QRCoderMode* mode,
                       CBC_QRCoderBitVector* bits) {
   int32_t e = BCExceptionNO;
-  CBC_QRCoderVersion* qcv = CBC_QRCoderVersion::GetVersionForNumber(version, e);
-  if (e != BCExceptionNO)
+  const auto* qcv = CBC_QRCoderVersion::GetVersionForNumber(version);
+  if (!qcv)
     return false;
-  int32_t numBits = mode->GetCharacterCountBits(qcv, e);
+  int32_t numBits = mode->GetCharacterCountBits(qcv->GetVersionNumber(), e);
   if (e != BCExceptionNO)
     return false;
   if (numBits > ((1 << numBits) - 1))
@@ -186,10 +184,10 @@ bool AppendLengthInfo(int32_t numLetters,
   return true;
 }
 
-void AppendBytes(const CFX_ByteString& content,
+void AppendBytes(const ByteString& content,
                  CBC_QRCoderMode* mode,
                  CBC_QRCoderBitVector* bits,
-                 CFX_ByteString encoding,
+                 ByteString encoding,
                  int32_t& e) {
   if (mode == CBC_QRCoderMode::sNUMERIC)
     AppendNumericBytes(content, bits, e);
@@ -205,48 +203,42 @@ void AppendBytes(const CFX_ByteString& content,
     e = BCExceptionUnsupportedMode;
 }
 
-void InitQRCode(int32_t numInputBytes,
-                CBC_QRCoderErrorCorrectionLevel* ecLevel,
+bool InitQRCode(int32_t numInputBytes,
+                const CBC_QRCoderErrorCorrectionLevel* ecLevel,
                 CBC_QRCoderMode* mode,
-                CBC_QRCoder* qrCode,
-                int32_t& e) {
+                CBC_QRCoder* qrCode) {
   qrCode->SetECLevel(ecLevel);
   qrCode->SetMode(mode);
-  for (int32_t versionNum = 1; versionNum <= kMaxVersion; versionNum++) {
-    CBC_QRCoderVersion* version =
-        CBC_QRCoderVersion::GetVersionForNumber(versionNum, e);
-    if (e != BCExceptionNO)
-      return;
+  for (int32_t i = 1; i <= CBC_QRCoderVersion::kMaxVersion; ++i) {
+    const auto* version = CBC_QRCoderVersion::GetVersionForNumber(i);
     int32_t numBytes = version->GetTotalCodeWords();
-    CBC_QRCoderECBlocks* ecBlocks = version->GetECBlocksForLevel(ecLevel);
+    const auto* ecBlocks = version->GetECBlocksForLevel(*ecLevel);
     int32_t numEcBytes = ecBlocks->GetTotalECCodeWords();
     int32_t numRSBlocks = ecBlocks->GetNumBlocks();
     int32_t numDataBytes = numBytes - numEcBytes;
     if (numDataBytes >= numInputBytes + 3) {
-      qrCode->SetVersion(versionNum);
+      qrCode->SetVersion(i);
       qrCode->SetNumTotalBytes(numBytes);
       qrCode->SetNumDataBytes(numDataBytes);
       qrCode->SetNumRSBlocks(numRSBlocks);
       qrCode->SetNumECBytes(numEcBytes);
       qrCode->SetMatrixWidth(version->GetDimensionForVersion());
-      return;
+      return true;
     }
   }
-  e = BCExceptionCannotFindBlockInfo;
+  return false;
 }
 
 std::unique_ptr<CBC_CommonByteArray> GenerateECBytes(
     CBC_CommonByteArray* dataBytes,
-    int32_t numEcBytesInBlock,
-    int32_t& e) {
+    int32_t numEcBytesInBlock) {
   int32_t numDataBytes = dataBytes->Size();
   std::vector<int32_t> toEncode(numDataBytes + numEcBytesInBlock);
   for (int32_t i = 0; i < numDataBytes; ++i)
     toEncode[i] = dataBytes->At(i);
   CBC_ReedSolomonEncoder encode(CBC_ReedSolomonGF256::QRCodeField);
   encode.Init();
-  encode.Encode(&toEncode, numEcBytesInBlock, e);
-  if (e != BCExceptionNO)
+  if (!encode.Encode(&toEncode, numEcBytesInBlock))
     return nullptr;
   auto ecBytes = pdfium::MakeUnique<CBC_CommonByteArray>(numEcBytesInBlock);
   for (int32_t i = 0; i < numEcBytesInBlock; ++i)
@@ -267,7 +259,7 @@ int32_t GetSpanByVersion(CBC_QRCoderMode* modeFirst,
       return 11;
     if (versionNum >= 10 && versionNum <= 26)
       return 15;
-    if (versionNum >= 27 && versionNum <= kMaxVersion)
+    if (versionNum >= 27 && versionNum <= CBC_QRCoderVersion::kMaxVersion)
       return 16;
     e = BCExceptionNoSuchVersion;
     return 0;
@@ -278,7 +270,7 @@ int32_t GetSpanByVersion(CBC_QRCoderMode* modeFirst,
       return 13;
     if (versionNum >= 10 && versionNum <= 26)
       return 15;
-    if (versionNum >= 27 && versionNum <= kMaxVersion)
+    if (versionNum >= 27 && versionNum <= CBC_QRCoderVersion::kMaxVersion)
       return 17;
     e = BCExceptionNoSuchVersion;
     return 0;
@@ -289,7 +281,7 @@ int32_t GetSpanByVersion(CBC_QRCoderMode* modeFirst,
       return 6;
     if (versionNum >= 10 && versionNum <= 26)
       return 8;
-    if (versionNum >= 27 && versionNum <= kMaxVersion)
+    if (versionNum >= 27 && versionNum <= CBC_QRCoderVersion::kMaxVersion)
       return 9;
     e = BCExceptionNoSuchVersion;
     return 0;
@@ -305,7 +297,7 @@ int32_t CalculateMaskPenalty(CBC_CommonByteMatrix* matrix) {
 }
 
 int32_t ChooseMaskPattern(CBC_QRCoderBitVector* bits,
-                          CBC_QRCoderErrorCorrectionLevel* ecLevel,
+                          const CBC_QRCoderErrorCorrectionLevel* ecLevel,
                           int32_t version,
                           CBC_CommonByteMatrix* matrix,
                           int32_t& e) {
@@ -352,14 +344,10 @@ void GetNumDataBytesAndNumECBytesForBlockID(int32_t numTotalBytes,
   }
 }
 
-void TerminateBits(int32_t numDataBytes,
-                   CBC_QRCoderBitVector* bits,
-                   int32_t& e) {
+bool TerminateBits(int32_t numDataBytes, CBC_QRCoderBitVector* bits) {
   size_t capacity = numDataBytes << 3;
-  if (bits->Size() > capacity) {
-    e = BCExceptionDataTooMany;
-    return;
-  }
+  if (bits->Size() > capacity)
+    return false;
 
   for (int32_t i = 0; i < 4 && bits->Size() < capacity; ++i)
     bits->AppendBit(0);
@@ -371,15 +359,13 @@ void TerminateBits(int32_t numDataBytes,
       bits->AppendBit(0);
   }
 
-  if (bits->Size() % 8 != 0) {
-    e = BCExceptionDigitLengthMustBe8;
-    return;
-  }
+  if (bits->Size() % 8 != 0)
+    return false;
+
   int32_t numPaddingBytes = numDataBytes - bits->sizeInBytes();
   for (int32_t k = 0; k < numPaddingBytes; ++k)
     bits->AppendBits(k % 2 ? 0x11 : 0xec, 8);
-  if (bits->Size() != capacity)
-    e = BCExceptionBitsNotEqualCacity;
+  return bits->Size() == capacity;
 }
 
 void MergeString(std::vector<ModeStringPair>* result,
@@ -394,8 +380,8 @@ void MergeString(std::vector<ModeStringPair>* result,
                                      CBC_QRCoderMode::sBYTE, versionNum, e);
       if (e != BCExceptionNO)
         return;
-      if (element2.first == CBC_QRCoderMode::sBYTE &&
-          element1.second.GetLength() < tmp) {
+      if (element2.first == CBC_QRCoderMode::sBYTE && tmp >= 0 &&
+          element1.second.GetLength() < static_cast<size_t>(tmp)) {
         element2.second = element1.second + element2.second;
         result->erase(result->begin() + i);
         i--;
@@ -413,8 +399,8 @@ void MergeString(std::vector<ModeStringPair>* result,
                                      CBC_QRCoderMode::sBYTE, versionNum, e);
       if (e != BCExceptionNO)
         return;
-      if (element2.first == CBC_QRCoderMode::sBYTE &&
-          element1.second.GetLength() < tmp) {
+      if (element2.first == CBC_QRCoderMode::sBYTE && tmp >= 0 &&
+          element1.second.GetLength() < static_cast<size_t>(tmp)) {
         element2.second = element1.second + element2.second;
         result->erase(result->begin() + i);
         i--;
@@ -424,8 +410,8 @@ void MergeString(std::vector<ModeStringPair>* result,
                              CBC_QRCoderMode::sALPHANUMERIC, versionNum, e);
       if (e != BCExceptionNO)
         return;
-      if (element2.first == CBC_QRCoderMode::sALPHANUMERIC &&
-          element1.second.GetLength() < tmp) {
+      if (element2.first == CBC_QRCoderMode::sALPHANUMERIC && tmp >= 0 &&
+          element1.second.GetLength() < static_cast<size_t>(tmp)) {
         element2.second = element1.second + element2.second;
         result->erase(result->begin() + i);
         i--;
@@ -437,25 +423,27 @@ void MergeString(std::vector<ModeStringPair>* result,
     MergeString(result, versionNum, e);
 }
 
-void SplitString(const CFX_ByteString& content,
+void SplitString(const ByteString& content,
                  std::vector<ModeStringPair>* result) {
-  int32_t index = 0;
-  while (index < content.GetLength() &&
-         ((content[index] >= 0xA1 && content[index] <= 0xAA) ||
-          (content[index] >= 0xB0 && content[index] <= 0xFA))) {
+  size_t index = 0;
+  while (index < content.GetLength()) {
+    uint8_t c = static_cast<uint8_t>(content[index]);
+    if (!((c >= 0xA1 && c <= 0xAA) || (c >= 0xB0 && c <= 0xFA)))
+      break;
     index += 2;
   }
   if (index)
-    result->push_back({CBC_QRCoderMode::sGBK, content.Mid(0, index)});
+    result->push_back({CBC_QRCoderMode::sGBK, content.Left(index)});
   if (index >= content.GetLength())
     return;
 
-  int32_t flag = index;
+  size_t flag = index;
   while (GetAlphaNumericCode(content[index]) == -1 &&
-         index < content.GetLength() &&
-         !((content[index] >= 0xA1 && content[index] <= 0xAA) ||
-           (content[index] >= 0xB0 && content[index] <= 0xFA))) {
-#if _FXM_PLATFORM_ == _FXM_PLATFORM_WINDOWS_
+         index < content.GetLength()) {
+    uint8_t c = static_cast<uint8_t>(content[index]);
+    if (((c >= 0xA1 && c <= 0xAA) || (c >= 0xB0 && c <= 0xFA)))
+      break;
+#if _FX_PLATFORM_ == _FX_PLATFORM_WINDOWS_
     bool high = !!IsDBCSLeadByte(content[index]);
 #else
     bool high = content[index] > 127;
@@ -493,17 +481,16 @@ void SplitString(const CFX_ByteString& content,
   }
   flag = index;
   if (index < content.GetLength())
-    SplitString(content.Mid(index, content.GetLength() - index), result);
+    SplitString(content.Right(content.GetLength() - index), result);
 }
 
-CBC_QRCoderMode* ChooseMode(const CFX_ByteString& content,
-                            CFX_ByteString encoding) {
+CBC_QRCoderMode* ChooseMode(const ByteString& content, ByteString encoding) {
   if (encoding.Compare("SHIFT_JIS") == 0)
     return CBC_QRCoderMode::sKANJI;
 
   bool hasNumeric = false;
   bool hasAlphaNumeric = false;
-  for (int32_t i = 0; i < content.GetLength(); i++) {
+  for (size_t i = 0; i < content.GetLength(); i++) {
     if (isdigit(content[i])) {
       hasNumeric = true;
     } else if (GetAlphaNumericCode(content[i]) != -1) {
@@ -519,18 +506,16 @@ CBC_QRCoderMode* ChooseMode(const CFX_ByteString& content,
   return CBC_QRCoderMode::sBYTE;
 }
 
-void InterleaveWithECBytes(CBC_QRCoderBitVector* bits,
+bool InterleaveWithECBytes(CBC_QRCoderBitVector* bits,
                            int32_t numTotalBytes,
                            int32_t numDataBytes,
                            int32_t numRSBlocks,
-                           CBC_QRCoderBitVector* result,
-                           int32_t& e) {
+                           CBC_QRCoderBitVector* result) {
   ASSERT(numTotalBytes >= 0);
   ASSERT(numDataBytes >= 0);
-  if (bits->sizeInBytes() != static_cast<size_t>(numDataBytes)) {
-    e = BCExceptionBitsBytesNotMatch;
-    return;
-  }
+  if (bits->sizeInBytes() != static_cast<size_t>(numDataBytes))
+    return false;
+
   int32_t dataBytesOffset = 0;
   int32_t maxNumDataBytes = 0;
   int32_t maxNumEcBytes = 0;
@@ -543,19 +528,19 @@ void InterleaveWithECBytes(CBC_QRCoderBitVector* bits,
                                            numEcBytesInBlosk);
     auto dataBytes = pdfium::MakeUnique<CBC_CommonByteArray>();
     dataBytes->Set(bits->GetArray(), dataBytesOffset, numDataBytesInBlock);
-    std::unique_ptr<CBC_CommonByteArray> ecBytes(
-        GenerateECBytes(dataBytes.get(), numEcBytesInBlosk, e));
-    if (e != BCExceptionNO)
-      return;
+    std::unique_ptr<CBC_CommonByteArray> ecBytes =
+        GenerateECBytes(dataBytes.get(), numEcBytesInBlosk);
+    if (!ecBytes)
+      return false;
+
     maxNumDataBytes = std::max(maxNumDataBytes, dataBytes->Size());
     maxNumEcBytes = std::max(maxNumEcBytes, ecBytes->Size());
     blocks[i].SetData(std::move(dataBytes), std::move(ecBytes));
     dataBytesOffset += numDataBytesInBlock;
   }
-  if (numDataBytes != dataBytesOffset) {
-    e = BCExceptionBytesNotMatchOffset;
-    return;
-  }
+  if (numDataBytes != dataBytesOffset)
+    return false;
+
   for (int32_t x = 0; x < maxNumDataBytes; x++) {
     for (size_t j = 0; j < blocks.size(); j++) {
       const CBC_CommonByteArray* dataBytes = blocks[j].GetDataBytes();
@@ -570,8 +555,7 @@ void InterleaveWithECBytes(CBC_QRCoderBitVector* bits,
         result->AppendBits(ecBytes->At(y), 8);
     }
   }
-  if (static_cast<size_t>(numTotalBytes) != result->sizeInBytes())
-    e = BCExceptionSizeInBytesDiffer;
+  return static_cast<size_t>(numTotalBytes) == result->sizeInBytes();
 }
 
 }  // namespace
@@ -580,41 +564,39 @@ CBC_QRCoderEncoder::CBC_QRCoderEncoder() {}
 
 CBC_QRCoderEncoder::~CBC_QRCoderEncoder() {}
 
-void CBC_QRCoderEncoder::Encode(const CFX_WideString& content,
-                                CBC_QRCoderErrorCorrectionLevel* ecLevel,
-                                CBC_QRCoder* qrCode,
-                                int32_t& e) {
-  CFX_ByteString encoding = "utf8";
-  CFX_ByteString utf8Data;
+// static
+bool CBC_QRCoderEncoder::Encode(const WideString& content,
+                                const CBC_QRCoderErrorCorrectionLevel* ecLevel,
+                                CBC_QRCoder* qrCode) {
+  ByteString encoding = "utf8";
+  ByteString utf8Data;
   CBC_UtilCodingConvert::UnicodeToUTF8(content, utf8Data);
   CBC_QRCoderMode* mode = ChooseMode(utf8Data, encoding);
   CBC_QRCoderBitVector dataBits;
+  int32_t e = BCExceptionNO;
   AppendBytes(utf8Data, mode, &dataBits, encoding, e);
   if (e != BCExceptionNO)
-    return;
+    return false;
   int32_t numInputBytes = dataBits.sizeInBytes();
-  InitQRCode(numInputBytes, ecLevel, mode, qrCode, e);
-  if (e != BCExceptionNO)
-    return;
+  if (!InitQRCode(numInputBytes, ecLevel, mode, qrCode))
+    return false;
   CBC_QRCoderBitVector headerAndDataBits;
   AppendModeInfo(mode, &headerAndDataBits);
   int32_t numLetters = mode == CBC_QRCoderMode::sBYTE ? dataBits.sizeInBytes()
                                                       : content.GetLength();
   if (!AppendLengthInfo(numLetters, qrCode->GetVersion(), mode,
                         &headerAndDataBits)) {
-    e = BCExceptionGeneric;
-    return;
+    return false;
   }
   headerAndDataBits.AppendBitVector(&dataBits);
-  TerminateBits(qrCode->GetNumDataBytes(), &headerAndDataBits, e);
-  if (e != BCExceptionNO)
-    return;
+  if (!TerminateBits(qrCode->GetNumDataBytes(), &headerAndDataBits))
+    return false;
   CBC_QRCoderBitVector finalBits;
-  InterleaveWithECBytes(&headerAndDataBits, qrCode->GetNumTotalBytes(),
-                        qrCode->GetNumDataBytes(), qrCode->GetNumRSBlocks(),
-                        &finalBits, e);
-  if (e != BCExceptionNO)
-    return;
+  if (!InterleaveWithECBytes(&headerAndDataBits, qrCode->GetNumTotalBytes(),
+                             qrCode->GetNumDataBytes(),
+                             qrCode->GetNumRSBlocks(), &finalBits)) {
+    return false;
+  }
 
   auto matrix = pdfium::MakeUnique<CBC_CommonByteMatrix>(
       qrCode->GetMatrixWidth(), qrCode->GetMatrixWidth());
@@ -622,16 +604,15 @@ void CBC_QRCoderEncoder::Encode(const CFX_WideString& content,
   int32_t maskPattern = ChooseMaskPattern(
       &finalBits, qrCode->GetECLevel(), qrCode->GetVersion(), matrix.get(), e);
   if (e != BCExceptionNO)
-    return;
+    return false;
 
   qrCode->SetMaskPattern(maskPattern);
   CBC_QRCoderMatrixUtil::BuildMatrix(&finalBits, qrCode->GetECLevel(),
                                      qrCode->GetVersion(),
                                      qrCode->GetMaskPattern(), matrix.get(), e);
   if (e != BCExceptionNO)
-    return;
+    return false;
 
   qrCode->SetMatrix(std::move(matrix));
-  if (!qrCode->IsValid())
-    e = BCExceptionInvalidQRCode;
+  return qrCode->IsValid();
 }
