@@ -6,8 +6,10 @@
 //
 // Copyright 2013, 2014 Igalia S.L.
 // Copyright 2014 Luigi Scarso <luigi.scarso@gmail.com>
-// Copyright 2014, 2017 Albert Astals Cid <aacid@kde.org>
+// Copyright 2014, 2017, 2018 Albert Astals Cid <aacid@kde.org>
 // Copyright 2015 Dmytro Morgun <lztoad@gmail.com>
+// Copyright 2018 Adrian Johnson <ajohnson@redneon.com>
+// Copyright 2018 Klarälvdalens Datakonsult AB, a KDAB Group company, <info@kdab.com>. Work sponsored by the LiMux project of the city of Munich
 //
 //========================================================================
 
@@ -226,13 +228,13 @@ static GBool isTextString(Object *value)
       return okay;                                                      \
     }
 
-ARRAY_CHECKER(isRGBColorOrOptionalArray4, isRGBColor,        4, gTrue,  gTrue );
-ARRAY_CHECKER(isPositiveOrOptionalArray4, isPositive,        4, gTrue,  gTrue );
-ARRAY_CHECKER(isPositiveOrArray4,         isPositive,        4, gTrue,  gFalse);
-ARRAY_CHECKER(isBorderStyle,              isBorderStyleName, 4, gTrue,  gTrue );
-ARRAY_CHECKER(isNumberArray4,             isNumber,          4, gFalse, gFalse);
-ARRAY_CHECKER(isNumberOrArrayN,           isNumber,          0, gTrue,  gFalse);
-ARRAY_CHECKER(isTableHeaders,             isTextString,      0, gFalse, gFalse);
+ARRAY_CHECKER(isRGBColorOrOptionalArray4, isRGBColor,        4, gTrue,  gTrue )
+ARRAY_CHECKER(isPositiveOrOptionalArray4, isPositive,        4, gTrue,  gTrue )
+ARRAY_CHECKER(isPositiveOrArray4,         isPositive,        4, gTrue,  gFalse)
+ARRAY_CHECKER(isBorderStyle,              isBorderStyleName, 4, gTrue,  gTrue )
+ARRAY_CHECKER(isNumberArray4,             isNumber,          4, gFalse, gFalse)
+ARRAY_CHECKER(isNumberOrArrayN,           isNumber,          0, gTrue,  gFalse)
+ARRAY_CHECKER(isTableHeaders,             isTextString,      0, gFalse, gFalse)
 
 
 // Type of functions used to do type-checking on attribute values
@@ -248,6 +250,8 @@ struct AttributeMapEntry {
 };
 
 struct AttributeDefaults {
+  AttributeDefaults() {}; // needed to support old clang
+
   Object Inline  = Object(objName, "Inline");
   Object LrTb = Object(objName, "LrTb");
   Object Normal = Object(objName, "Normal");
@@ -265,7 +269,7 @@ static const AttributeDefaults attributeDefaults;
 
 
 #define ATTR_LIST_END \
-  { Attribute::Unknown, NULL, NULL, gFalse, NULL }
+  { Attribute::Unknown, nullptr, nullptr, gFalse, nullptr }
 
 #define ATTR_WITH_DEFAULT(name, inherit, check, defval) \
   { Attribute::name,           \
@@ -277,7 +281,7 @@ static const AttributeDefaults attributeDefaults;
 #define ATTR(name, inherit, check) \
   { Attribute::name,           \
     #name,                     \
-    NULL,                      \
+    nullptr,                   \
     inherit,                   \
     check }
 
@@ -380,24 +384,24 @@ static const AttributeMapEntry *attributeMapAll[] = {
   attributeMapCommonPrintField,
   attributeMapCommonTable,
   attributeMapCommonTableCell,
-  NULL,
+  nullptr,
 };
 
 static const AttributeMapEntry *attributeMapShared[] = {
   attributeMapCommonShared,
-  NULL,
+  nullptr,
 };
 
 static const AttributeMapEntry *attributeMapBlock[] = {
   attributeMapCommonShared,
   attributeMapCommonBlock,
-  NULL,
+  nullptr,
 };
 
 static const AttributeMapEntry *attributeMapInline[] = {
   attributeMapCommonShared,
   attributeMapCommonInline,
-  NULL,
+  nullptr,
 };
 
 static const AttributeMapEntry *attributeMapTableCell[] = {
@@ -405,34 +409,34 @@ static const AttributeMapEntry *attributeMapTableCell[] = {
   attributeMapCommonBlock,
   attributeMapCommonTable,
   attributeMapCommonTableCell,
-  NULL,
+  nullptr,
 };
 
 static const AttributeMapEntry *attributeMapRubyText[] = {
   attributeMapCommonShared,
   attributeMapCommonInline,
   attributeMapCommonRubyText,
-  NULL,
+  nullptr,
 };
 
 static const AttributeMapEntry *attributeMapColumns[] = {
   attributeMapCommonShared,
   attributeMapCommonInline,
   attributeMapCommonColumns,
-  NULL,
+  nullptr,
 };
 
 static const AttributeMapEntry *attributeMapList[] = {
   attributeMapCommonShared,
   attributeMapCommonList,
-  NULL,
+  nullptr,
 };
 
 static const AttributeMapEntry *attributeMapTable[] = {
   attributeMapCommonShared,
   attributeMapCommonBlock,
   attributeMapCommonTable,
-  NULL,
+  nullptr,
 };
 
 static const AttributeMapEntry *attributeMapIllustration[] = {
@@ -442,7 +446,7 @@ static const AttributeMapEntry *attributeMapIllustration[] = {
   attributeMapCommonShared,
   attributeMapCommonBlock,
   attributeMapCommonInline,
-  NULL,
+  nullptr,
 };
 
 // Table mapping owners of attributes to their names.
@@ -569,7 +573,7 @@ getAttributeMapEntry(const AttributeMapEntry **entryList, Attribute::Type type)
     }
     entryList++;
   }
-  return NULL;
+  return nullptr;
 }
 
 static inline const AttributeMapEntry *
@@ -586,7 +590,7 @@ getAttributeMapEntry(const AttributeMapEntry **entryList, const char *name)
     }
     entryList++;
   }
-  return NULL;
+  return nullptr;
 }
 
 static inline const OwnerMapEntry *getOwnerMapEntry(Attribute::Owner owner)
@@ -595,7 +599,7 @@ static inline const OwnerMapEntry *getOwnerMapEntry(Attribute::Owner owner)
     if (owner == ownerMap[i].owner)
       return &ownerMap[i];
   }
-  return NULL;
+  return nullptr;
 }
 
 static inline const OwnerMapEntry *getOwnerMapEntry(const char *name)
@@ -604,7 +608,7 @@ static inline const OwnerMapEntry *getOwnerMapEntry(const char *name)
     if (strcmp(name, ownerMap[i].name) == 0)
       return &ownerMap[i];
   }
-  return NULL;
+  return nullptr;
 }
 
 static const char *ownerToName(Attribute::Owner owner)
@@ -625,7 +629,7 @@ static inline const TypeMapEntry *getTypeMapEntry(StructElement::Type type)
     if (type == typeMap[i].type)
       return &typeMap[i];
   }
-  return NULL;
+  return nullptr;
 }
 
 static inline const TypeMapEntry *getTypeMapEntry(const char *name)
@@ -634,7 +638,7 @@ static inline const TypeMapEntry *getTypeMapEntry(const char *name)
     if (strcmp(name, typeMap[i].name) == 0)
       return &typeMap[i];
   }
-  return NULL;
+  return nullptr;
 }
 
 static const char *typeToName(StructElement::Type type)
@@ -666,20 +670,20 @@ Attribute::Attribute(const char *nameA, int nameLenA, Object *valueA):
   name(nameA, nameLenA),
   value(),
   hidden(gFalse),
-  formatted(NULL)
+  formatted(nullptr)
 {
   assert(valueA);
   value = valueA->copy();
 }
 
-Attribute::Attribute(Type type, Object *valueA):
-  type(type),
+Attribute::Attribute(Type typeA, Object *valueA):
+  type(typeA),
   owner(UserProperties), // TODO: Determine corresponding owner from Type
   revision(0),
   name(),
   value(),
   hidden(gFalse),
-  formatted(NULL)
+  formatted(nullptr)
 {
   assert(valueA);
 
@@ -714,7 +718,7 @@ const char *Attribute::getOwnerName() const
 Object *Attribute::getDefaultValue(Attribute::Type type)
 {
   const AttributeMapEntry *entry = getAttributeMapEntry(attributeMapAll, type);
-  return entry ? const_cast<Object*>(entry->defval) : NULL;
+  return entry ? const_cast<Object*>(entry->defval) : nullptr;
 }
 
 void Attribute::setFormattedValue(const char *formattedA)
@@ -726,7 +730,7 @@ void Attribute::setFormattedValue(const char *formattedA)
       formatted = new GooString(formattedA);
   } else {
     delete formatted;
-    formatted = NULL;
+    formatted = nullptr;
   }
 }
 
@@ -774,7 +778,7 @@ Attribute *Attribute::parseUserProperty(Dict *property)
 
   obj = property->lookup("N");
   if (obj.isString()) {
-    GooString *s = obj.getString();
+    const GooString *s = obj.getString();
     name = s->getCString();
     nameLen = s->getLength();
   } else if (obj.isName())
@@ -814,12 +818,12 @@ Attribute *Attribute::parseUserProperty(Dict *property)
 //------------------------------------------------------------------------
 
 StructElement::StructData::StructData():
-  altText(0),
-  actualText(0),
-  id(0),
-  title(0),
-  expandedAbbr(0),
-  language(0),
+  altText(nullptr),
+  actualText(nullptr),
+  id(nullptr),
+  title(nullptr),
+  expandedAbbr(nullptr),
+  language(nullptr),
   revision(0)
 {
 }
@@ -928,9 +932,9 @@ const Attribute *StructElement::findAttribute(Attribute::Type attributeType, GBo
     return parent->findAttribute(attributeType, inherit, attributeOwner);
 
   if (attributeType == Attribute::Unknown || attributeType == Attribute::UserProperty)
-    return NULL;
+    return nullptr;
 
-  const Attribute *result = NULL;
+  const Attribute *result = nullptr;
 
   if (attributeOwner == Attribute::UnknownOwner) {
     // Search for the attribute, no matter who the owner is
@@ -965,7 +969,7 @@ const Attribute *StructElement::findAttribute(Attribute::Type attributeType, GBo
       return parent->findAttribute(attributeType, inherit, attributeOwner);
   }
 
-  return NULL;
+  return nullptr;
 }
 
 GooString* StructElement::appendSubTreeText(GooString *string, GBool recursive) const
@@ -984,7 +988,7 @@ GooString* StructElement::appendSubTreeText(GooString *string, GBool recursive)
   }
 
   if (!recursive)
-    return NULL;
+    return nullptr;
 
   // Do a depth-first traversal, to get elements in logical order
   if (!string)
@@ -1063,7 +1067,7 @@ void StructElement::parse(Dict *element)
 
   // Type name may not be standard, resolve through RoleMap first.
   if (treeRoot->getRoleMap()) {
-    type = roleMapResolve(treeRoot->getRoleMap(), obj.getName(), NULL);
+    type = roleMapResolve(treeRoot->getRoleMap(), obj.getName(), nullptr);
   }
 
   // Resolving through RoleMap may leave type as Unknown, e.g. for types
@@ -1186,7 +1190,7 @@ StructElement *StructElement::parseChild(Object *ref,
   assert(childObj);
   assert(ref);
 
-  StructElement *child = NULL;
+  StructElement *child = nullptr;
 
   if (childObj->isInt()) {
     child = new StructElement(childObj->getInt(), treeRoot, this);
@@ -1196,9 +1200,9 @@ StructElement *StructElement::parseChild(Object *ref,
      *      page will be always scanned when calling StructElement::getText().
      */
     Object mcidObj = childObj->dictLookup("MCID");
-    if (mcidObj.isInt()) {
+    if (!mcidObj.isInt()) {
       error(errSyntaxError, -1, "MCID object is wrong type ({0:s})", mcidObj.getTypeName());
-      return NULL;
+      return nullptr;
     }
 
     child = new StructElement(mcidObj.getInt(), treeRoot, this);
@@ -1244,7 +1248,7 @@ StructElement *StructElement::parseChild(Object *ref,
         treeRoot->parentTreeAdd(ref->getRef(), child);
     } else {
       delete child;
-      child = NULL;
+      child = nullptr;
     }
   }
 
