@@ -15,14 +15,14 @@
 //
 // Copyright (C) 2005 Dan Sheridan <dan.sheridan@postman.org.uk>
 // Copyright (C) 2005 Brad Hards <bradh@frogmouth.net>
-// Copyright (C) 2006, 2008, 2010, 2012-2014, 2016, 2017 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2006, 2008, 2010, 2012-2014, 2016-2018 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2007-2008 Julien Rebetez <julienr@svn.gnome.org>
 // Copyright (C) 2007 Carlos Garcia Campos <carlosgc@gnome.org>
 // Copyright (C) 2009, 2010 Ilya Gorenbein <igorenbein@finjan.com>
 // Copyright (C) 2010 Hib Eris <hib@hiberis.nl>
 // Copyright (C) 2012, 2013, 2016 Thomas Freitag <Thomas.Freitag@kabelmail.de>
 // Copyright (C) 2012, 2013 Fabio D'Urso <fabiodurso@hotmail.it>
-// Copyright (C) 2013, 2014 Adrian Johnson <ajohnson@redneon.com>
+// Copyright (C) 2013, 2014, 2017 Adrian Johnson <ajohnson@redneon.com>
 // Copyright (C) 2013 Pino Toscano <pino@kde.org>
 // Copyright (C) 2016 Jakub Alba <jakubalba@gmail.com>
 //
@@ -32,6 +32,7 @@
 //========================================================================
 
 #include <config.h>
+#include "poppler-config.h"
 
 #ifdef USE_GCC_PRAGMAS
 #pragma implementation
@@ -71,7 +72,7 @@
 #define permHighResPrint  (1<<11) // bit 12
 #define defPermFlags 0xfffc
 
-#if MULTITHREADED
+#ifdef MULTITHREADED
 #  define xrefLocker()   MutexLocker locker(&mutex)
 #  define xrefCondLocker(X)  MutexLocker locker(&mutex, (X))
 #else
@@ -94,6 +95,9 @@ public:
 
   ~ObjectStream();
 
+  ObjectStream(const ObjectStream &) = delete;
+  ObjectStream& operator=(const ObjectStream &) = delete;
+
   // Return the object number of this object stream.
   int getObjStrNum() { return objStrNum; }
 
@@ -151,8 +155,8 @@ ObjectStream::ObjectStream(XRef *xref, int objStrNumA, int recursion) {
 
   objStrNum = objStrNumA;
   nObjects = 0;
-  objs = NULL;
-  objNums = NULL;
+  objs = nullptr;
+  objNums = nullptr;
   ok = gFalse;
 
   objStr = xref->fetch(objStrNum, 0, recursion);
@@ -261,16 +265,16 @@ Object ObjectStream::getObject(int objIdx, int objNum) {
 //------------------------------------------------------------------------
 
 void XRef::init() {
-#if MULTITHREADED
+#ifdef MULTITHREADED
   gInitMutex(&mutex);
 #endif
   ok = gTrue;
   errCode = errNone;
-  entries = NULL;
+  entries = nullptr;
   capacity = 0;
   size = 0;
   modified = gFalse;
-  streamEnds = NULL;
+  streamEnds = nullptr;
   streamEndsLen = 0;
   objStrs = new PopplerCache(5);
   mainXRefEntriesOffset = 0;
@@ -325,7 +329,7 @@ XRef::XRef(BaseStream *strA, Goffset pos, Goffset mainXRefEntriesOffsetA, GBool
     // read the xref table
     } else {
       std::vector<Goffset> followedXRefStm;
-      readXRef(&prevXRefOffset, &followedXRefStm, NULL);
+      readXRef(&prevXRefOffset, &followedXRefStm, nullptr);
 
       // if there was a problem with the xref table,
       // try to reconstruct it
@@ -384,7 +388,7 @@ XRef::~XRef() {
   if (strOwner) {
     delete str;
   }
-#if MULTITHREADED
+#ifdef MULTITHREADED
   gDestroyMutex(&mutex);
 #endif
 }
@@ -417,7 +421,7 @@ XRef *XRef::copy() {
   if (xref->reserve(size) == 0) {
     error(errSyntaxError, -1, "unable to allocate {0:d} entries", size);
     delete xref;
-    return NULL;
+    return nullptr;
   }
   xref->size = size;
   for (int i = 0; i < size; ++i) {
@@ -451,7 +455,7 @@ int XRef::reserve(int newSize)
     }
 
     void *p = greallocn_checkoverflow(entries, realNewSize, sizeof(XRefEntry));
-    if (p == NULL) {
+    if (p == nullptr) {
       return 0;
     }
 
@@ -506,8 +510,8 @@ GBool XRef::readXRef(Goffset *pos, std::vector<Goffset> *followedXRefStm, std::v
   GBool more;
 
   // start up a parser, parse one token
-  parser = new Parser(NULL,
-	     new Lexer(NULL,
+  parser = new Parser(nullptr,
+	     new Lexer(nullptr,
 	       str->makeSubStream(start + *pos, gFalse, 0, Object(objNull))),
 	     gTrue);
   obj = parser->getObj(gTrue);
@@ -554,7 +558,7 @@ GBool XRef::readXRefTable(Parser *parser, Goffset *pos, std::vector<Goffset> *fo
   GBool more;
   Object obj, obj2;
   Goffset pos2;
-  int first, n, i;
+  int first, n;
 
   while (1) {
     obj = parser->getObj(gTrue);
@@ -570,7 +574,7 @@ GBool XRef::readXRefTable(Parser *parser, Goffset *pos, std::vector<Goffset> *fo
       goto err0;
     }
     n = obj.getInt();
-    if (first < 0 || n < 0 || first + n < 0) {
+    if (first < 0 || n < 0 || first > INT_MAX - n) {
       goto err0;
     }
     if (first + n > size) {
@@ -579,7 +583,7 @@ GBool XRef::readXRefTable(Parser *parser, Goffset *pos, std::vector<Goffset> *fo
         goto err0;
       }
     }
-    for (i = first; i < first + n; ++i) {
+    for (int i = first; i < first + n; ++i) {
       obj = parser->getObj(gTrue);
       if (obj.isInt()) {
 	entry.offset = obj.getInt();
@@ -623,6 +627,7 @@ GBool XRef::readXRefTable(Parser *parser, Goffset *pos, std::vector<Goffset> *fo
 	  entries[0].obj = std::move(entries[1].obj);
 
 	  entries[1].offset = -1;
+	  entries[1].obj.setToNull();
 	}
       }
     }
@@ -733,7 +738,7 @@ GBool XRef::readXRefStream(Stream *xrefStr, Goffset *pos) {
       return gFalse;
     }
   }
-  if (w[0] > (int)sizeof(int) || w[1] > (int)sizeof(long long) || w[2] > (int)sizeof(int)) {
+  if (w[0] > (int)sizeof(int) || w[1] > (int)sizeof(long long) || w[2] > (int)sizeof(long long)) {
     return gFalse;
   }
 
@@ -781,9 +786,12 @@ GBool XRef::readXRefStream(Stream *xrefStr, Goffset *pos) {
 }
 
 GBool XRef::readXRefStreamSection(Stream *xrefStr, int *w, int first, int n) {
-  unsigned long long offset;
-  int type, gen, c, i, j;
+  unsigned long long offset, gen;
+  int type, c, i, j;
 
+  if (first > INT_MAX - n) {
+    return gFalse;
+  }
   if (first + n < 0) {
     return gFalse;
   }
@@ -824,6 +832,10 @@ GBool XRef::readXRefStreamSection(Stream *xrefStr, int *w, int first, int n) {
       }
       gen = (gen << 8) + c;
     }
+    if (gen > INT_MAX) {
+      error(errSyntaxError, -1, "Gen inside xref table too large (bigger than INT_MAX)");
+      return gFalse;
+    }
     if (entries[i].offset == -1) {
       switch (type) {
       case 0:
@@ -857,11 +869,10 @@ GBool XRef::constructXRef(GBool *wasReconstructed, GBool needCatalogDict) {
   char buf[256];
   Goffset pos;
   int num, gen;
-  int newSize;
   int streamEndsSize;
   char *p;
   GBool gotRoot;
-  char* token = NULL;
+  char* token = nullptr;
   bool oneCycle = true;
   int offset = 0;
 
@@ -869,7 +880,7 @@ GBool XRef::constructXRef(GBool *wasReconstructed, GBool needCatalogDict) {
   gfree(entries);
   capacity = 0;
   size = 0;
-  entries = NULL;
+  entries = nullptr;
 
   gotRoot = gFalse;
   streamEndsLen = streamEndsSize = 0;
@@ -904,8 +915,8 @@ GBool XRef::constructXRef(GBool *wasReconstructed, GBool needCatalogDict) {
 
       // got trailer dictionary
       if (!strncmp(p, "trailer", 7)) {
-        parser = new Parser(NULL,
-		 new Lexer(NULL,
+        parser = new Parser(nullptr,
+		 new Lexer(nullptr,
 		   str->makeSubStream(pos + 7, gFalse, 0, Object(objNull))),
 		 gFalse);
         Object newTrailerDict = parser->getObj();
@@ -931,11 +942,11 @@ GBool XRef::constructXRef(GBool *wasReconstructed, GBool needCatalogDict) {
           if ((*p & 0xff) == 0) {
             //new line, continue with next line!
             str->getLine(buf, 256);
-            p = buf - 1;
-          }
-	  do {
+            p = buf;
+          } else {
 	    ++p;
-	  } while (*p && isspace(*p & 0xff));
+	  }
+	  while (*p && isspace(*p & 0xff)) ++p;
 	  if (isdigit(*p & 0xff)) {
 	    gen = atoi(p);
 	    do {
@@ -945,14 +956,18 @@ GBool XRef::constructXRef(GBool *wasReconstructed, GBool needCatalogDict) {
               if ((*p & 0xff) == 0) {
                 //new line, continue with next line!
                 str->getLine(buf, 256);
-                p = buf - 1;
-              }
-	      do {
+                p = buf;
+              } else {
 		++p;
-	      } while (*p && isspace(*p & 0xff));
+	      }
+	      while (*p && isspace(*p & 0xff)) ++p;
 	      if (!strncmp(p, "obj", 3)) {
 		if (num >= size) {
-		  newSize = (num + 1 + 255) & ~255;
+		  if (unlikely(num >= INT_MAX - 1 - 255)) {
+		    error(errSyntaxError, -1, "Bad object number");
+		    return gFalse;
+		  }
+		  const int newSize = (num + 1 + 255) & ~255;
 		  if (newSize < 0) {
 		    error(errSyntaxError, -1, "Bad object number");
 		    return gFalse;
@@ -1042,7 +1057,7 @@ void XRef::getEncryptionParameters(Guchar **fileKeyA, CryptAlgorithm *encAlgorit
     *keyLengthA = keyLength;
   } else {
     // null encryption parameters
-    *fileKeyA = NULL;
+    *fileKeyA = nullptr;
     *encAlgorithmA = cryptRC4;
     *keyLengthA = 0;
   }
@@ -1126,7 +1141,7 @@ Object XRef::fetch(int num, int gen, int recursion) {
 
   case xrefEntryUncompressed:
   {
-    if (e->gen != gen) {
+    if (e->gen != gen || e->offset < 0) {
       goto err;
     }
     parser = new Parser(this,
@@ -1162,7 +1177,7 @@ Object XRef::fetch(int num, int gen, int recursion) {
       delete parser;
       goto err;
     }
-    Object obj = parser->getObj(gFalse, (encrypted && !e->getFlag(XRefEntry::Unencrypted)) ? fileKey : NULL,
+    Object obj = parser->getObj(gFalse, (encrypted && !e->getFlag(XRefEntry::Unencrypted)) ? fileKey : nullptr,
 		   encAlgorithm, keyLength, num, gen, recursion);
     delete parser;
     return obj;
@@ -1181,7 +1196,7 @@ Object XRef::fetch(int num, int gen, int recursion) {
       goto err;
     }
 
-    ObjectStream *objStr = NULL;
+    ObjectStream *objStr = nullptr;
     ObjectStreamKey key(e->offset);
     PopplerCacheItem *item = objStrs->lookup(key);
     if (item) {
@@ -1193,7 +1208,7 @@ Object XRef::fetch(int num, int gen, int recursion) {
       objStr = new ObjectStream(this, e->offset, recursion + 1);
       if (!objStr->isOk()) {
 	delete objStr;
-	objStr = NULL;
+	objStr = nullptr;
 	goto err;
       } else {
 	// XRef could be reconstructed in constructor of ObjectStream:
@@ -1221,13 +1236,13 @@ Object XRef::fetch(int num, int gen, int recursion) {
 }
 
 void XRef::lock() {
-#if MULTITHREADED
+#ifdef MULTITHREADED
   gLockMutex(&mutex);
 #endif
 }
 
 void XRef::unlock() {
-#if MULTITHREADED
+#ifdef MULTITHREADED
   gUnlockMutex(&mutex);
 #endif
 }
@@ -1526,7 +1541,10 @@ GBool XRef::parseEntry(Goffset offset, XRefEntry *entry)
 {
   GBool r;
 
-  Parser parser(NULL, new Lexer(NULL,
+  if (unlikely(entry == nullptr))
+    return gFalse;
+
+  Parser parser(nullptr, new Lexer(nullptr,
      str->makeSubStream(offset, gFalse, 20, Object(objNull))), gTrue);
 
   Object obj1, obj2, obj3;
@@ -1567,7 +1585,7 @@ void XRef::readXRefUntil(int untilEntryNum, std::vector<int> *xrefStreamObjsNum)
     }
     if (followed) {
       error(errSyntaxError, -1, "Circular XRef");
-      if (!(ok = constructXRef(NULL))) {
+      if (!(ok = constructXRef(nullptr))) {
         errCode = errDamaged;
       }
       break;
@@ -1710,7 +1728,6 @@ void XRef::scanSpecialFlags() {
   }
 
   // Mark objects referred from the Encrypt dict as Unencrypted
-  Object obj = trailerDict.dictLookupNF("Encrypt");
   markUnencrypted();
 }
 
