@@ -1,7 +1,7 @@
 /* poppler-document.cc: qt interface to poppler
  * Copyright (C) 2005, Net Integration Technologies, Inc.
  * Copyright (C) 2005, 2008, Brad Hards <bradh@frogmouth.net>
- * Copyright (C) 2005-2010, 2012, 2013, 2015, 2017, Albert Astals Cid <aacid@kde.org>
+ * Copyright (C) 2005-2010, 2012, 2013, 2015, 2017, 2018, Albert Astals Cid <aacid@kde.org>
  * Copyright (C) 2006-2010, Pino Toscano <pino@kde.org>
  * Copyright (C) 2010, 2011 Hib Eris <hib@hiberis.nl>
  * Copyright (C) 2012 Koji Otani <sho@bbr.jp>
@@ -10,6 +10,9 @@
  * Copyright (C) 2014 Adam Reichold <adamreichold@myopera.com>
  * Copyright (C) 2015 William Bader <williambader@hotmail.com>
  * Copyright (C) 2016 Jakub Alba <jakubalba@gmail.com>
+ * Copyright (C) 2017 Adrian Johnson <ajohnson@redneon.com>
+ * Copyright (C) 2017 Suzuki Toshiya <mpsuzuki@hiroshima-u.ac.jp>
+ * Copyright (C) 2018 Klarälvdalens Datakonsult AB, a KDAB Group company, <info@kdab.com>. Work sponsored by the LiMux project of the city of Munich
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -47,12 +50,8 @@
 #include "poppler-page-private.h"
 
 #if defined(USE_CMS)
-#if defined(USE_LCMS1)
-#include <lcms.h>
-#else
 #include <lcms2.h>
 #endif
-#endif
 
 namespace Poppler {
 
@@ -96,7 +95,7 @@ namespace Poppler {
 	{
 		delete doc;
 	}
-	return NULL;
+	return nullptr;
     }
 
     Document::Document(DocumentData *dataA)
@@ -112,9 +111,9 @@ namespace Poppler {
     Page *Document::page(int index) const
     {
 	Page *page = new Page(m_doc, index);
-	if (page->m_page->page == NULL) {
+	if (page->m_page->page == nullptr) {
 	  delete page;
-	  return NULL;
+	  return nullptr;
 	}
 
 	return page;
@@ -445,7 +444,7 @@ namespace Poppler {
 
 	QScopedPointer<GooString> goo(m_doc->doc->getDocInfoStringEntry(type.toLatin1().constData()));
 	QString str = UnicodeParsedString(goo.data());
-	return Poppler::convertDate(str.toLatin1().data());
+	return Poppler::convertDate(str.toLatin1().constData());
     }
 
     bool Document::setDate( const QString & key, const QDateTime & val )
@@ -466,7 +465,7 @@ namespace Poppler {
 
 	QScopedPointer<GooString> goo(m_doc->doc->getDocInfoCreatDate());
 	QString str = UnicodeParsedString(goo.data());
-	return Poppler::convertDate(str.toLatin1().data());
+	return Poppler::convertDate(str.toLatin1().constData());
     }
 
     bool Document::setCreationDate( const QDateTime & val )
@@ -487,7 +486,7 @@ namespace Poppler {
 
 	QScopedPointer<GooString> goo(m_doc->doc->getDocInfoModDate());
 	QString str = UnicodeParsedString(goo.data());
-	return Poppler::convertDate(str.toLatin1().data());
+	return Poppler::convertDate(str.toLatin1().constData());
     }
 
     bool Document::setModificationDate( const QDateTime & val )
@@ -569,7 +568,7 @@ namespace Poppler {
 	int index;
 
 	if (!m_doc->doc->getCatalog()->labelToIndex (&label_g, &index))
-	    return NULL;
+	    return nullptr;
 
 	return page(index);
     }
@@ -583,11 +582,11 @@ namespace Poppler {
     {
         Outline * outline = m_doc->doc->getOutline();
         if ( !outline )
-            return NULL;
+            return nullptr;
 
-        GooList * items = outline->getItems();
+        const GooList * items = outline->getItems();
         if ( !items || items->getLength() < 1 )
-            return NULL;
+            return nullptr;
 
         QDomDocument *toc = new QDomDocument();
         if ( items->getLength() > 0 )
@@ -599,7 +598,7 @@ namespace Poppler {
     LinkDestination *Document::linkDestination( const QString &name )
     {
         GooString * namedDest = QStringToGooString( name );
-        LinkDestinationData ldd(NULL, namedDest, m_doc, false);
+        LinkDestinationData ldd(nullptr, namedDest, m_doc, false);
         LinkDestination *ld = new LinkDestination(ldd);
         delete namedDest;
         return ld;
@@ -728,7 +727,7 @@ namespace Poppler {
     OptContentModel *Document::optionalContentModel()
     {
         if (m_doc->m_optContentModel.isNull()) {
-	    m_doc->m_optContentModel = new OptContentModel(m_doc->doc->getOptContentConfig(), 0);
+	    m_doc->m_optContentModel = new OptContentModel(m_doc->doc->getOptContentConfig(), nullptr);
 	}
         return (OptContentModel *)m_doc->m_optContentModel;
     }
@@ -753,7 +752,7 @@ namespace Poppler {
         GooString gooPermanentId;
         GooString gooUpdateId;
 
-        if (!m_doc->doc->getID(permanentId ? &gooPermanentId : 0, updateId ? &gooUpdateId : 0))
+        if (!m_doc->doc->getID(permanentId ? &gooPermanentId : nullptr, updateId ? &gooUpdateId : nullptr))
             return false;
 
         if (permanentId)
@@ -795,7 +794,7 @@ namespace Poppler {
         return result;
     }
 
-    QDateTime convertDate( char *dateString )
+    QDateTime convertDate( const char *dateString )
     {
         int year, mon, day, hour, min, sec, tzHours, tzMins;
         char tz;
@@ -826,6 +825,12 @@ namespace Poppler {
         return QDateTime();
     }
 
+    QDateTime convertDate( char *dateString )
+    {
+        return convertDate( (const char *) dateString );
+    }
+
+
     bool isCmsAvailable()
     {
 #if defined(USE_CMS)
@@ -836,7 +841,7 @@ namespace Poppler {
     }
 
     bool isOverprintPreviewAvailable() {
-#if SPLASH_CMYK
+#ifdef SPLASH_CMYK
         return true;
 #else
         return false;
