@@ -59,13 +59,17 @@ namespace DocMkrParserInternal
 //! Internal: store a picture information in DocMkrParser
 struct PictInfo {
   //! constructor
-  PictInfo() : m_id(-1), m_sndId(-1), m_align(1), m_print(false), m_invert(false),
-    m_action(0), m_actionString(""), m_extra("")
+  PictInfo()
+    : m_id(-1)
+    , m_sndId(-1)
+    , m_align(1)
+    , m_print(false)
+    , m_invert(false)
+    , m_action(0)
+    , m_actionString("")
+    , m_extra("")
   {
-    for (int i= 0; i < 2; i++)
-      m_next[i]=0;
-    for (int i = 0; i < 3; i++)
-      m_appleScript[i]="";
+    for (auto &next : m_next) next=0;
   }
   //! operator<<
   friend std::ostream &operator<<(std::ostream &o, PictInfo const &info);
@@ -153,8 +157,14 @@ std::ostream &operator<<(std::ostream &o, PictInfo const &info)
 //! Internal: the state of a DocMkrParser
 struct State {
   //! constructor
-  State() : m_idPictEntryMap(), m_idPictInfoMap(), m_zonePictInfoUnit(100),
-    m_actPage(0), m_numPages(0), m_headerHeight(0), m_footerHeight(0)
+  State()
+    : m_idPictEntryMap()
+    , m_idPictInfoMap()
+    , m_zonePictInfoUnit(100)
+    , m_actPage(0)
+    , m_numPages(0)
+    , m_headerHeight(0)
+    , m_footerHeight(0)
   {
   }
   //! return a pictinfo id corresponding to a zone and a local if
@@ -181,9 +191,8 @@ void State::findPictInfoUnit(int nZones)
   if (m_idPictInfoMap.empty())
     return;
   bool is100=true, is1000=true;
-  std::map<int,PictInfo>::const_iterator it=m_idPictInfoMap.begin();
-  for (; it != m_idPictInfoMap.end(); ++it) {
-    int id=it->first;
+  for (auto it : m_idPictInfoMap) {
+    int id=it.first;
     if (id > (nZones+3)*100 || id < 200)
       is100=false;
     if (id > (nZones+3)*1000 || id < 2000)
@@ -202,8 +211,10 @@ void State::findPictInfoUnit(int nZones)
 ////////////////////////////////////////////////////////////
 // constructor/destructor, ...
 ////////////////////////////////////////////////////////////
-DocMkrParser::DocMkrParser(MWAWInputStreamPtr input, MWAWRSRCParserPtr rsrcParser, MWAWHeader *header) :
-  MWAWTextParser(input, rsrcParser, header), m_state(), m_textParser()
+DocMkrParser::DocMkrParser(MWAWInputStreamPtr const &input, MWAWRSRCParserPtr const &rsrcParser, MWAWHeader *header)
+  : MWAWTextParser(input, rsrcParser, header)
+  , m_state()
+  , m_textParser()
 {
   init();
 }
@@ -252,10 +263,10 @@ void DocMkrParser::newPage(int number)
 ////////////////////////////////////////////////////////////
 void DocMkrParser::parse(librevenge::RVNGTextInterface *docInterface)
 {
-  if (!getInput().get() || !getRSRCParser() || !checkHeader(0L))  throw(libmwaw::ParseException());
+  if (!getInput().get() || !getRSRCParser() || !checkHeader(nullptr))  throw(libmwaw::ParseException());
   bool ok = false;
   try {
-    checkHeader(0L);
+    checkHeader(nullptr);
     ok = createZones();
     if (ok) {
       createDocument(docInterface);
@@ -312,13 +323,12 @@ bool DocMkrParser::createZones()
 {
   MWAWInputStreamPtr input = getInput();
   MWAWRSRCParserPtr rsrcParser = getRSRCParser();
-  std::multimap<std::string, MWAWEntry> &entryMap = rsrcParser->getEntriesMap();
-  std::multimap<std::string, MWAWEntry>::iterator it;
+  auto &entryMap = rsrcParser->getEntriesMap();
 
   if (!m_textParser->createZones())
     return false;
   // the different pict zones
-  it = entryMap.lower_bound("PICT");
+  auto it = entryMap.lower_bound("PICT");
   while (it != entryMap.end()) {
     if (it->first != "PICT")
       break;
@@ -405,9 +415,8 @@ bool DocMkrParser::createZones()
 void DocMkrParser::flushExtra()
 {
   MWAWRSRCParserPtr rsrcParser = getRSRCParser();
-  std::map<int,MWAWEntry>::const_iterator it = m_state->m_idPictEntryMap.begin();
-  for (; it != m_state->m_idPictEntryMap.end(); ++it) {
-    MWAWEntry const &entry=it->second;
+  for (auto it : m_state->m_idPictEntryMap) {
+    MWAWEntry const &entry=it.second;
     if (entry.isParsed()) continue;
     librevenge::RVNGBinaryData data;
     rsrcParser->parsePICT(entry,data);
@@ -422,11 +431,12 @@ void DocMkrParser::flushExtra()
 bool DocMkrParser::sendPicture(int zId, int lId, double /*lineW*/)
 {
   int pictId=m_state->pictInfoId(zId,lId);
-  if (m_state->m_idPictInfoMap.find(pictId)==m_state->m_idPictInfoMap.end()) {
+  auto it=m_state->m_idPictInfoMap.find(pictId);
+  if (it==m_state->m_idPictInfoMap.end()) {
     MWAW_DEBUG_MSG(("DocMkrText::sendPicture: can not find picture for zone=%d, id=%d\n",zId,lId));
     return false;
   }
-  DocMkrParserInternal::PictInfo const &info=m_state->m_idPictInfoMap.find(pictId)->second;
+  DocMkrParserInternal::PictInfo const &info=it->second;
   if (m_state->m_idPictEntryMap.find(info.m_id)==m_state->m_idPictEntryMap.end()) {
     MWAW_DEBUG_MSG(("DocMkrText::sendPicture: can not find picture for id=%d\n",info.m_id));
     return false;
@@ -447,7 +457,7 @@ bool DocMkrParser::sendPicture(int zId, int lId, double /*lineW*/)
   rsrcParser->parsePICT(entry,data);
   input->seek(pos,librevenge::RVNG_SEEK_SET);
 
-  int dataSz=int(data.size());
+  auto dataSz=int(data.size());
   if (!dataSz) {
     return false;
   }
@@ -457,16 +467,16 @@ bool DocMkrParser::sendPicture(int zId, int lId, double /*lineW*/)
     return false;
   }
   MWAWBox2f box;
-  MWAWPict::ReadResult res = MWAWPictData::check(pictInput, dataSz,box);
+  auto res = MWAWPictData::check(pictInput, dataSz,box);
   if (res == MWAWPict::MWAW_R_BAD) {
     MWAW_DEBUG_MSG(("DocMkrText::sendPicture: can not find the picture\n"));
     return false;
   }
   pictInput->seek(0,librevenge::RVNG_SEEK_SET);
-  shared_ptr<MWAWPict> thePict(MWAWPictData::get(pictInput, dataSz));
+  std::shared_ptr<MWAWPict> thePict(MWAWPictData::get(pictInput, dataSz));
   MWAWPosition pictPos=MWAWPosition(MWAWVec2f(0,0),box.size(), librevenge::RVNG_POINT);
-  MWAWPosition::XPos xpos= (info.m_align==1) ? MWAWPosition::XCenter :
-                           (info.m_align==3) ? MWAWPosition::XRight  : MWAWPosition::XLeft;
+  auto xpos= (info.m_align==1) ? MWAWPosition::XCenter :
+             (info.m_align==3) ? MWAWPosition::XRight  : MWAWPosition::XLeft;
   pictPos.setRelativePosition(MWAWPosition::Paragraph, xpos);
   pictPos.m_wrapping = MWAWPosition::WRunThrough;
   if (thePict) {
@@ -496,11 +506,11 @@ bool DocMkrParser::readPictInfo(MWAWEntry const &entry)
   DocMkrParserInternal::PictInfo info;
   info.m_id = static_cast<int>(input->readULong(2));
   info.m_align = static_cast<int>(input->readLong(2));
-  int val =static_cast<int>(input->readLong(2)); // 0|1
+  auto val =static_cast<int>(input->readLong(2)); // 0|1
   if (val) f << "unkn=" << val << ",";
 
-  int action =static_cast<int>(input->readLong(2)); // 0..b
-  int extraN = int(endPos-input->tell());
+  auto action =static_cast<int>(input->readLong(2)); // 0..b
+  auto extraN = int(endPos-input->tell());
   if (action < 0) {
     info.m_invert=true;
     action = -action;
@@ -524,7 +534,7 @@ bool DocMkrParser::readPictInfo(MWAWEntry const &entry)
     if (extraN < 1) {
       f << "actionArg##,";
     }
-    int fSz=static_cast<int>(input->readULong(1));
+    auto fSz=static_cast<int>(input->readULong(1));
     if (extraN < fSz+1) {
       f << "##[N=" << fSz << "],";
       break;
@@ -551,13 +561,13 @@ bool DocMkrParser::readPictInfo(MWAWEntry const &entry)
       f << "actionArg##,";
       break;
     }
-    for (int w=0; w < 3; w++) {
+    for (auto &w : info.m_appleScript) {
       std::string name("");
       for (int i = 0; i < 4; i++)
         name += char(input->readULong(1));
-      info.m_appleScript[w]=name;
+      w=name;
     }
-    int fSz=static_cast<int>(input->readULong(1));
+    auto fSz=static_cast<int>(input->readULong(1));
     if (extraN < fSz+13) {
       f << "##[N=" << fSz << "],";
       break;
@@ -625,7 +635,7 @@ bool DocMkrParser::readSTwD(MWAWEntry const &entry)
     if (val)
       f << "f" << i << "=" << val << ",";
   }
-  int flag =static_cast<int>(input->readLong(2)); // 320|7d0|1388 ?
+  auto flag =static_cast<int>(input->readLong(2)); // 320|7d0|1388 ?
   f << "fl=" << std::hex << flag << std::dec << ",";
   f << "dim=" << static_cast<int>(input->readLong(2)) << ","; // 0x1Fa|0x200
   for (int i=0; i < 2; i++) { // f0=1, f1=0|1
@@ -663,7 +673,7 @@ bool DocMkrParser::readXtr2(MWAWEntry const &entry)
   }
   // f0=79|a8|b9|99
   for (int i=0; i < N; i++) {
-    int val =static_cast<int>(input->readULong(1));
+    auto val =static_cast<int>(input->readULong(1));
     if (val)
       f << "f" << i << "=" << std::hex << val << std::dec << ",";
   }
