@@ -61,16 +61,20 @@ namespace MoreTextInternal
 {
 ////////////////////////////////////////
 //! Internal: the paragraph of a MoreText
-struct Paragraph : public MWAWParagraph {
+struct Paragraph final : public MWAWParagraph {
   //! constructor
-  Paragraph(): MWAWParagraph(), m_listType(0), m_customListLevel(),
-    m_pageBreak(false), m_keepOutlineTogether(false)
+  Paragraph()
+    : MWAWParagraph()
+    , m_listType(0)
+    , m_customListLevel()
+    , m_pageBreak(false)
+    , m_keepOutlineTogether(false)
   {
     m_marginsFromParent[0]=0.3;
     m_marginsFromParent[1]=0;
   }
   //! destructor
-  virtual ~Paragraph();
+  ~Paragraph() final;
   //! set the left margin in inch
   void setLeftMargin(double margin, bool fromParent)
   {
@@ -154,11 +158,10 @@ struct Paragraph : public MWAWParagraph {
     }
 
     m_listLevelIndex = level+1;
-    shared_ptr<MWAWList> parentList;
+    std::shared_ptr<MWAWList> parentList;
     if (*parent.m_listId>=0)
       parentList=listManager.getList(*parent.m_listId);
-    shared_ptr<MWAWList> list=
-      listManager.getNewList(parentList, level+1, listLevel);
+    auto list=listManager.getNewList(parentList, level+1, listLevel);
     if (list)
       m_listId=list->getId();
     else
@@ -189,8 +192,7 @@ struct Outline {
   Outline()
   {
     // set to default value
-    for (int i=0; i < 4; i++)
-      m_fonts[i]=MWAWFont(3,12);
+    for (auto &font : m_fonts) font=MWAWFont(3,12);
     m_paragraphs[0].m_listType=1;
   }
   //! the paragraphs : organizer, slide, tree, unknowns
@@ -203,10 +205,13 @@ struct Outline {
 //! Internal and low level: the outline modifier header of a MoreText
 struct OutlineMod {
   //! constructor
-  OutlineMod(): m_type(-1), m_flags(0), m_entry(), m_extra("")
+  OutlineMod()
+    : m_type(-1)
+    , m_flags(0)
+    , m_entry()
+    , m_extra("")
   {
-    for (int i=0; i<2; i++)
-      m_unknowns[i]=0;
+    for (auto &unkn : m_unknowns) unkn=0;
   }
   //! returns the data id to change in Outline
   int getModId() const
@@ -321,7 +326,9 @@ struct OutlineMod {
 //! Internal: the comment data of a MoreText
 struct Comment {
   //! constructor
-  Comment() : m_entry(), m_extra("")
+  Comment()
+    : m_entry()
+    , m_extra("")
   {
   }
   //! operator<<
@@ -342,12 +349,17 @@ struct Topic {
   //! an enum used to define the different type of data attached to a topic
   enum AttachementType { AOutline=0, AComment, ASpeakerNote };
   //! constructor
-  Topic() : m_entry(), m_level(0), m_isCloned(false), m_cloneId(-1), m_numPageBreak(-1), m_isStartSlide(false), m_extra("")
+  Topic()
+    : m_entry()
+    , m_level(0)
+    , m_isCloned(false)
+    , m_cloneId(-1)
+    , m_numPageBreak(-1)
+    , m_isStartSlide(false)
+    , m_extra("")
   {
-    for (int i=0; i < 3; i++) {
-      m_hasList[i]=false;
-      m_attachList[i]=-1;
-    }
+    for (auto &hasData : m_hasList) hasData=false;
+    for (auto &attachData : m_attachList) attachData=-1;
   }
   //! operator<<
   friend std::ostream &operator<<(std::ostream &o, Topic const &topic)
@@ -393,8 +405,17 @@ struct Topic {
 //! Internal: the state of a MoreText
 struct State {
   //! constructor
-  State() : m_version(-1), m_topicList(), m_commentList(), m_speakerList(), m_outlineList(),
-    m_actualComment(0), m_actualSpeaker(0), m_actualOutline(0), m_numPages(-1), m_actualPage(1)
+  State()
+    : m_version(-1)
+    , m_topicList()
+    , m_commentList()
+    , m_speakerList()
+    , m_outlineList()
+    , m_actualComment(0)
+    , m_actualSpeaker(0)
+    , m_actualOutline(0)
+    , m_numPages(-1)
+    , m_actualPage(1)
   {
   }
   //! the file version
@@ -418,25 +439,25 @@ struct State {
 
 ////////////////////////////////////////
 //! Internal: the subdocument of a MoreText
-class SubDocument : public MWAWSubDocument
+class SubDocument final : public MWAWSubDocument
 {
 public:
-  SubDocument(MoreText &pars, MWAWInputStreamPtr input, int zId, int what) :
-    MWAWSubDocument(pars.m_mainParser, input, MWAWEntry()), m_textParser(&pars), m_id(zId), m_what(what) {}
-
-  //! destructor
-  virtual ~SubDocument() {}
-
-  //! operator!=
-  virtual bool operator!=(MWAWSubDocument const &doc) const;
-  //! operator!==
-  virtual bool operator==(MWAWSubDocument const &doc) const
+  SubDocument(MoreText &pars, MWAWInputStreamPtr const &input, int zId, int what)
+    : MWAWSubDocument(pars.m_mainParser, input, MWAWEntry())
+    , m_textParser(&pars)
+    , m_id(zId)
+    , m_what(what)
   {
-    return !operator!=(doc);
   }
 
+  //! destructor
+  ~SubDocument() final {}
+
+  //! operator!=
+  bool operator!=(MWAWSubDocument const &doc) const final;
+
   //! the parser function
-  void parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType type);
+  void parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType type) final;
 
 protected:
   /** the text parser */
@@ -446,8 +467,8 @@ protected:
   //! a int to know what to send 0: header/footer, 1: comment, 2:note
   int m_what;
 private:
-  SubDocument(SubDocument const &orig);
-  SubDocument &operator=(SubDocument const &orig);
+  SubDocument(SubDocument const &orig) = delete;
+  SubDocument &operator=(SubDocument const &orig) = delete;
 };
 
 void SubDocument::parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType /*type*/)
@@ -484,7 +505,7 @@ void SubDocument::parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType /*ty
 bool SubDocument::operator!=(MWAWSubDocument const &doc) const
 {
   if (MWAWSubDocument::operator!=(doc)) return true;
-  SubDocument const *sDoc = dynamic_cast<SubDocument const *>(&doc);
+  auto const *sDoc = dynamic_cast<SubDocument const *>(&doc);
   if (!sDoc) return true;
   if (m_textParser != sDoc->m_textParser) return true;
   if (m_id != sDoc->m_id) return true;
@@ -495,13 +516,16 @@ bool SubDocument::operator!=(MWAWSubDocument const &doc) const
 ////////////////////////////////////////////////////////////
 // constructor/destructor, ...
 ////////////////////////////////////////////////////////////
-MoreText::MoreText(MoreParser &parser) :
-  m_parserState(parser.getParserState()), m_state(new MoreTextInternal::State), m_mainParser(&parser)
+MoreText::MoreText(MoreParser &parser)
+  : m_parserState(parser.getParserState())
+  , m_state(new MoreTextInternal::State)
+  , m_mainParser(&parser)
 {
 }
 
 MoreText::~MoreText()
-{ }
+{
+}
 
 int MoreText::version() const
 {
@@ -520,13 +544,13 @@ int MoreText::numPages() const
   return m_state->m_numPages;
 }
 
-shared_ptr<MWAWSubDocument> MoreText::getHeaderFooter(bool header)
+std::shared_ptr<MWAWSubDocument> MoreText::getHeaderFooter(bool header)
 {
-  shared_ptr<MWAWSubDocument> res;
+  std::shared_ptr<MWAWSubDocument> res;
   size_t id=header ? 1 : 2;
   if (id >= m_state->m_topicList.size())
     return res;
-  MoreTextInternal::Topic const &topic= m_state->m_topicList[id];
+  auto const &topic= m_state->m_topicList[id];
   // check if the content is empty
   int comment=topic.m_attachList[MoreTextInternal::Topic::AComment];
   if (comment < 0 || comment >= int(m_state->m_commentList.size()))
@@ -552,24 +576,25 @@ bool MoreText::createZones()
   // first create the list of cloned topic
   std::vector<int> clonedList;
   for (size_t i=0; i < m_state->m_topicList.size(); i++) {
-    MoreTextInternal::Topic const &topic=m_state->m_topicList[i];
+    auto const &topic=m_state->m_topicList[i];
     if (topic.m_isCloned)
       clonedList.push_back(int(i));
   }
   // now associated each topic with its outline, its comment, its original topic (if clone)
-  int numCloned=int(clonedList.size());
+  auto numCloned=int(clonedList.size());
   size_t actAttach[3]= {0,0,0};
   size_t numAttach[3]= {0,0,0};
   numAttach[MoreTextInternal::Topic::AOutline]=m_state->m_outlineList.size();
   numAttach[MoreTextInternal::Topic::AComment]=m_state->m_commentList.size();
   numAttach[MoreTextInternal::Topic::ASpeakerNote]=m_state->m_speakerList.size();
-  for (size_t i=0; i < m_state->m_topicList.size(); i++) {
-    MoreTextInternal::Topic &topic=m_state->m_topicList[i];
+  int n=0;
+  for (auto &topic : m_state->m_topicList) {
+    ++n;
     for (int j=0; j < 3; j++) {
       if (!topic.m_hasList[j])
         continue;
       if (actAttach[j] >= numAttach[j]) {
-        MWAW_DEBUG_MSG(("MoreText::createZones: can not find attach-%d for topic %d\n", int(j), int(i)));
+        MWAW_DEBUG_MSG(("MoreText::createZones: can not find attach-%d for topic %d\n", j, n-1));
         continue;
       }
       topic.m_attachList[j]=int(actAttach[j]++);
@@ -592,7 +617,7 @@ bool MoreText::createZones()
     if (cloneId < 0)
       continue;
     if (cloneId==0 || cloneId > numCloned) {
-      MWAW_DEBUG_MSG(("MoreText::createZones: can not find original for topic %d\n", int(i)));
+      MWAW_DEBUG_MSG(("MoreText::createZones: can not find original for topic %d\n", n-1));
       topic.m_cloneId=-1;
     }
     else
@@ -601,7 +626,7 @@ bool MoreText::createZones()
 
   // now check that we have no loop
   for (size_t i=0; i < m_state->m_topicList.size(); i++) {
-    MoreTextInternal::Topic &topic=m_state->m_topicList[i];
+    auto &topic=m_state->m_topicList[i];
     if (topic.m_cloneId<0)
       continue;
     std::set<size_t> parent;
@@ -609,8 +634,7 @@ bool MoreText::createZones()
   }
   // now compute the number of page
   int nPages = 1;
-  for (size_t i=0; i < m_state->m_topicList.size(); i++) {
-    MoreTextInternal::Topic const &topic=m_state->m_topicList[i];
+  for (auto const &topic : m_state->m_topicList) {
     if (topic.m_numPageBreak >= 0) {
       nPages+=topic.m_numPageBreak;
     }
@@ -631,7 +655,7 @@ int MoreText::getLastTopicChildId(int tId) const
     MWAW_DEBUG_MSG(("MoreText::getLastTopicChildId: can not find topic %d\n", tId));
     return tId;
   }
-  size_t p=size_t(tId);
+  auto p=size_t(tId);
   int level=m_state->m_topicList[p].m_level;
   while (p+1<numTopic && m_state->m_topicList[p].m_level>level)
     p++;
@@ -650,14 +674,14 @@ int MoreText::checkTopicList(size_t tId, std::set<size_t> &parent)
     throw libmwaw::ParseException();
   }
   parent.insert(tId);
-  MoreTextInternal::Topic &topic=m_state->m_topicList[tId];
+  auto &topic=m_state->m_topicList[tId];
   int nPages=0;
   int outId=topic.m_attachList[MoreTextInternal::Topic::AOutline];
   if (outId>=0) {
     if (m_state->m_outlineList[size_t(outId)].m_paragraphs[0].m_pageBreak)
       nPages++;
   }
-  int id=int(tId);
+  auto id=int(tId);
   if (topic.m_cloneId>=0) {
     if (parent.find(size_t(topic.m_cloneId))!=parent.end()) {
       MWAW_DEBUG_MSG(("MoreText::checkTopicList: find a loop, remove a clone\n"));
@@ -697,14 +721,14 @@ bool MoreText::readTopic(MWAWEntry const &entry)
   ascFile.addPos(pos);
   ascFile.addNote("Entries(Topic)");
 
-  int N=int(entry.length()/10);
+  auto N=int(entry.length()/10);
   for (int i=0; i < N; i++) {
     pos=input->tell();
     MoreTextInternal::Topic topic;
     f.str("");
     topic.m_level = static_cast<int>(input->readLong(2));
     bool isAClone=false;
-    int flag = static_cast<int>(input->readULong(2)); // some flag ?
+    auto flag = static_cast<int>(input->readULong(2)); // some flag ?
     if ((flag&1)==0) f << "hidden,";
     if (flag&4) f << "marked,";
     if (flag&0x10) topic.m_isCloned=true;
@@ -738,7 +762,7 @@ bool MoreText::readTopic(MWAWEntry const &entry)
         topic.m_entry = MWAWEntry();
       }
     }
-    int val = static_cast<int>(input->readLong(2)); // a small number 1 or 2
+    auto val = static_cast<int>(input->readLong(2)); // a small number 1 or 2
     if (val)
       f << "f1=" << val << ",";
     topic.m_extra=f.str();
@@ -771,7 +795,7 @@ bool MoreText::readComment(MWAWEntry const &entry)
   ascFile.addPos(pos);
   ascFile.addNote("Entries(Comment)");
 
-  int N=int(entry.length()/8);
+  auto N=int(entry.length()/8);
   for (int i=0; i < N; i++) {
     pos=input->tell();
     MoreTextInternal::Comment comment;
@@ -784,7 +808,7 @@ bool MoreText::readComment(MWAWEntry const &entry)
       f << "###";
       comment.m_entry.setLength(0);
     }
-    int val = static_cast<int>(input->readLong(2)); // always 4 ?
+    auto val = static_cast<int>(input->readLong(2)); // always 4 ?
     if (val != 4)
       f << "f0=" << val << ",";
     val = static_cast<int>(input->readULong(2)); // some flag ? find 0x3333 0x200d ...
@@ -817,7 +841,7 @@ bool MoreText::readSpeakerNote(MWAWEntry const &entry)
   entry.setParsed(true);
 
   f << "Entries(SpeakerNote):";
-  int N=int(entry.length()/4);
+  auto N=int(entry.length()/4);
   for (int i=0; i < N; i++) {
     long fPos = input->readLong(4);
     f << "S" << i << ":pos=" << std::hex << fPos << std::dec << ",";
@@ -845,7 +869,7 @@ bool MoreText::sendMainText()
   std::vector<MWAWParagraph> paraStack;
   /* skip 0: unknown, 1: header(comment), 2: footer(comment), 3: title?, after text */
   for (size_t i=4; i < m_state->m_topicList.size(); i++) {
-    MoreTextInternal::Topic const &topic = m_state->m_topicList[i];
+    auto const &topic = m_state->m_topicList[i];
     MWAWEntry const &entry=topic.m_entry;
     if (!entry.valid()) { // a clone ?
       sendTopic(int(i),0,paraStack);
@@ -904,7 +928,7 @@ bool MoreText::sendTopic(int tId, int dLevel, std::vector<MWAWParagraph> &paraSt
     MWAW_DEBUG_MSG(("MoreText::sendTopic: can not find the topic note %d!", tId));
     return false;
   }
-  MoreTextInternal::Topic const &topic=m_state->m_topicList[size_t(tId)];
+  auto const &topic=m_state->m_topicList[size_t(tId)];
   MWAWEntry entry=topic.m_entry;
   int level=topic.m_level+dLevel;
   if (level < 0) {
@@ -923,8 +947,7 @@ bool MoreText::sendTopic(int tId, int dLevel, std::vector<MWAWParagraph> &paraSt
     entry=m_state->m_commentList[size_t(comment)].m_entry;
   }
   else if (topic.m_cloneId>=0 && topic.m_cloneId < static_cast<int>(m_state->m_topicList.size())) {
-    MoreTextInternal::Topic const &cTopic =
-      m_state->m_topicList[size_t(topic.m_cloneId)];
+    auto const &cTopic = m_state->m_topicList[size_t(topic.m_cloneId)];
     entry=cTopic.m_entry;
     actId = topic.m_cloneId;
     lastId = getLastTopicChildId(topic.m_cloneId);
@@ -937,7 +960,7 @@ bool MoreText::sendTopic(int tId, int dLevel, std::vector<MWAWParagraph> &paraSt
   MoreTextInternal::Paragraph para;
   int outl=topic.m_attachList[MoreTextInternal::Topic::AOutline];
   if (outl>=0 && outl < static_cast<int>(m_state->m_outlineList.size())) {
-    MoreTextInternal::Outline const &outline=m_state->m_outlineList[size_t(outl)];
+    auto const &outline=m_state->m_outlineList[size_t(outl)];
     if (outline.m_paragraphs[0].m_pageBreak)
       m_mainParser->newPage(++m_state->m_actualPage);
     para=outline.m_paragraphs[0];
@@ -1023,7 +1046,7 @@ bool MoreText::sendText(MWAWEntry const &entry, MWAWFont const &font)
     long actPos = input->tell();
     if (actPos >= endPos)
       break;
-    unsigned char c=static_cast<unsigned char>(input->readULong(1));
+    auto c=static_cast<unsigned char>(input->readULong(1));
     if (c!=0x1b) {
       listener->insertCharacter(c);
       f << c;
@@ -1034,7 +1057,7 @@ bool MoreText::sendText(MWAWEntry const &entry, MWAWFont const &font)
       MWAW_DEBUG_MSG(("MoreText::sendText: text end by 0x1b\n"));
       continue;
     }
-    int fld=static_cast<int>(input->readULong(1));
+    auto fld=static_cast<int>(input->readULong(1));
     bool sendFont=true;
     switch (fld) {
     case 0x9:
@@ -1115,8 +1138,7 @@ bool MoreText::sendText(MWAWEntry const &entry, MWAWFont const &font)
         break;
       }
       uint16_t values[5];
-      for (int i=0; i < 5; i++)
-        values[i]=static_cast<uint16_t>(input->readULong(2));
+      for (auto &v : values) v=static_cast<uint16_t>(input->readULong(2));
       if (values[0]!=0xe || values[4]!=0xe) {
         MWAW_DEBUG_MSG(("MoreText::sendText: field fCol: color sep seems bad\n"));
         f << "@[fCol###]";
@@ -1234,8 +1256,7 @@ bool MoreText::sendText(MWAWEntry const &entry, MWAWFont const &font)
         break;
       }
       uint16_t values[4];
-      for (int i=0; i < 4; i++)
-        values[i]=static_cast<uint16_t>(input->readULong(2));
+      for (auto &v : values) v=static_cast<uint16_t>(input->readULong(2));
       if (values[0]!=0xc || values[3]!=0xc) {
         MWAW_DEBUG_MSG(("MoreText::sendText: field the separator seems bad\n"));
         f << "@[field###]";
@@ -1308,7 +1329,7 @@ bool MoreText::sendText(MWAWEntry const &entry, MWAWFont const &font)
         MWAW_DEBUG_MSG(("MoreText::sendText: field f9 seems too short\n"));
         break;
       }
-      long sz=long(input->readULong(4));
+      auto sz=long(input->readULong(4));
       if (sz<22 || actPos+sz>=endPos) {
         MWAW_DEBUG_MSG(("MoreText::sendText: field f9: bad field size\n"));
         f << "###";
@@ -1329,12 +1350,11 @@ bool MoreText::sendText(MWAWEntry const &entry, MWAWFont const &font)
       if (val!=0x100)
         f << "type=" << std::hex << val << std::dec << ",";
       float dim[4];
-      for (int i=0; i < 4; ++i)
-        dim[i] = float(input->readLong(2));
+      for (auto &d : dim) d = float(input->readLong(2));
       MWAWBox2f bdbox(MWAWVec2f(dim[1],dim[0]), MWAWVec2f(dim[3],dim[2]));
       f << "bdbox=" << bdbox << ",";
       if (sz>22) {
-        shared_ptr<MWAWPict> pict(MWAWPictData::get(input, static_cast<int>(sz)-22));
+        std::shared_ptr<MWAWPict> pict(MWAWPictData::get(input, static_cast<int>(sz)-22));
         MWAWEmbeddedObject picture;
         if (pict && pict->getBinary(picture)) {
           MWAWPosition pictPos(MWAWVec2f(0,0), bdbox.size(), librevenge::RVNG_POINT);
@@ -1358,7 +1378,7 @@ bool MoreText::sendText(MWAWEntry const &entry, MWAWFont const &font)
       break;
     }
     default: {
-      int sz=static_cast<int>(input->readULong(2));
+      auto sz=static_cast<int>(input->readULong(2));
       if (sz>4 && actPos+sz<=endPos) {
         input->seek(actPos+sz-4, librevenge::RVNG_SEEK_SET);
         if (static_cast<int>(input->readULong(2))==sz &&
@@ -1410,7 +1430,7 @@ bool MoreText::readFonts(MWAWEntry const &entry)
       MWAW_DEBUG_MSG(("MoreText::readFonts: problem reading a font\n"));
       break;
     }
-    int fSz=int(input->readULong(1));
+    auto fSz=int(input->readULong(1));
     if (fSz==0)
       break;
     if (pos+1+fSz+2 > endPos) {
@@ -1426,7 +1446,7 @@ bool MoreText::readFonts(MWAWEntry const &entry)
     for (int i=0; i < fSz; i++)
       name+=char(input->readULong(1));
     if ((fSz&1)==0) input->seek(1, librevenge::RVNG_SEEK_CUR);
-    int id=static_cast<int>(input->readULong(2));
+    auto id=static_cast<int>(input->readULong(2));
     f << name << ",id=" << id << ",";
     if (name.length())
       m_parserState->m_fontConverter->setCorrespondance(id, name);
@@ -1463,7 +1483,7 @@ bool MoreText::readOutlineList(MWAWEntry const &entry)
   entry.setParsed(true);
 
   f << "Entries(Outline):";
-  int N=int(entry.length()/4);
+  auto N=int(entry.length()/4);
   std::vector<MWAWEntry> posList;
   for (int i=0; i < N; i++) {
     MWAWEntry tEntry;
@@ -1480,8 +1500,7 @@ bool MoreText::readOutlineList(MWAWEntry const &entry)
   ascFile.addPos(pos);
   ascFile.addNote(f.str().c_str());
 
-  for (size_t i=0; i < posList.size(); i++) {
-    MWAWEntry const &tEntry=posList[i];
+  for (auto const &tEntry : posList) {
     if (!tEntry.valid())
       continue;
     MoreTextInternal::Outline outline;
@@ -1514,12 +1533,12 @@ bool MoreText::readOutline(MWAWEntry const &entry, MoreTextInternal::Outline &ou
   input->seek(pos+4, librevenge::RVNG_SEEK_SET); // skip size
 
   f << "Outline[O" << entry.id() << "]:";
-  int val=static_cast<int>(input->readULong(2));
+  auto val=static_cast<int>(input->readULong(2));
   if (val!=6*(vers-1)) {
     MWAW_DEBUG_MSG(("MoreText::readOutline: find unexpected type\n"));
     f << "#f0=" << val << ",";
   }
-  int N=static_cast<int>(input->readULong(2));
+  auto N=static_cast<int>(input->readULong(2));
   f << "N=" << N << ",";
   long lastListPos = pos+8+N*16;
   if (lastListPos > endPos) {
@@ -1540,14 +1559,12 @@ bool MoreText::readOutline(MWAWEntry const &entry, MoreTextInternal::Outline &ou
       f << "#f0=" << val << ",";
 
     outlineMod.m_flags=int(input->readULong(1));
-    for (int i=0; i <2; i++)
-      outlineMod.m_unknowns[i]=int(input->readLong(2));
+    for (auto &unkn : outlineMod.m_unknowns) unkn=int(input->readLong(2));
     outlineMod.m_type=int(input->readULong(2));
     int values[4];
-    for (int i=0; i < 4; i++)
-      values[i] = static_cast<int>(input->readULong(2));
+    for (auto &v : values) v = static_cast<int>(input->readULong(2));
     int const which= outlineMod.getModId();
-    MoreTextInternal::Paragraph &para = outline.m_paragraphs[which];
+    auto &para = outline.m_paragraphs[which];
     MWAWFont &font=outline.m_fonts[which];
     uint32_t fFlags=font.flags();
     bool haveExtra=false;
@@ -1789,15 +1806,16 @@ bool MoreText::readOutline(MWAWEntry const &entry, MoreTextInternal::Outline &ou
     input->seek(pos+16, librevenge::RVNG_SEEK_SET);
   }
 
-  for (size_t n=0; n < outlineModList.size(); n++) {
-    MoreTextInternal::OutlineMod const &outlineMod=outlineModList[n];
+  int n=0;
+  for (auto const &outlineMod : outlineModList) {
+    n++;
     if (!outlineMod.m_entry.valid())
       continue;
     f.str("");
-    f << "Outline[O" << entry.id() << "-A" << n << "]:";
+    f << "Outline[O" << entry.id() << "-A" << n-1 << "]:";
     bool ok=false;
 
-    MoreTextInternal::Paragraph &para = outline.m_paragraphs[outlineMod.getModId()];
+    auto &para = outline.m_paragraphs[outlineMod.getModId()];
     MWAWFont &font=outline.m_fonts[outlineMod.getModId()];
     switch (outlineMod.m_type) {
     case 0x301: {
@@ -1857,14 +1875,14 @@ bool MoreText::readFont(MWAWEntry const &entry, std::string &fName, int &fId)
   long pos = entry.begin();
   input->seek(pos, librevenge::RVNG_SEEK_SET);
 
-  int fSz=static_cast<int>(input->readULong(1));
+  auto fSz=static_cast<int>(input->readULong(1));
   long remain=entry.length()-long(1+fSz);
   if (fSz==0 || remain<0 || remain==1)
     return false;
   if (remain>=2 && remain!=2+(1-(fSz%2)))
     return false;
   for (int i=0; i < fSz; i++) {
-    char c=char(input->readULong(1));
+    auto c=char(input->readULong(1));
     if (c==0) return false;
     fName+=c;
   }
@@ -1888,7 +1906,7 @@ bool MoreText::readCustomListLevel(MWAWEntry const &entry, MWAWListLevel &level)
   long pos = entry.begin();
   input->seek(pos, librevenge::RVNG_SEEK_SET);
   MWAWFont font;
-  int fId=static_cast<int>(input->readULong(2));
+  auto fId=static_cast<int>(input->readULong(2));
   if (fId==0xFFFF) // default
     ;
   else if (fId&0x8000) {
@@ -1897,12 +1915,12 @@ bool MoreText::readCustomListLevel(MWAWEntry const &entry, MWAWListLevel &level)
   }
   else
     f << "#fId=" << std::hex << fId << std::dec << ",";
-  int fSz = static_cast<int>(input->readLong(2));
+  auto fSz = static_cast<int>(input->readLong(2));
   if (fSz != -1) {
     font.setSize(float(fSz));
     f << "fSz=" << fSz << ",";
   }
-  int fFlags=static_cast<int>(input->readULong(1));
+  auto fFlags=static_cast<int>(input->readULong(1));
   uint32_t flags=0;
   if (fFlags&1) flags |= MWAWFont::boldBit;
   if (fFlags&2) flags |= MWAWFont::italicBit;
@@ -1913,13 +1931,12 @@ bool MoreText::readCustomListLevel(MWAWEntry const &entry, MWAWListLevel &level)
     f << "#fFlags=" << std::hex << (fFlags&0xE0) << std::dec << ",";
   font.setFlags(flags);
 
-  int fColor = static_cast<int>(input->readLong(1));
+  auto fColor = static_cast<int>(input->readLong(1));
   if (fColor==1)
     input->seek(6, librevenge::RVNG_SEEK_CUR);
   else if (fColor==3) {
     unsigned char color[3];
-    for (int i=0; i < 3; i++)
-      color[i]=static_cast<unsigned char>(input->readULong(2)>>8);
+    for (auto &c : color) c=static_cast<unsigned char>(input->readULong(2)>>8);
     font.setColor(MWAWColor(color[0], color[1], color[2]));
   }
   else {
@@ -1957,7 +1974,7 @@ bool MoreText::readCustomListLevel(MWAWEntry const &entry, MWAWListLevel &level)
   if (val != 0xb)
     f << "f6=" << val << ",";
   if (fId!=0xFFFF) { // maybe the font name
-    int fontSz=static_cast<int>(input->readULong(1));
+    auto fontSz=static_cast<int>(input->readULong(1));
     if (!fontSz || input->tell()+fontSz >= entry.end())
       input->seek(-1,librevenge::RVNG_SEEK_CUR);
     else {
@@ -1971,7 +1988,7 @@ bool MoreText::readCustomListLevel(MWAWEntry const &entry, MWAWListLevel &level)
     }
   }
 
-  int labelSz = static_cast<int>(input->readULong(1));
+  auto labelSz = static_cast<int>(input->readULong(1));
   if (input->tell()+labelSz != entry.end())
     return false;
 
@@ -1979,7 +1996,7 @@ bool MoreText::readCustomListLevel(MWAWEntry const &entry, MWAWListLevel &level)
   if (fId == 0xFFFF)
     fId=3;
   for (int c=0; c < labelSz; c++) {
-    unsigned char ch=static_cast<unsigned char>(input->readULong(1));
+    auto ch=static_cast<unsigned char>(input->readULong(1));
     f << ch;
     int unicode = m_parserState->m_fontConverter->unicode(fId, static_cast<unsigned char>(ch));
     if (unicode!=-1)
@@ -2012,10 +2029,10 @@ bool MoreText::readTabs(MWAWEntry const &entry, MoreTextInternal::Paragraph &par
   long pos = entry.begin();
   input->seek(pos, librevenge::RVNG_SEEK_SET);
 
-  int nTabs=static_cast<int>(input->readULong(2));
+  auto nTabs=static_cast<int>(input->readULong(2));
   if (entry.length()!=4+4*nTabs)
     return false;
-  int repeat=static_cast<int>(input->readLong(2));
+  auto repeat=static_cast<int>(input->readLong(2));
   if (uint16_t(repeat)==0x8000) // special case
     f << "def[center,right],";
   else
@@ -2025,7 +2042,7 @@ bool MoreText::readTabs(MWAWEntry const &entry, MoreTextInternal::Paragraph &par
     libmwaw::DebugStream f2;
     MWAWTabStop tab;
     tab.m_position = double(input->readULong(2))/1440.;
-    int val=static_cast<int>(input->readULong(1));
+    auto val=static_cast<int>(input->readULong(1));
     switch (val&0xF) {
     case 1: // left
       break;
@@ -2050,6 +2067,7 @@ bool MoreText::readTabs(MWAWEntry const &entry, MoreTextInternal::Paragraph &par
       break;
     case 3: // more large space
       f2 << "dot[large],";
+      MWAW_FALLTHROUGH;
     case 2:
       tab.m_leaderCharacter = '.';
       break;
@@ -2057,7 +2075,7 @@ bool MoreText::readTabs(MWAWEntry const &entry, MoreTextInternal::Paragraph &par
       f2 << "#leader=" << (val>>4) << ",";
       break;
     }
-    char decimalChar = char(input->readULong(1));
+    auto decimalChar = char(input->readULong(1));
     if (decimalChar) {
       int unicode= m_parserState->m_fontConverter->unicode(3, static_cast<unsigned char>(decimalChar));
       if (unicode==-1)
