@@ -101,8 +101,12 @@ struct PLC {
    * - T_STRUCT: a structured type ( which unknown size) */
   typedef enum { T_CST=0, T_STRUCT, T_UNKNOWN} Type;
   //! constructor
-  PLC(PLCType w= MsWks4TextInternal::Unknown, Position p=P_UNKNOWN, Type t=T_UNKNOWN) :
-    m_type(w), m_pos(p), m_contentType(t) {}
+  PLC(PLCType w= MsWks4TextInternal::Unknown, Position p=P_UNKNOWN, Type t=T_UNKNOWN)
+    : m_type(w)
+    , m_pos(p)
+    , m_contentType(t)
+  {
+  }
   //! PLC type
   PLCType m_type;
   //! the way to define the text positions
@@ -115,7 +119,8 @@ struct PLC {
 struct KnownPLC {
 public:
   //! creates the mapping
-  KnownPLC() : m_knowns()
+  KnownPLC()
+    : m_knowns()
   {
     createMapping();
   }
@@ -123,7 +128,7 @@ public:
   //! returns the PLC corresponding to a name
   PLC get(std::string const &name)
   {
-    std::map<std::string, PLC>::iterator pos = m_knowns.find(name);
+    auto pos = m_knowns.find(name);
     if (pos == m_knowns.end()) return PLC();
     return pos->second;
   }
@@ -163,7 +168,12 @@ class FontName
 {
 public:
   //! constructor
-  FontName() : m_name(""), m_id(-1), m_unknown(0) { }
+  FontName()
+    : m_name("")
+    , m_id(-1)
+    , m_unknown(0)
+  {
+  }
 
   //! operator<<
   friend std::ostream &operator<<(std::ostream &o, FontName const &ft);
@@ -189,9 +199,19 @@ struct Font {
   //! the field type: pagenumber, note, DTT=date/time/type
   enum FieldType { None, Page, Eobj, Note, DTT, DTTUnk};
   //! constructor: default value Geneva:12
-  Font(int fId=3, int size=12) :m_font(fId,float(size)), m_fieldType(None), m_error("") {}
+  explicit Font(int fId=3, int size=12)
+    : m_font(fId,float(size))
+    , m_fieldType(None)
+    , m_error("")
+  {
+  }
   //! copy constructor
-  explicit Font(MWAWFont const &ft) :m_font(ft), m_fieldType(None), m_error("") {}
+  explicit Font(MWAWFont const &ft)
+    : m_font(ft)
+    , m_fieldType(None)
+    , m_error("")
+  {
+  }
   //! operator <<
   friend std::ostream &operator<<(std::ostream &o, Font const &ft);
   //! the font
@@ -231,13 +251,15 @@ std::ostream &operator<<(std::ostream &o, Font const &ft)
 }
 
 /** Internal: class to store a paragraph properties */
-struct Paragraph : public MWAWParagraph {
+struct Paragraph final : public MWAWParagraph {
   //! constructor
-  Paragraph(): MWAWParagraph(), m_pageBreak(false)
+  Paragraph()
+    : MWAWParagraph()
+    , m_pageBreak(false)
   {
   }
   //! destructor
-  virtual ~Paragraph();
+  ~Paragraph() final;
   //! operator <<
   friend std::ostream &operator<<(std::ostream &o, Paragraph const &ind);
 
@@ -259,7 +281,14 @@ Paragraph::~Paragraph()
 /** Internal: class to store footnote definition */
 struct Ftnt {
   //! constructor
-  Ftnt() : m_type(-1), m_id(-1), m_begin(-1), m_end(-1), m_error("") {}
+  Ftnt()
+    : m_type(-1)
+    , m_id(-1)
+    , m_begin(-1)
+    , m_end(-1)
+    , m_error("")
+  {
+  }
   //! operator<<
   friend std::ostream &operator<<(std::ostream &o, Ftnt const &ftnt);
   /** the note type
@@ -294,7 +323,13 @@ std::ostream &operator<<(std::ostream &o, Ftnt const &ftnt)
 /** Internal: class to store field definition: TOKN entry*/
 struct Token {
   //! constructor
-  Token() : m_type(MWAWField::None), m_textLength(-1), m_unknown(-1), m_error("") {}
+  Token()
+    : m_type(MWAWField::None)
+    , m_textLength(-1)
+    , m_unknown(-1)
+    , m_error("")
+  {
+  }
   //! operator <<
   friend std::ostream &operator<<(std::ostream &o, Token const &tok);
   //! the type
@@ -345,7 +380,14 @@ std::ostream &operator<<(std::ostream &o, Token const &tok)
 /** Internal: class to store field definition: TOKN entry*/
 struct Object {
   //! constructor
-  Object() : m_type(-1), m_id(-1), m_dim(), m_fileId(-1), m_error("") {}
+  Object()
+    : m_type(-1)
+    , m_id(-1)
+    , m_dim()
+    , m_fileId(-1)
+    , m_error("")
+  {
+  }
   //! operator <<
   friend std::ostream &operator<<(std::ostream &o, Object const &tok);
   //! the object type
@@ -373,7 +415,13 @@ std::ostream &operator<<(std::ostream &o, Object const &obj)
 /** Internal: class to store the PLC: Pointer List Content ? */
 struct DataPLC {
   //! constructor
-  DataPLC(): m_name(""), m_type(Unknown), m_value(-1), m_error("") {}
+  DataPLC()
+    : m_name("")
+    , m_type(Unknown)
+    , m_value(-1)
+    , m_error("")
+  {
+  }
   //! operator<<
   friend std::ostream &operator<<(std::ostream &o, DataPLC const &plc);
   //! the entry name
@@ -402,9 +450,21 @@ std::ostream &operator<<(std::ostream &o, DataPLC const &plc)
  */
 struct State {
   //! constructor
-  State() : m_paragraph(), m_defFont(3,12), m_fontNames(),
-    m_fontList(), m_paragraphList(), m_pgdList(), m_ftntList(),
-    m_ftntMap(), m_eobjMap(), m_plcList(), m_knownPLC(), m_main(false) {}
+  State()
+    : m_paragraph()
+    , m_defFont(3,12)
+    , m_fontNames()
+    , m_fontList()
+    , m_paragraphList()
+    , m_pgdList()
+    , m_ftntList()
+    , m_ftntMap()
+    , m_eobjMap()
+    , m_plcList()
+    , m_knownPLC()
+    , m_main(false)
+  {
+  }
 
   //! returns true if we parse the main block
   bool parseMain() const
@@ -462,16 +522,23 @@ protected:
 ////////////////////////////////////////////////////////////
 // constructor/destructor
 ////////////////////////////////////////////////////////////
-MsWks4Text::MsWks4Text(MsWksDocument &document) :
-  m_mainParser(&document.getMainParser()), m_parserState(), m_document(document), m_textPositions(),
-  m_state(), m_FODsList(), m_FDPCs(), m_FDPPs()
+MsWks4Text::MsWks4Text(MsWksDocument &document)
+  : m_mainParser(&document.getMainParser())
+  , m_parserState()
+  , m_document(document)
+  , m_textPositions()
+  , m_state()
+  , m_FODsList()
+  , m_FDPCs()
+  , m_FDPPs()
 {
   m_parserState=m_mainParser->getParserState();
   m_state.reset(new MsWks4TextInternal::State);
 }
 
 MsWks4Text::~MsWks4Text()
-{ }
+{
+}
 
 ////////////////////////////////////////////////////////////
 // number of page
@@ -479,11 +546,10 @@ MsWks4Text::~MsWks4Text()
 int MsWks4Text::numPages() const
 {
   int res = 1;
-  size_t nbFODS = m_FODsList.size();
-  for (size_t i = 0; i < nbFODS; i++) {
-    if (m_FODsList[i].m_type != DataFOD::ATTR_PARAG) continue;
+  for (auto const &fod : m_FODsList) {
+    if (fod.m_type != DataFOD::ATTR_PARAG) continue;
 
-    int id = m_FODsList[i].m_id;
+    int id = fod.m_id;
     if (id < 0) continue;
     if (m_state->m_paragraphList[size_t(id)].m_pageBreak) res++;
   }
@@ -514,11 +580,10 @@ bool MsWks4Text::readStructures(MWAWInputStreamPtr input, bool mainOle)
   m_state->m_ftntMap.clear();
   m_state->m_pgdList.resize(0);
 
-  std::multimap<std::string, MWAWEntry> &entryMap = m_document.getEntryMap();
-  std::multimap<std::string, MWAWEntry>::iterator pos;
+  auto &entryMap = m_document.getEntryMap();
 
   /* the text limit */
-  pos = entryMap.find("TEXT");
+  auto pos = entryMap.find("TEXT");
   if (entryMap.end() == pos) {
     MWAW_DEBUG_MSG(("MsWks4Text::readStructures: error: no TEXT in header index table\n"));
     throw libmwaw::ParseException();
@@ -542,17 +607,15 @@ bool MsWks4Text::readStructures(MWAWInputStreamPtr input, bool mainOle)
   }
 
   /* read character FODs (FOrmatting Descriptors) */
-  size_t numFDP = m_FDPCs.size();
   std::vector<DataFOD> fdps;
 
-  for (size_t i = 0; i < numFDP; i++)
-    readFDP(input, *(m_FDPCs[i]), fdps, FDPParser(&MsWks4Text::readFont));
+  for (auto fdp : m_FDPCs)
+    readFDP(input, *fdp, fdps, FDPParser(&MsWks4Text::readFont));
   m_FODsList = mergeSortedLists(fdps, m_FODsList);
 
   fdps.resize(0);
-  numFDP = m_FDPPs.size();
-  for (size_t i = 0; i < numFDP; i++)
-    readFDP(input, *(m_FDPPs[i]), fdps, FDPParser(&MsWks4Text::readParagraph));
+  for (auto fdp : m_FDPPs)
+    readFDP(input, *fdp, fdps, FDPParser(&MsWks4Text::readParagraph));
   m_FODsList = mergeSortedLists(fdps, m_FODsList);
 
   //
@@ -619,7 +682,7 @@ bool MsWks4Text::readFootNote(MWAWInputStreamPtr input, int id)
     listener->insertChar(' ');
     return false;
   }
-  MsWks4TextInternal::Ftnt const &ft = m_state->m_ftntList[size_t(id)];
+  auto const &ft = m_state->m_ftntList[size_t(id)];
   if (ft.m_begin < m_textPositions.begin() || ft.m_end > m_textPositions.end()) {
     MWAW_DEBUG_MSG(("MsWks4Text::readFootNote: invalid zone\n"));
     listener->insertChar(' ');
@@ -655,9 +718,9 @@ bool MsWks4Text::readText(MWAWInputStreamPtr input,  MWAWEntry const &zone,
 
   m_state->setParse(mainOle);
 
-  std::vector<DataFOD>::iterator FODs_iter = m_FODsList.begin();
   // update the property to correspond to the text
   int prevFId = -1, prevPId = -1;
+  auto FODs_iter = m_FODsList.begin();
   for (; FODs_iter!= m_FODsList.end(); ++FODs_iter) {
     DataFOD const &fod = *(FODs_iter);
     if (fod.m_pos >= zone.begin()) break;
@@ -677,7 +740,7 @@ bool MsWks4Text::readText(MWAWInputStreamPtr input,  MWAWEntry const &zone,
   else setProperty(MsWks4TextInternal::Paragraph());
 
   bool first = true;
-  MsWks4TextInternal::Font::FieldType fType = MsWks4TextInternal::Font::None;
+  auto fType = MsWks4TextInternal::Font::None;
   bool pageBreak = false;
   int page=1;
   libmwaw::DebugFile &ascFile = m_document.ascii();
@@ -694,7 +757,7 @@ bool MsWks4Text::readText(MWAWInputStreamPtr input,  MWAWEntry const &zone,
     if (++FODs_iter!= m_FODsList.end()) lastPos = uint32_t((*FODs_iter).m_pos);
     else lastPos = uint32_t(zone.end());
     --FODs_iter;
-    int len = int(lastPos-actPos);
+    auto len = int(lastPos-actPos);
 
     if (fod.m_type == DataFOD::ATTR_TEXT) {
 #if DEBUG_FP
@@ -750,7 +813,7 @@ bool MsWks4Text::readText(MWAWInputStreamPtr input,  MWAWEntry const &zone,
         if (m_state->m_plcList[size_t(fod.m_id)].m_type == MsWks4TextInternal::PGD)
           pageBreak = true;
         else if (m_state->m_plcList[size_t(fod.m_id)].m_type == MsWks4TextInternal::EOBJ) {
-          std::map<long, MsWks4TextInternal::Object>::const_iterator eobjIt=m_state->m_eobjMap.find(long(actPos));
+          auto eobjIt=m_state->m_eobjMap.find(long(actPos));
           if (eobjIt == m_state->m_eobjMap.end()) {
             MWAW_DEBUG_MSG(("MsWks4Text::readText: can not find object\n"));
           }
@@ -777,8 +840,8 @@ bool MsWks4Text::readText(MWAWInputStreamPtr input,  MWAWEntry const &zone,
       }
     }
 
-    for (uint32_t i = uint32_t(len); i>0; i--) {
-      uint8_t readVal = static_cast<uint8_t>(input->readULong(1));
+    for (auto i = uint32_t(len); i>0; i--) {
+      auto readVal = static_cast<uint8_t>(input->readULong(1));
       s += char(readVal);
       if (isObject) {
         isObject = false;
@@ -817,7 +880,7 @@ bool MsWks4Text::readText(MWAWInputStreamPtr input,  MWAWEntry const &zone,
       }
       default: {
         if (!listener) break;
-        uint32_t extra = static_cast<uint32_t>(listener->insertCharacter(static_cast<unsigned char>(readVal), input, input->tell()+long(i)-1));
+        auto extra = static_cast<uint32_t>(listener->insertCharacter(static_cast<unsigned char>(readVal), input, input->tell()+long(i)-1));
         if (extra > i-1) {
           MWAW_DEBUG_MSG(("MsWks4Text::readText: warning: extra is too large\n"));
           input->seek(-long(extra+1-i), librevenge::RVNG_SEEK_CUR);
@@ -860,34 +923,35 @@ bool MsWks4Text::readPLC(MWAWInputStreamPtr input,
     return false;
   }
 
-  uint32_t page_offset = static_cast<uint32_t>(entry.begin());
+  auto page_offset = static_cast<uint32_t>(entry.begin());
   long length = entry.length();
   long endPos = entry.end();
 
   input->seek(long(page_offset), librevenge::RVNG_SEEK_SET);
-  uint16_t nPLC = static_cast<uint16_t>(input->readULong(2)); /* number of PLC */
+  auto nPLC = static_cast<uint16_t>(input->readULong(2)); /* number of PLC */
   if (4*nPLC+10 > length) {
     MWAW_DEBUG_MSG(("MsWks4Text::readPLC: error: nPLC = %i, pSize=%ld\n",
                     nPLC, length));
     return false;
   }
-  int dataSz = static_cast<int>(input->readULong(2));
+  auto dataSz = static_cast<int>(input->readULong(2));
   bool ok = true;
 
   libmwaw::DebugFile &ascFile = m_document.ascii();
   libmwaw::DebugStream f, f2;
 
-  MsWks4PLCInternal::PLC plcType = m_state->m_knownPLC.get(entry.name());
+  auto plcType = m_state->m_knownPLC.get(entry.name());
   if (plcType.m_type == MsWks4TextInternal::Unknown) {
     MWAW_DEBUG_MSG(("MsWks4Text::readPLC: unknown PLC name=%s, type=%s\n",
                     entry.name().c_str(), entry.type().c_str()));
   }
 
-  long nPLC_ = static_cast<long>(static_cast<unsigned long>(nPLC));
+  auto nPLC_ = static_cast<long>(static_cast<unsigned long>(nPLC));
   if (4*nPLC_+10 + dataSz *nPLC_ != length || length/(4+dataSz) < nPLC_) {
     MWAW_DEBUG_MSG(("MsWks4Text::readPLC: error: odd dataSize: nPLC = %i, pSize=%ld\n",
                     nPLC, length));
-    dataSz=0;
+    if (length/(4+dataSz) < nPLC_)
+      dataSz=0;
     ok = false;
   }
 
@@ -899,7 +963,7 @@ bool MsWks4Text::readPLC(MWAWInputStreamPtr input,
   long lastPtr =m_textPositions.begin();
   f << ",pos = (";
   for (int i = 0; i <= nPLC; i++) {
-    long pos = long(input->readULong(4));
+    auto pos = long(input->readULong(4));
     switch (plcType.m_pos) {
     case MsWks4PLCInternal::PLC::P_ABS:
       if (pos == 0) pos = m_textPositions.begin();
@@ -934,7 +998,7 @@ bool MsWks4Text::readPLC(MWAWInputStreamPtr input,
 
     DataFOD fod;
     fod.m_type = DataFOD::ATTR_PLC;
-    fod.m_pos = ok ? pos : 0;
+    fod.m_pos = posOk ? pos : 0;
 
     textPtrs.push_back(fod.m_pos);
     if (i != nPLC) fods.push_back(fod);
@@ -949,7 +1013,7 @@ bool MsWks4Text::readPLC(MWAWInputStreamPtr input,
     MsWks4TextInternal::DataPLC plc;
     plc.m_type = plcType.m_type;
     plc.m_name = entry.name();
-    bool printPLC = true;
+    bool printPLC = true, dataOk=true;
 
     switch (plcType.m_contentType) {
     case MsWks4PLCInternal::PLC::T_CST : {
@@ -979,7 +1043,7 @@ bool MsWks4Text::readPLC(MWAWInputStreamPtr input,
         std::string mess;
         if (!(this->*parser)
             (input, dataSz+pos, textPtrs[i], textPtrs[i+1], static_cast<int>(i), mess)) {
-          ok = false;
+          dataOk = false;
           break;
         }
         plc.m_error = mess;
@@ -994,12 +1058,12 @@ bool MsWks4Text::readPLC(MWAWInputStreamPtr input,
 #if !defined(__clang__)
     default:
 #endif
-      ok = false;
+      dataOk = false;
       break;
     }
 
     fods[i].m_id = static_cast<int>(m_state->m_plcList.size());
-    if (ok) fods[i].m_defPos = pos;
+    if (dataOk) fods[i].m_defPos = pos;
     m_state->m_plcList.push_back(plc);
 
     if (printPLC) {
@@ -1012,7 +1076,7 @@ bool MsWks4Text::readPLC(MWAWInputStreamPtr input,
     pos += dataSz;
     input->seek(pos, librevenge::RVNG_SEEK_SET);
 
-    if (!ok) plcType.m_contentType = MsWks4PLCInternal::PLC::T_UNKNOWN;
+    if (!dataOk) plcType.m_contentType = MsWks4PLCInternal::PLC::T_UNKNOWN;
   }
 
   if (fods.size())
@@ -1036,7 +1100,7 @@ bool MsWks4Text::readPLC(MWAWInputStreamPtr input,
 
 // default
 bool MsWks4Text::defDataParser(MWAWInputStreamPtr input, long endPos,
-                               long , long , int , std::string &mess)
+                               long, long, int, std::string &mess)
 {
   mess = "";
   libmwaw::DebugStream f;
@@ -1044,7 +1108,7 @@ bool MsWks4Text::defDataParser(MWAWInputStreamPtr input, long endPos,
   long actPos = input->tell();
   long length = endPos-actPos;
   int sz = length%4==0 ? 4 : length%2==0 ? 2 : 1;
-  int nbElt = int(length/sz);
+  auto nbElt = int(length/sz);
 
   f << "[" << sz << "]{" << std::hex;
   for (int c = 0; c < nbElt; c++) f << input->readULong(sz) << ",";
@@ -1066,8 +1130,8 @@ bool MsWks4Text::readFontNames(MWAWInputStreamPtr input, MWAWEntry const &entry)
   entry.setParsed(true);
 
   input->seek(debPos, librevenge::RVNG_SEEK_SET);
-  uint32_t len = static_cast<uint32_t>(input->readULong(2));
-  uint32_t n_fonts = static_cast<uint32_t>(input->readULong(2));
+  auto len = static_cast<uint32_t>(input->readULong(2));
+  auto n_fonts = static_cast<uint32_t>(input->readULong(2));
   libmwaw::DebugFile &ascFile = m_document.ascii();
   libmwaw::DebugStream f;
 
@@ -1094,7 +1158,7 @@ bool MsWks4Text::readFontNames(MWAWInputStreamPtr input, MWAWEntry const &entry)
   while (input->tell() > 0 && long(input->tell()+8) < endPos
          && m_state->m_fontNames.size() < n_fonts) {
     debPos = input->tell();
-    uint16_t string_size = static_cast<uint16_t>(input->readULong(1));
+    auto string_size = static_cast<uint16_t>(input->readULong(1));
 
     std::string s;
     for (; string_size>0; string_size--) s.append(1, char(input->readULong(1)));
@@ -1152,12 +1216,12 @@ bool MsWks4Text::readFont(MWAWInputStreamPtr &input, long endPos,
   while (input->tell() <= endPos-2) {
     bool ok = true;
 
-    int val = static_cast<int>(input->readLong(1));
+    auto val = static_cast<int>(input->readLong(1));
     long pos = input->tell();
 
     switch (val) {
     case 0x3: {
-      int v = static_cast<int>(input->readLong(1));
+      auto v = static_cast<int>(input->readLong(1));
       switch (v) {
       case 1:
         font.m_fieldType = MsWks4TextInternal::Font::Page;
@@ -1176,7 +1240,7 @@ bool MsWks4Text::readFont(MWAWInputStreamPtr &input, long endPos,
     }
     case 0x12: { // date or title
       font.m_fieldType = MsWks4TextInternal::Font::DTT;
-      int unkn = static_cast<int>(input->readLong(1));
+      auto unkn = static_cast<int>(input->readLong(1));
       if (unkn) {
         font.m_fieldType = MsWks4TextInternal::Font::DTTUnk;
         f << "#DTT=" << unkn;
@@ -1188,57 +1252,57 @@ bool MsWks4Text::readFont(MWAWInputStreamPtr &input, long endPos,
       break;
     }
     case 0x4: {
-      int v = static_cast<int>(input->readLong(1));
+      auto v = static_cast<int>(input->readLong(1));
       if (v != 1) f << "##bold=" << v << ",";
       textAttributeBits |= MWAWFont::boldBit;
       break;
     }
     case 0x5: {
-      int v = static_cast<int>(input->readLong(1));
+      auto v = static_cast<int>(input->readLong(1));
       if (v != 1) f << "##it=" << v << ",";
       textAttributeBits |= MWAWFont::italicBit;
       break;
     }
     case 0x6: {
-      int v = static_cast<int>(input->readLong(1));
+      auto v = static_cast<int>(input->readLong(1));
       if (v != 1) f << "##under=" << v << ",";
       font.m_font.setUnderlineStyle(MWAWFont::Line::Simple);
       break;
     }
     case 0x7: {
-      int v = static_cast<int>(input->readLong(1));
+      auto v = static_cast<int>(input->readLong(1));
       if (v != 1) f << "##emboss=" << v << ",";
       textAttributeBits |= MWAWFont::embossBit;
       break;
     }
     case 0x8: {
-      int v = static_cast<int>(input->readLong(1));
+      auto v = static_cast<int>(input->readLong(1));
       if (v != 1) f << "##shadow=" << v << ",";
       textAttributeBits |= MWAWFont::shadowBit;
       break;
     }
     case 0x9: { // small interval between char
       font.m_font.setDeltaLetterSpacing(-1);
-      int v = static_cast<int>(input->readLong(1));
+      auto v = static_cast<int>(input->readLong(1));
       if (v != 1) f << "##Condensed=" << v << ",";
       else f << "Cond,";
       break;
     }
     case 0xa: { // big interval between char
       font.m_font.setDeltaLetterSpacing(1);
-      int v = static_cast<int>(input->readLong(1));
+      auto v = static_cast<int>(input->readLong(1));
       if (v != 1) f << "##Expand=" << v << ",";
       else f << "Expand,";
       break;
     }
     case 0x18: {
-      int v = static_cast<int>(input->readLong(1));
+      auto v = static_cast<int>(input->readLong(1));
       if (v != 1) f << "##super=" << v << ",";
       font.m_font.set(MWAWFont::Script::super100());
       break;
     }
     case 0x19: {
-      int v = static_cast<int>(input->readLong(1));
+      auto v = static_cast<int>(input->readLong(1));
       if (v != 1) f << "##sub =" << v << ",";
       font.m_font.set(MWAWFont::Script::sub100());
     }
@@ -1249,7 +1313,7 @@ bool MsWks4Text::readFont(MWAWInputStreamPtr &input, long endPos,
         ok = false;
         break;
       }
-      int v = static_cast<int>(input->readLong(2));
+      auto v = static_cast<int>(input->readLong(2));
 
       if (val == 0xe) { // fontId
         if (v < 0 || v >= int(m_state->m_fontNames.size())) {
@@ -1274,8 +1338,7 @@ bool MsWks4Text::readFont(MWAWInputStreamPtr &input, long endPos,
         break;
       }
 
-      for (int i = 0; i < 3; i++)
-        col[i] = static_cast<unsigned char>(input->readULong(2)>>8);
+      for (auto &c : col) c = static_cast<unsigned char>(input->readULong(2)>>8);
       break;
     }
     default:
@@ -1287,7 +1350,7 @@ bool MsWks4Text::readFont(MWAWInputStreamPtr &input, long endPos,
 
     input->seek(pos, librevenge::RVNG_SEEK_SET);
     // try to guess the attribute size
-    int v = static_cast<int>(input->readULong(1));
+    auto v = static_cast<int>(input->readULong(1));
     if (v == 1) ok = true;
     else if (pos+2 < endPos && v == 0 && input->readULong(1) == 1) ok = true;
 
@@ -1341,8 +1404,8 @@ bool MsWks4Text::readParagraph(MWAWInputStreamPtr &input, long endPos,
   bool customSpacing = false;
 
   while (input->tell() <  endPos) {
-    int val = static_cast<int>(input->readLong(1));
-    long pos = long(input->tell());
+    auto val = static_cast<int>(input->readLong(1));
+    auto pos = long(input->tell());
 
     bool ok = true;
     switch (val) {
@@ -1351,7 +1414,7 @@ bool MsWks4Text::readParagraph(MWAWInputStreamPtr &input, long endPos,
         ok = false;
         break;
       }
-      int value = static_cast<int>(input->readLong(1));
+      auto value = static_cast<int>(input->readLong(1));
       switch (value) {
       case 0:
         parag.m_justify = MWAWParagraph::JustificationLeft;
@@ -1377,7 +1440,7 @@ bool MsWks4Text::readParagraph(MWAWInputStreamPtr &input, long endPos,
         ok = false;
         break;
       }
-      int value= static_cast<int>(input->readLong(1));
+      auto value= static_cast<int>(input->readLong(1));
       if (value == 1) parag.m_pageBreak = true;
       else f << "#pgBreak="<<value<<",";
       break;
@@ -1387,7 +1450,7 @@ bool MsWks4Text::readParagraph(MWAWInputStreamPtr &input, long endPos,
         ok = false;
         break;
       }
-      int value= static_cast<int>(input->readLong(1));
+      auto value= static_cast<int>(input->readLong(1));
       if (value != 1) {
         f << "#bullet="<<value<<",";
         break;
@@ -1396,8 +1459,7 @@ bool MsWks4Text::readParagraph(MWAWInputStreamPtr &input, long endPos,
       level.m_type = MWAWListLevel::BULLET;
       level.m_labelWidth = 0.1;
       libmwaw::appendUnicode(0x2022, level.m_bullet);
-      shared_ptr<MWAWList> list =
-        m_parserState->m_listManager->getNewList(shared_ptr<MWAWList>(), 1, level);
+      auto list = m_parserState->m_listManager->getNewList(std::shared_ptr<MWAWList>(), 1, level);
       if (!list) {
         f << "#bullet,";
         MWAW_DEBUG_MSG(("MsWks4Text::readParagraph: can not create bullet list\n"));
@@ -1412,7 +1474,7 @@ bool MsWks4Text::readParagraph(MWAWInputStreamPtr &input, long endPos,
         ok = false;
         break;
       }
-      int value = static_cast<int>(input->readLong(2));
+      auto value = static_cast<int>(input->readLong(2));
       if (value > 100) {
         ok = false;
         break;
@@ -1446,7 +1508,7 @@ bool MsWks4Text::readParagraph(MWAWInputStreamPtr &input, long endPos,
         ok = false;
         break;
       }
-      int v = static_cast<int>(input->readLong(2));
+      auto v = static_cast<int>(input->readLong(2));
       if (v < -300 || v > 1000) {
         ok = false;
         break;
@@ -1463,17 +1525,17 @@ bool MsWks4Text::readParagraph(MWAWInputStreamPtr &input, long endPos,
         ok = false;
         break;
       }
-      int nbt = static_cast<int>(input->readLong(2));
+      auto nbt = static_cast<int>(input->readLong(2));
       if (nbt < 0 || nbt > 20) {
         ok = false;
         break;
       }
 
       parag.m_tabs->resize(size_t(nbt));
-      for (size_t i = 0; i < size_t(nbt); i++) {
-        int value = static_cast<int>(input->readULong(2));
+      for (auto &tab : *parag.m_tabs) {
+        auto value = static_cast<int>(input->readULong(2));
         int flag = (value & 0xc000) >> 14;
-        enum MWAWTabStop::Alignment align = MWAWTabStop::LEFT;
+        auto align = MWAWTabStop::LEFT;
         switch (flag) {
         case 0:
           align = MWAWTabStop::LEFT;
@@ -1492,8 +1554,8 @@ bool MsWks4Text::readParagraph(MWAWInputStreamPtr &input, long endPos,
         }
         value &= 0x3fff;
 
-        (*parag.m_tabs)[i].m_alignment = align;
-        (*parag.m_tabs)[i].m_position = value/72.0;
+        tab.m_alignment = align;
+        tab.m_position = value/72.0;
       }
       input->seek(pos+42, librevenge::RVNG_SEEK_SET);
       break;
@@ -1503,7 +1565,7 @@ bool MsWks4Text::readParagraph(MWAWInputStreamPtr &input, long endPos,
         ok = false;
         break;
       }
-      int value= static_cast<int>(input->readLong(1));
+      auto value= static_cast<int>(input->readLong(1));
       if (value != 1)
         f << "#spacingType=" << value << ",";
       else if (customSpacing)
@@ -1545,7 +1607,7 @@ bool MsWks4Text::readParagraph(MWAWInputStreamPtr &input, long endPos,
 //
 ////////////////////////////////////////////////////////////
 bool MsWks4Text::eobjDataParser(MWAWInputStreamPtr input, long endPos,
-                                long bot, long , int id, std::string &mess)
+                                long bot, long, int id, std::string &mess)
 {
   mess = "";
   libmwaw::DebugStream f;
@@ -1599,7 +1661,7 @@ bool MsWks4Text::ftntDataParser(MWAWInputStreamPtr input, long endPos,
     ftnt.m_end = eot;
   }
   for (int i = 0; i < 4; i++) {
-    int val = static_cast<int>(input->readLong(2));
+    auto val = static_cast<int>(input->readLong(2));
     if (val) f << "unkn" << i << "=" << val << ",";
   }
   ftnt.m_error= f.str();
@@ -1620,7 +1682,7 @@ bool MsWks4Text::ftntDataParser(MWAWInputStreamPtr input, long endPos,
 // Fixme: only parsed.....
 ////////////////////////////////////////////////////////////
 bool MsWks4Text::pgdDataParser(MWAWInputStreamPtr input, long endPos,
-                               long , long, int , std::string &mess)
+                               long, long, int, std::string &mess)
 {
   mess = "";
   libmwaw::DebugStream f;
@@ -1633,7 +1695,7 @@ bool MsWks4Text::pgdDataParser(MWAWInputStreamPtr input, long endPos,
     return false;
   }
   f << "val=" << input->readULong(1); // 0, 1, 2, 0x12, 0x1a
-  int val = static_cast<int>(input->readLong(1));
+  auto val = static_cast<int>(input->readLong(1));
   if (val) f << ":" << val;
   mess = f.str();
   return true;
@@ -1660,7 +1722,7 @@ bool MsWks4Text::toknDataParser(MWAWInputStreamPtr input, long endPos,
     return true;
   }
   MsWks4TextInternal::Token tok;
-  int type = static_cast<int>(input->readLong(2));
+  auto type = static_cast<int>(input->readLong(2));
   int beginType = 0;
   switch (type) {
   case 1:
@@ -1696,7 +1758,7 @@ bool MsWks4Text::toknDataParser(MWAWInputStreamPtr input, long endPos,
   switch (beginType) {
   case 1: {
     tok.m_unknown = static_cast<int>(input->readULong(2));
-    int v = static_cast<int>(input->readLong(2));
+    auto v = static_cast<int>(input->readLong(2));
     if (v) f << std::hex << "###unkn0=" << v << "," << std::dec;
     break;
   }
@@ -1718,7 +1780,7 @@ bool MsWks4Text::toknDataParser(MWAWInputStreamPtr input, long endPos,
 
   // sometimes, followed by the bot or by val, 0, bot
   // sometines, no
-  int debDataPos = static_cast<int>(input->readLong(2));
+  auto debDataPos = static_cast<int>(input->readLong(2));
   if (m_textPositions.begin()+debDataPos == bot);
   else {
     MWAW_DEBUG_MSG(("MsWks4Text::toknDataParser: odd token\n"));
@@ -1744,14 +1806,13 @@ bool MsWks4Text::toknDataParser(MWAWInputStreamPtr input, long endPos,
 ////////////////////////////////////////////////////////////
 bool MsWks4Text::findFDPStructures(MWAWInputStreamPtr &input, int which)
 {
-  std::vector<MWAWEntry const *> &zones = which ? m_FDPCs : m_FDPPs;
+  auto &zones = which ? m_FDPCs : m_FDPPs;
   zones.resize(0);
 
   char const *indexName = which ? "BTEC" : "BTEP";
   char const *sIndexName = which ? "FDPC" : "FDPP";
 
-  std::multimap<std::string, MWAWEntry>::iterator pos =
-    m_document.getEntryMap().lower_bound(indexName);
+  auto pos = m_document.getEntryMap().lower_bound(indexName);
   std::vector<MWAWEntry const *> listIndexed;
   while (pos != m_document.getEntryMap().end()) {
     MWAWEntry const &entry = pos++->second;
@@ -1768,9 +1829,7 @@ bool MsWks4Text::findFDPStructures(MWAWInputStreamPtr &input, int which)
     bool ok = true;
     for (size_t j = 0; j < nFind-1-i; j++) {
       if (listIndexed[j]->id() <= listIndexed[j+1]->id()) continue;
-      MWAWEntry const *tmp = listIndexed[j];
-      listIndexed[j] = listIndexed[j+1];
-      listIndexed[j+1] = tmp;
+      std::swap(listIndexed[j],listIndexed[j+1]);
       ok = false;
     }
     if (ok) break;
@@ -1781,7 +1840,6 @@ bool MsWks4Text::findFDPStructures(MWAWInputStreamPtr &input, int which)
 
   // create a map offset -> entry
   std::map<long, MWAWEntry const *> offsetMap;
-  std::map<long, MWAWEntry const *>::iterator offsIt;
   pos = m_document.getEntryMap().lower_bound(sIndexName);
   while (pos != m_document.getEntryMap().end()) {
     MWAWEntry const &entry = pos++->second;
@@ -1790,22 +1848,19 @@ bool MsWks4Text::findFDPStructures(MWAWInputStreamPtr &input, int which)
                      (entry.begin(), &entry));
   }
 
-  for (size_t i = 0; i < nFind; i++) {
-    MWAWEntry const &entry = *(listIndexed[i]);
-
+  for (auto entry : listIndexed) {
     std::vector<long> textPtrs;
     std::vector<long> listValues;
 
-    if (!readSimplePLC(input, entry, textPtrs, listValues)) return false;
+    if (!readSimplePLC(input, *entry, textPtrs, listValues)) return false;
 
     size_t numV = listValues.size();
     if (textPtrs.size() != numV+1) return false;
 
-    for (size_t j = 0; j < numV; j++) {
-      long position = listValues[j];
+    for (auto position : listValues) {
       if (position <= 0) return false;
 
-      offsIt = offsetMap.find(position);
+      auto offsIt = offsetMap.find(position);
       if (offsIt == offsetMap.end()) return false;
 
       zones.push_back(offsIt->second);
@@ -1820,10 +1875,10 @@ bool MsWks4Text::findFDPStructuresByHand(MWAWInputStreamPtr &/*input*/, int whic
   char const *indexName = which ? "FDPC" : "FDPP";
   MWAW_DEBUG_MSG(("MsWks4Text::findFDPStructuresByHand: error: need to create %s list by hand \n", indexName));
 
-  std::vector<MWAWEntry const *> &zones = which ? m_FDPCs : m_FDPPs;
+  auto &zones = which ? m_FDPCs : m_FDPPs;
   zones.resize(0);
 
-  std::multimap<std::string, MWAWEntry>::iterator pos = m_document.getEntryMap().lower_bound(indexName);
+  auto pos = m_document.getEntryMap().lower_bound(indexName);
   while (pos != m_document.getEntryMap().end()) {
     MWAWEntry const &entry = pos++->second;
     if (!entry.hasName(indexName)) break;
@@ -1847,7 +1902,7 @@ bool MsWks4Text::readFDP(MWAWInputStreamPtr &input, MWAWEntry const &entry,
   }
 
   entry.setParsed();
-  uint32_t page_offset = static_cast<uint32_t>(entry.begin());
+  auto page_offset = static_cast<uint32_t>(entry.begin());
   long length = entry.length();
   long endPage = entry.end();
 
@@ -1863,7 +1918,7 @@ bool MsWks4Text::readFDP(MWAWInputStreamPtr &input, MWAWEntry const &entry,
   libmwaw::DebugFile &ascFile = m_document.ascii();
   libmwaw::DebugStream f;
   input->seek(long(page_offset), librevenge::RVNG_SEEK_SET);
-  uint16_t cfod = static_cast<uint16_t>(input->readULong(deplSize));
+  auto cfod = static_cast<uint16_t>(input->readULong(deplSize));
 
   f << "FDP: N="<<static_cast<int>(cfod);
   f << ", unk=" << input->readLong(2);
@@ -1873,7 +1928,7 @@ bool MsWks4Text::readFDP(MWAWInputStreamPtr &input, MWAWEntry const &entry,
     return false;
   }
 
-  int firstFod = static_cast<int>(fods.size());
+  auto firstFod = static_cast<int>(fods.size());
   long lastLimit = firstFod ? fods.back().m_pos : 0;
 
   long lastReadPos = 0L;
@@ -1920,7 +1975,7 @@ bool MsWks4Text::readFDP(MWAWInputStreamPtr &input, MWAWEntry const &entry,
      the FPROP is located. */
   f << ", Tpos:defP=(" << std::hex;
   for (fods_iter = fods.begin() + firstFod; fods_iter!= fods.end(); ++fods_iter) {
-    int depl = static_cast<int>(input->readULong(deplSize));
+    auto depl = static_cast<int>(input->readULong(deplSize));
     /* check size of bfprop  */
     if ((depl < headerSize+(4+deplSize)*cfod && depl > 0) ||
         long(page_offset)+depl  > endPage) {
@@ -1947,14 +2002,14 @@ bool MsWks4Text::readFDP(MWAWInputStreamPtr &input, MWAWEntry const &entry,
     long pos = (*fods_iter).m_defPos;
     if (pos == 0) continue;
 
-    std::map<long,int>::iterator it= mapPtr.find(pos);
+    auto it= mapPtr.find(pos);
     if (it != mapPtr.end()) {
       (*fods_iter).m_id = mapPtr[pos];
       continue;
     }
 
     input->seek(pos, librevenge::RVNG_SEEK_SET);
-    int szProp = static_cast<int>(input->readULong(1));
+    auto szProp = static_cast<int>(input->readULong(1));
     szProp++;
     if (szProp == 0) {
       MWAW_DEBUG_MSG(("MsWks4Text::readFDP: error: 0 == szProp at file offset 0x%lx\n", static_cast<unsigned long int>(input->tell()-1)));
