@@ -58,7 +58,8 @@ namespace PowerPoint1ParserInternal
 //! Internal: a ruler
 struct Ruler {
   //! constructor
-  Ruler() : m_tabs()
+  Ruler()
+    : m_tabs()
   {
   }
   //! the outline
@@ -66,10 +67,8 @@ struct Ruler {
     //! constructor
     Outline()
     {
-      for (int i=0; i<2; ++i) {
-        m_margins[i]=0;
-        m_interlines[i]=10;
-      }
+      for (auto &margin : m_margins) margin=0;
+      for (auto &interline : m_interlines) interline=0;
     }
     //! operator<<
     friend std::ostream &operator<<(std::ostream &o, Outline const &outline)
@@ -102,13 +101,20 @@ struct Scheme {
 //! Internal: a text zone of a PowerPoint1Parser
 struct TextZone {
   //! constructor
-  TextZone() : m_lineList(), m_schemeId(-1)
+  TextZone()
+    : m_lineList()
+    , m_schemeId(-1)
   {
   }
   //! small structure used to store a line of text and its format
   struct Line {
     //! constructor
-    Line() : m_text(), m_format(), m_ruler(), m_justify(MWAWParagraph::JustificationLeft), m_outlineLevel(0)
+    Line()
+      : m_text()
+      , m_format()
+      , m_ruler()
+      , m_justify(MWAWParagraph::JustificationLeft)
+      , m_outlineLevel(0)
     {
     }
     //! the text entry
@@ -125,8 +131,8 @@ struct TextZone {
   //! return true if the zone has no text
   bool empty() const
   {
-    for (size_t i=0; i<m_lineList.size(); ++i) {
-      if (m_lineList[i].m_text.valid()) return false;
+    for (auto const &line : m_lineList) {
+      if (line.m_text.valid()) return false;
     }
     return true;
   }
@@ -138,7 +144,14 @@ struct TextZone {
 //! Internal: a frame of a PowerPoint1Parser
 struct Frame {
   //! constructor
-  Frame() : m_type(-1), m_dimension(), m_cornerSize(0), m_style(), m_rulerId(-1), m_pictureId(-1), m_textId(-1,-1)
+  Frame()
+    : m_type(-1)
+    , m_dimension()
+    , m_cornerSize(0)
+    , m_style()
+    , m_rulerId(-1)
+    , m_pictureId(-1)
+    , m_textId(-1,-1)
   {
   }
   //! the type: 0:line, 1:rect, 2: textbox, ...
@@ -159,7 +172,9 @@ struct Frame {
 //! Internal: a slide of a PowerPoint1Parser
 struct Slide {
   //! constructor
-  Slide() : m_useMasterPage(true), m_schemeId(-1)
+  Slide()
+    : m_useMasterPage(true)
+    , m_schemeId(-1)
   {
   }
   //! the textzone: main's and note's zone
@@ -176,13 +191,23 @@ struct Slide {
 //! Internal: the state of a PowerPoint1Parser
 struct State {
   //! constructor
-  State() : m_isMacFile(true), m_unit(1), m_zoneListBegin(0), m_zonesList(), m_origin(0,0), m_rulersList(), m_idToSlideMap(), m_idToSchemeMap(), m_idToUserColorMap(), m_picturesIdList(), m_schemesIdList(),
-    m_badEntry()
+  State()
+    : m_isMacFile(true)
+    , m_unit(1)
+    , m_zoneListBegin(0)
+    , m_zonesList()
+    , m_origin(0,0)
+    , m_rulersList()
+    , m_idToSlideMap()
+    , m_idToSchemeMap()
+    , m_idToUserColorMap()
+    , m_picturesIdList()
+    , m_schemesIdList()
+    , m_badEntry()
   {
-    for (int i=0; i<2; ++i)
-      m_slideIds[i]=m_printInfoIds[i]=-1;
-    for (int i=0; i<10; ++i)
-      m_zoneIds[i]=-1;
+    for (auto &slideId : m_slideIds) slideId=-1;
+    for (auto &printInfoId : m_printInfoIds) printInfoId=-1;
+    for (auto &zoneId : m_zoneIds) zoneId=-1;
   }
   //! try to return a zone
   MWAWEntry const &getZoneEntry(int id) const
@@ -271,24 +296,36 @@ bool State::getPattern(int id, MWAWGraphicStyle::Pattern &pattern) const
 
 ////////////////////////////////////////
 //! Internal: the subdocument of a PowerPointParser
-class SubDocument : public MWAWSubDocument
+class SubDocument final : public MWAWSubDocument
 {
 public:
   //! constructor for text
-  SubDocument(PowerPoint1Parser &pars, MWAWInputStreamPtr input, TextZone const *textZone, MWAWVec2i const &tId, int rulerId) :
-    MWAWSubDocument(&pars, input, MWAWEntry()), m_slide(0), m_textZone(textZone), m_id(tId), m_rulerId(rulerId) {}
+  SubDocument(PowerPoint1Parser &pars, MWAWInputStreamPtr const &input, TextZone const *textZone, MWAWVec2i const &tId, int rulerId)
+    : MWAWSubDocument(&pars, input, MWAWEntry())
+    , m_slide(nullptr)
+    , m_textZone(textZone)
+    , m_id(tId)
+    , m_rulerId(rulerId)
+  {
+  }
   //! constructor for slide note
-  SubDocument(PowerPoint1Parser &pars, MWAWInputStreamPtr input, Slide const *slide) :
-    MWAWSubDocument(&pars, input, MWAWEntry()), m_slide(slide), m_textZone(0), m_id(), m_rulerId(-1) {}
+  SubDocument(PowerPoint1Parser &pars, MWAWInputStreamPtr const &input, Slide const *slide)
+    : MWAWSubDocument(&pars, input, MWAWEntry())
+    , m_slide(slide)
+    , m_textZone(nullptr)
+    , m_id()
+    , m_rulerId(-1)
+  {
+  }
 
   //! destructor
-  virtual ~SubDocument() {}
+  ~SubDocument() final {}
 
   //! operator!=
-  virtual bool operator!=(MWAWSubDocument const &doc) const
+  bool operator!=(MWAWSubDocument const &doc) const final
   {
     if (MWAWSubDocument::operator!=(doc)) return true;
-    SubDocument const *sDoc = dynamic_cast<SubDocument const *>(&doc);
+    auto const *sDoc = dynamic_cast<SubDocument const *>(&doc);
     if (!sDoc) return true;
     if (m_slide != sDoc->m_slide) return true;
     if (m_textZone != sDoc->m_textZone) return true;
@@ -296,14 +333,9 @@ public:
     if (m_rulerId != sDoc->m_rulerId) return true;
     return false;
   }
-  //! operator!==
-  virtual bool operator==(MWAWSubDocument const &doc) const
-  {
-    return !operator!=(doc);
-  }
 
   //! the parser function
-  void parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType type);
+  void parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType type) final;
 
 protected:
   //! the slide
@@ -316,8 +348,8 @@ protected:
   int m_rulerId;
 
 private:
-  SubDocument(SubDocument const &);
-  SubDocument &operator=(SubDocument const &);
+  SubDocument(SubDocument const &) = delete;
+  SubDocument &operator=(SubDocument const &) = delete;
 };
 
 void SubDocument::parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType)
@@ -326,7 +358,7 @@ void SubDocument::parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType)
     MWAW_DEBUG_MSG(("PowerPoint1ParserInternal::SubDocument::parse: no listener\n"));
     return;
   }
-  PowerPoint1Parser *parser=dynamic_cast<PowerPoint1Parser *>(m_parser);
+  auto *parser=dynamic_cast<PowerPoint1Parser *>(m_parser);
   if (!parser) {
     MWAW_DEBUG_MSG(("PowerPoint1ParserInternal::SubDocument::parse: no parser\n"));
     return;
@@ -350,8 +382,9 @@ void SubDocument::parse(MWAWListenerPtr &listener, libmwaw::SubDocumentType)
 ////////////////////////////////////////////////////////////
 // constructor/destructor, ...
 ////////////////////////////////////////////////////////////
-PowerPoint1Parser::PowerPoint1Parser(MWAWInputStreamPtr input, MWAWRSRCParserPtr rsrcParser, MWAWHeader *header) :
-  MWAWPresentationParser(input, rsrcParser, header), m_state(new PowerPoint1ParserInternal::State)
+PowerPoint1Parser::PowerPoint1Parser(MWAWInputStreamPtr const &input, MWAWRSRCParserPtr const &rsrcParser, MWAWHeader *header)
+  : MWAWPresentationParser(input, rsrcParser, header)
+  , m_state(new PowerPoint1ParserInternal::State)
 {
   setAsciiName("main-1");
 }
@@ -382,13 +415,13 @@ bool PowerPoint1Parser::getColor(int colorId, int schemeId, MWAWColor &color) co
 ////////////////////////////////////////////////////////////
 void PowerPoint1Parser::parse(librevenge::RVNGPresentationInterface *docInterface)
 {
-  if (!getInput().get() || !checkHeader(0L))  throw(libmwaw::ParseException());
+  if (!getInput().get() || !checkHeader(nullptr))  throw(libmwaw::ParseException());
   bool ok = true;
   try {
     // create the asciiFile
     ascii().setStream(getInput());
     ascii().open(asciiName());
-    checkHeader(0L);
+    checkHeader(nullptr);
     ok = createZones();
     if (ok) {
       createDocument(docInterface);
@@ -426,7 +459,7 @@ void PowerPoint1Parser::createDocument(librevenge::RVNGPresentationInterface *do
     MWAWPageSpan ps(getPageSpan());
     int sId=m_state->m_slidesIdList[0][i];
     if (m_state->m_idToSlideMap.find(sId)!=m_state->m_idToSlideMap.end()) {
-      PowerPoint1ParserInternal::Slide const &slide=m_state->m_idToSlideMap.find(sId)->second;
+      auto const &slide=m_state->m_idToSlideMap.find(sId)->second;
       if (slide.m_useMasterPage)
         ps.setMasterPageName(librevenge::RVNGString("Master"));
       MWAWColor backColor;
@@ -546,7 +579,7 @@ bool PowerPoint1Parser::readListZones(int &docInfoId)
   f << "Entries(ListZones):";
   // v3: N in 4, then 16+8*N (potential extra data)
   long pos=input->tell();
-  int N=int(input->readULong(2));
+  auto N=int(input->readULong(2));
   f << "N=" << N << ",";
   if (!input->checkPosition(m_state->m_zoneListBegin+N*8)) {
     MWAW_DEBUG_MSG(("PowerPoint1Parser::readListZones: the number of zones seems bad\n"));
@@ -555,9 +588,9 @@ bool PowerPoint1Parser::readListZones(int &docInfoId)
     ascii().addNote(f.str().c_str());
     return false;
   }
-  int val=int(input->readULong(2)); // always 4
+  auto val=int(input->readULong(2)); // always 4
   if (val!=4) f << "f0=" << val << ",";
-  long endPos=long(input->readULong(4));
+  auto endPos=long(input->readULong(4));
   if (!input->checkPosition(endPos) || input->checkPosition(endPos+1)) {
     MWAW_DEBUG_MSG(("PowerPoint1Parser::readListZones: the endPos seems bad\n"));
     f << "###endPos=" << std::hex << endPos << std::dec << ",";
@@ -579,7 +612,7 @@ bool PowerPoint1Parser::readListZones(int &docInfoId)
   std::set<long> posList;
   for (int i=0; i<N; ++i) {
     unsigned long length=input->readULong(4);
-    long begin=long(input->readULong(4));
+    auto begin=long(input->readULong(4));
     if (length&0x80000000) {
       f << "*";
       length&=0x7FFFFFFF;
@@ -609,7 +642,7 @@ bool PowerPoint1Parser::readListZones(int &docInfoId)
   for (size_t i=0; i<m_state->m_zonesList.size(); ++i) {
     MWAWEntry &zone=m_state->m_zonesList[size_t(i)];
     if (!zone.valid()) continue;
-    std::set<long>::const_iterator it=posList.find(zone.begin());
+    auto it=posList.find(zone.begin());
     bool ok=it!=posList.end();
     if (ok) {
       if (++it==posList.end() || *it!=zone.end())
@@ -658,8 +691,8 @@ void PowerPoint1Parser::sendSlides()
 
 void PowerPoint1Parser::checkForUnparsedZones()
 {
-  for (size_t i=0; i<m_state->m_picturesIdList.size(); ++i) {
-    MWAWEntry const &entry=m_state->getZoneEntry(m_state->m_picturesIdList[i]);
+  for (auto id : m_state->m_picturesIdList) {
+    MWAWEntry const &entry=m_state->getZoneEntry(id);
     if (!entry.valid() || entry.isParsed()) continue;
     static bool first=true;
     if (first) {
@@ -670,9 +703,7 @@ void PowerPoint1Parser::checkForUnparsedZones()
     readPicture(entry, picture);
   }
   // check if there remains some unparsed zone
-  size_t numZones=m_state->m_zonesList.size();
-  for (size_t i=0; i<numZones; ++i) {
-    MWAWEntry const &entry=m_state->m_zonesList[i];
+  for (auto const &entry : m_state->m_zonesList) {
     if (!entry.valid() || entry.isParsed()) continue;
     static bool first=true;
     if (first) {
@@ -709,7 +740,7 @@ bool PowerPoint1Parser::readFramesList(MWAWEntry const &entry, std::vector<Power
   f << "Entries(Frames)[Z" << entry.id() << "]:";
   ascii().addPos(pos);
   ascii().addNote(f.str().c_str());
-  size_t N=size_t(entry.length()/dataSz);
+  auto N=size_t(entry.length()/dataSz);
   frameList.resize(N);
   int tId=0;
   MWAWColor colors[4]= {MWAWColor::black(),MWAWColor::white(),MWAWColor::black(),MWAWColor::black()}; // frame, fill, shadow, pat2
@@ -719,7 +750,7 @@ bool PowerPoint1Parser::readFramesList(MWAWEntry const &entry, std::vector<Power
     f.str("");
     f << "Frames[F" << fr << "]:";
     int dim[4];
-    for (int i=0; i<4; ++i) dim[i]=int(input->readLong(2));
+    for (auto &d : dim) d=int(input->readLong(2));
     if (!isMacFile) {
       std::swap(dim[0],dim[1]);
       std::swap(dim[2],dim[3]);
@@ -802,12 +833,12 @@ bool PowerPoint1Parser::readFramesList(MWAWEntry const &entry, std::vector<Power
         if (val) f << "fl2=" << val << ",";
       }
       for (int i=0; i<4; ++i) { // frame, fill, shadow, pat2
-        int col=int(input->readULong(1));
+        auto col=int(input->readULong(1));
         int const(expected[])= { 1, 4, 2, 0};
         if (schemeId>=0 && !getColor(col, schemeId, colors[i])) f << "##col,";
         if (col!=expected[i]) f << "col" << i << "=" << col << ",";
       }
-      if (!isMacFile) std::swap<MWAWColor>(colors[1],colors[3]);
+      if (!isMacFile) std::swap(colors[1],colors[3]);
     }
     MWAWGraphicStyle &style=frame.m_style;
     val=int(input->readULong(1));
@@ -1021,7 +1052,7 @@ bool PowerPoint1Parser::readTextZone(MWAWEntry const &entry, PowerPoint1ParserIn
       zone.m_lineList.push_back(line);
       break;
     }
-    int sSz=int(input->readULong(2));
+    auto sSz=int(input->readULong(2));
     if (input->tell()+sSz+(isMacFile ? (sSz&1)+2:16)>endPos) {
       input->seek(pos, librevenge::RVNG_SEEK_SET);
       n--;
@@ -1057,7 +1088,7 @@ bool PowerPoint1Parser::readTextZone(MWAWEntry const &entry, PowerPoint1ParserIn
         val=int(input->readULong(2));
         if (val) f << "f" << i << "=" << val << ",";
       }
-      int nFonts=int(input->readULong(2));
+      auto nFonts=int(input->readULong(2));
       if (pos+nFonts*14+8>endPos) {
         input->seek(pos, librevenge::RVNG_SEEK_SET);
         n--;
@@ -1076,7 +1107,7 @@ bool PowerPoint1Parser::readTextZone(MWAWEntry const &entry, PowerPoint1ParserIn
         val=int(input->readULong(2));
         if (val) f << "f" << i << "=" << val << ",";
       }
-      int nRulers=int(input->readULong(2));
+      auto nRulers=int(input->readULong(2));
       if (pos+nRulers*6>endPos) {
         input->seek(pos, librevenge::RVNG_SEEK_SET);
         n--;
@@ -1121,7 +1152,7 @@ bool PowerPoint1Parser::readSlide(MWAWEntry const &entry, std::vector<int> &list
   entry.setParsed(true);
   listIds.push_back(entry.id());
   PowerPoint1ParserInternal::Slide bugSlide;
-  PowerPoint1ParserInternal::Slide *slide=&bugSlide;
+  auto *slide=&bugSlide;
   if (m_state->m_idToSlideMap.find(entry.id())==m_state->m_idToSlideMap.end()) {
     m_state->m_idToSlideMap[entry.id()]=PowerPoint1ParserInternal::Slide();
     slide=&m_state->m_idToSlideMap.find(entry.id())->second;
@@ -1133,7 +1164,7 @@ bool PowerPoint1Parser::readSlide(MWAWEntry const &entry, std::vector<int> &list
   input->seek(entry.begin(), librevenge::RVNG_SEEK_SET);
   libmwaw::DebugStream f;
   f << "Entries(Slide)[Z" << entry.id() << "]:";
-  int numZones=int(m_state->m_zonesList.size());
+  auto numZones=int(m_state->m_zonesList.size());
   long childIds[5]= {-1,-1,-1,-1,-1};
   long id=input->readLong(4);
   if (id>=0 && id<numZones) {
@@ -1294,8 +1325,7 @@ bool PowerPoint1Parser::readDocInfo(MWAWEntry const &entry)
   libmwaw::DebugStream f;
   f << "Entries(DocInfo)[Z" << entry.id() << "]:";
   int val;
-  int numZones=int(m_state->m_zonesList.size());
-  int dim[4];
+  auto numZones=int(m_state->m_zonesList.size());
   f << "unkn=[";
   for (int i=0; i<4; ++i) { // list of 0 or big number
     val=int(input->readLong(2));
@@ -1312,10 +1342,11 @@ bool PowerPoint1Parser::readDocInfo(MWAWEntry const &entry)
     val=int(input->readLong(1));
     if (val) f << "f" << i << "=" << val << ",";
   }
-  for (int i=0; i<4; ++i) dim[i]=int(input->readLong(2));
+  int dim[4];
+  for (auto &d : dim) d=int(input->readLong(2));
   f << "dim[screen]=" << MWAWBox2i(MWAWVec2i(dim[1],dim[0]),MWAWVec2i(dim[3],dim[2])) << ",";
   int pages[2];
-  for (int i=0; i<2; ++i) pages[i]=int(input->readLong(2));
+  for (auto &p : pages) p=int(input->readLong(2));
   f << "num[pages]=" << pages[0] << ",";
   if (pages[0]!=pages[1]) f << "act[page]=" << pages[1] << ",";
   for (int i=0; i<2; ++i) { // id1=0, id2=0|big number
@@ -1337,14 +1368,14 @@ bool PowerPoint1Parser::readDocInfo(MWAWEntry const &entry)
     val=int(input->readULong(!isMacFile ? 2 : 4));
     if (val) f << "id" << i+4 << "=" << std::hex << val << std::dec << ",";
   }
-  for (int i=0; i<4; ++i) dim[i]=int(input->readLong(2));
+  for (int &i : dim) i=int(input->readLong(2));
   if (!isMacFile) {
     std::swap(dim[0],dim[1]);
     std::swap(dim[2],dim[3]);
   }
   MWAWBox2i pageBox(MWAWVec2i(dim[1],dim[0]),MWAWVec2i(dim[3],dim[2]));
   f << "dim[page]=" << pageBox << ",";
-  for (int i=0; i<4; ++i) dim[i]=int(input->readLong(2));
+  for (int &i : dim) i=int(input->readLong(2));
   if (!isMacFile) {
     std::swap(dim[0],dim[1]);
     std::swap(dim[2],dim[3]);
@@ -1393,7 +1424,7 @@ bool PowerPoint1Parser::readDocInfo(MWAWEntry const &entry)
   f << "DocInfo-2:";
   val=int(input->readULong(!isMacFile ? 2 : 4));
   if (val) f << "id=" << std::hex << val << std::dec << ",";
-  for (int i=0; i<4; ++i) dim[i]=int(input->readLong(2));
+  for (auto &d : dim) d=int(input->readLong(2));
   f << "dim=" << MWAWBox2i(MWAWVec2i(dim[1],dim[0]),MWAWVec2i(dim[3],dim[2])) << ",";
   val=int(input->readLong(1)); // 0|1
   if (val) f << "f0=" << val << ",";
@@ -1409,7 +1440,7 @@ bool PowerPoint1Parser::readDocInfo(MWAWEntry const &entry)
     val=int(input->readLong(2));
     if (val) f << "f" << 3+i << "=" << val << ",";
   }
-  for (int i=0; i<4; ++i) dim[i]=int(input->readLong(2));
+  for (auto &d : dim) d=int(input->readLong(2));
   f << "dim2=" << MWAWBox2i(MWAWVec2i(dim[1],dim[0]),MWAWVec2i(dim[3],dim[2])) << ",";
   f << "unkn=[";
   for (int i=0; i<3; ++i) { // 3 small int
@@ -1447,7 +1478,7 @@ bool PowerPoint1Parser::readDocInfo(MWAWEntry const &entry)
     }
   }
   f << "],";
-  for (int i=0; i<4; ++i) dim[i]=int(input->readULong(2));
+  for (auto &d: dim) d=int(input->readULong(2));
   f << "page=" << MWAWVec2i(dim[0],dim[1]) << ",";
   f << "dim?=" << MWAWVec2i(dim[3],dim[2]) << ","; // frame, slide dim?
   for (int i=0; i<2; ++i) { // f0=1, f1=0|80
@@ -1494,20 +1525,20 @@ bool PowerPoint1Parser::readPictureDefinition(MWAWEntry const &entry, size_t pId
   input->seek(pos, librevenge::RVNG_SEEK_SET);
   libmwaw::DebugStream f;
   f << "Entries(Picture)[Z"<< entry.id() << "-" << pId << "]:def,";
-  int val=int(input->readULong(2)); // big number [0-3]XXX
+  auto val=int(input->readULong(2)); // big number [0-3]XXX
   if (val) f << "id=" << std::hex << val << std::dec << ",";
-  int type=int(input->readULong(2)); // 1-4
+  auto type=int(input->readULong(2)); // 1-4
   if (type)
     f << "type=" << type << ",";
   int dim[4];
-  for (int i=0; i<4; ++i) dim[i]=int(input->readLong(2));
+  for (auto &d : dim) d=int(input->readLong(2));
   f << "dim=" << MWAWBox2i(MWAWVec2i(dim[0],dim[1]),MWAWVec2i(dim[2],dim[3])) << ",";
   val=int(input->readULong(2));
   if (val!=2) {
     f << "###type2=" << val << ",";
     MWAW_DEBUG_MSG(("PowerPoint1Parser::readPictureDefinition: find unexpected type 2\n"));
   }
-  int child=int(input->readULong(2));
+  auto child=int(input->readULong(2));
   if (child>=0 && child<int(m_state->m_zonesList.size())) {
     f << "child[id]=Z" << child << ",";
     if (pId>=m_state->m_picturesIdList.size())
@@ -1593,8 +1624,8 @@ bool PowerPoint1Parser::readZoneIdList(MWAWEntry const &entry, int zId)
   f << "Entries(" << wh << ")[Z"<< entry.id() << "]:";
   ascii().addPos(pos);
   ascii().addNote(f.str().c_str());
-  size_t N=size_t(entry.length()/6);
-  int const numZones=int(m_state->m_zonesList.size());
+  auto N=size_t(entry.length()/6);
+  auto const numZones=int(m_state->m_zonesList.size());
   std::vector<int> unknownList;
   std::vector<int> &list=zId==0 ? m_state->m_picturesIdList : zId==3 ? m_state->m_schemesIdList : unknownList;
   list.resize(N, -1);
@@ -1607,8 +1638,8 @@ bool PowerPoint1Parser::readZoneIdList(MWAWEntry const &entry, int zId)
       f << "Scheme-S" << i << ":";
     else
       f << wh << "-" << i << ":";
-    int type=int(input->readULong(2));
-    int id=int(input->readLong(4));
+    auto type=int(input->readULong(2));
+    auto id=int(input->readLong(4));
     if (type==0 || id==-1) {
       f << "_,";
       ascii().addPos(pos);
@@ -1644,11 +1675,11 @@ bool PowerPoint1Parser::readZoneIdList2(MWAWEntry const &entry, int zId)
   libmwaw::DebugStream f;
   std::string const wh(zId==0 ? "Picture" : zId==1 ? "Ruler" : zId==2 ? "Scheme" : "UnknownList");
   f << "Entries(" << wh << ")[Z"<< entry.id() << "]:list,";
-  int val=int(input->readULong(2)); // 8001
+  auto val=int(input->readULong(2)); // 8001
   if (val!=0x8001) f << "f0=" << std::hex << val << std::dec << ",";
   val=int(input->readULong(2)); // big number
   if (val) f << "id=" << std::hex << val << std::dec << ",";
-  size_t N=size_t(input->readULong(2));
+  auto N=size_t(input->readULong(2));
   f << "N=" << N << ",";
   if (16+4*long(N)>entry.length()) {
     MWAW_DEBUG_MSG(("PowerPoint1Parser::readZoneIdList2: the N value seems bad\n"));
@@ -1667,7 +1698,7 @@ bool PowerPoint1Parser::readZoneIdList2(MWAWEntry const &entry, int zId)
   }
   ascii().addPos(pos);
   ascii().addNote(f.str().c_str());
-  int const numZones=int(m_state->m_zonesList.size());
+  auto const numZones=int(m_state->m_zonesList.size());
   std::vector<int> list;
   list.resize(N, -1);
   for (size_t i=0; i<N; ++i) {
@@ -1681,8 +1712,8 @@ bool PowerPoint1Parser::readZoneIdList2(MWAWEntry const &entry, int zId)
       f << "Scheme-S" << i << ":";
     else
       f << wh << "-" << i << ":";
-    int type=int(input->readULong(2));
-    int id=int(input->readLong(2));
+    auto type=int(input->readULong(2));
+    auto id=int(input->readLong(2));
     if (type==0 || id==-1) {
       f << "_,";
       ascii().addPos(pos);
@@ -1775,16 +1806,16 @@ bool PowerPoint1Parser::readRulers(MWAWEntry const &entry)
   ascii().addPos(pos);
   ascii().addNote(f.str().c_str());
 
-  size_t N=size_t(entry.length()/66);
+  auto N=size_t(entry.length()/66);
   m_state->m_rulersList.resize(N);
   for (size_t i=0; i<N; ++i) {
     pos=input->tell();
     f.str("");
     f << "Ruler-R" << i+1 << ":";
-    PowerPoint1ParserInternal::Ruler &ruler=m_state->m_rulersList[i];
-    int val=int(input->readULong(2)); // 1, 2, 5, 8, 1fff, 2000, 2001,
+    auto &ruler=m_state->m_rulersList[i];
+    auto val=int(input->readULong(2)); // 1, 2, 5, 8, 1fff, 2000, 2001,
     if (val) f << "f0=" << std::hex << val << std::dec << ",";
-    int nTabs=int(input->readULong(2));
+    auto nTabs=int(input->readULong(2));
     if (nTabs>10) {
       MWAW_DEBUG_MSG(("PowerPoint1Parser::readRulers: the number of tab seems bad\n"));
       f << "###n[tabs]=" << nTabs << ",";
@@ -1804,11 +1835,10 @@ bool PowerPoint1Parser::readRulers(MWAWEntry const &entry)
     }
     f << "],";
     f << "levels=[";
-    for (int j=0; j<5; ++j) {
-      PowerPoint1ParserInternal::Ruler::Outline &outline=ruler.m_outlines[j];
+    for (auto &outline : ruler.m_outlines) {
       f << "[";
-      for (int k=0; k<2; ++k) outline.m_margins[k]=int(input->readULong(2));
-      for (int k=0; k<2; ++k) outline.m_interlines[k]=10*int(input->readULong(1));
+      for (int &margin : outline.m_margins) margin=int(input->readULong(2));
+      for (int &interline : outline.m_interlines) interline=10*int(input->readULong(1));
       f << outline << ",";
       f << "fl=" << std::hex << input->readULong(2) << std::dec << ",";
       f << "],";
@@ -1840,26 +1870,25 @@ bool PowerPoint1Parser::readRuler(MWAWEntry const &entry, size_t id)
     m_state->m_rulersList.resize(id+1);
   PowerPoint1ParserInternal::Ruler &ruler=m_state->m_rulersList[id];
   f << "levels=[";
-  for (int j=0; j<5; ++j) {
-    PowerPoint1ParserInternal::Ruler::Outline &outline=ruler.m_outlines[j];
+  for (auto &outline : ruler.m_outlines) {
     f << "[";
-    for (int k=0; k<2; ++k) outline.m_margins[k]=int(input->readULong(2));
-    for (int k=0; k<2; ++k) outline.m_interlines[k]=int(input->readULong(2));
+    for (int &margin : outline.m_margins) margin=int(input->readULong(2));
+    for (int &interline : outline.m_interlines) interline=int(input->readULong(2));
     f << outline << ",";
     f << "fl=" << std::hex << input->readULong(2) << std::dec << ",";
     f << "],";
   }
   f << "],";
-  int val=int(input->readULong(2)); // 2-3: align?
+  auto val=int(input->readULong(2)); // 2-3: align?
   if (val!=3) f << "f0=" << val << ",";
-  int nTabs=int(input->readULong(2));
+  auto nTabs=int(input->readULong(2));
   if (input->tell()+4*nTabs>entry.end()) {
     MWAW_DEBUG_MSG(("PowerPoint1Parser::readRuler: the number of tab seems bad\n"));
     f << "###n[tabs]=" << nTabs << ",";
     nTabs=0;
   }
   f << "tabs=[";
-  for (int j=0, bit=1; j<nTabs; ++j, bit<<=1) {
+  for (int j=0; j<nTabs; ++j) {
     MWAWTabStop tab;
     tab.m_position=double(input->readULong(2))/8./72.;
     val=int(input->readULong(2));
@@ -1911,7 +1940,7 @@ bool PowerPoint1Parser::readColors(MWAWEntry const &entry)
     val=int(input->readLong(2));
     if (val) f << "f" << i << "=" << val << ",";
   }
-  int N=int(input->readULong(2));
+  auto N=int(input->readULong(2));
   f << "N=" << N << ",";
   if ((isMacFile && 8+(N+1)*8 != int(entry.length())) || (!isMacFile && 8+(N+1)*8 > int(entry.length()))) {
     MWAW_DEBUG_MSG(("PowerPoint1Parser::readColors: the N value seems bad\n"));
@@ -1934,7 +1963,7 @@ bool PowerPoint1Parser::readColors(MWAWEntry const &entry)
     val=int(input->readLong(2));
     if (val) {
       unsigned char col[3];
-      for (int j=0; j<3; ++j) col[j]=static_cast<unsigned char>(input->readULong(2)>>8);
+      for (auto &c : col) c=static_cast<unsigned char>(input->readULong(2)>>8);
       MWAWColor color(col[0],col[1],col[2]);
       m_state->m_idToUserColorMap[i]=color;
       f << color << ",";
@@ -1965,7 +1994,7 @@ bool PowerPoint1Parser::readColorZone(MWAWEntry const &entry)
   input->seek(pos, librevenge::RVNG_SEEK_SET);
   libmwaw::DebugStream f;
   f << "Entries(Color)[Z"<< entry.id() << "]:menu,";
-  int N=int(input->readULong(2));
+  auto N=int(input->readULong(2));
   f << "N=" << N << ",";
   if ((isMacFile && 48+2*N!=int(entry.length())) || (!isMacFile && 43+2*N>int(entry.length()))) {
     MWAW_DEBUG_MSG(("PowerPoint1Parser::readColorZone: the N value seems bad\n"));
@@ -1976,10 +2005,10 @@ bool PowerPoint1Parser::readColorZone(MWAWEntry const &entry)
     ascii().addNote("_");
     return true;
   }
-  int val=int(input->readLong(2)); // always a
+  auto val=int(input->readLong(2)); // always a
   if (val!=10) f << "f0=" << val << ",";
-  int id=int(input->readLong(isMacFile ? 4 : 2));
-  int const numZones=int(m_state->m_zonesList.size());
+  auto id=int(input->readLong(isMacFile ? 4 : 2));
+  auto const numZones=int(m_state->m_zonesList.size());
   if (id>0 && id<numZones)
     f << "colors=Z" << id << ",";
   else {
@@ -2029,7 +2058,7 @@ bool PowerPoint1Parser::readFonts(MWAWEntry const &entry)
   input->seek(pos, librevenge::RVNG_SEEK_SET);
   libmwaw::DebugStream f;
   f << "Entries(FontDef)[Z"<< entry.id() << "]:";
-  size_t N=size_t(entry.length()/6);
+  auto N=size_t(entry.length()/6);
   if (!isMacFile) {
     N=size_t(input->readULong(2)); // always 6?
     if (long(6+7*N)>entry.length()) {
@@ -2049,7 +2078,7 @@ bool PowerPoint1Parser::readFonts(MWAWEntry const &entry)
     MWAWFont font;
     font.setId(int(input->readULong(2)));
     font.setSize(float(input->readULong(2)));
-    int flag = int(input->readULong(isMacFile ? 1 : 2));
+    auto flag = int(input->readULong(isMacFile ? 1 : 2));
     uint32_t flags=0;
     if (flag&0x1) flags |= MWAWFont::boldBit;
     if (flag&0x2) flags |= MWAWFont::italicBit;
@@ -2059,7 +2088,7 @@ bool PowerPoint1Parser::readFonts(MWAWEntry const &entry)
     if (flag&0xE0) f << "#flag=" << (flag>>5) << ",";
     font.setFlags(flags);
     f << font.getDebugString(getParserState()->m_fontConverter);
-    int val=int(input->readULong(1)); // 1-4: another flag or maybe the font's color ?
+    auto val=int(input->readULong(1)); // 1-4: another flag or maybe the font's color ?
     if (val) f << "fl=" << std::hex << val << std::dec << ",";
     ascii().addPos(pos);
     ascii().addNote(f.str().c_str());
@@ -2092,7 +2121,7 @@ bool PowerPoint1Parser::readFontNames(MWAWEntry const &entry)
     if (val!=expected[i])
       f << "f" << i << "=" << std::hex << val << std::dec << ",";
   }
-  size_t N=size_t(input->readULong(2)); // always 6?
+  auto N=size_t(input->readULong(2)); // always 6?
   if (long(16+52*N)>entry.length()) {
     MWAW_DEBUG_MSG(("PowerPoint1Parser::readFontNames: the zone seems bad\n"));
     return false;
@@ -2127,7 +2156,7 @@ bool PowerPoint1Parser::readFontNames(MWAWEntry const &entry)
     }
     std::string name; // Helv, Tms Rmn, ZapfDingbats
     for (int c=0; c<32; ++c) {
-      char ch=char(input->readULong(1));
+      auto ch=char(input->readULong(1));
       if (!ch) break;
       name+=ch;
     }
@@ -2193,12 +2222,12 @@ bool PowerPoint1Parser::readScheme(MWAWEntry const &entry, int id)
   val=int(input->readLong(2));
   if (val!=7) f << "max[color]=##" << val << ",";
   f << "colors=[";
-  for (int i=0; i<8; ++i) {
+  for (auto &color : scheme.m_colors) {
     val=int(input->readULong(2));
     unsigned char col[3];
-    for (int j=0; j<3; ++j) col[j]=static_cast<unsigned char>(input->readULong(2)>>8);
-    scheme.m_colors[i]=MWAWColor(col[0],col[1],col[2]);
-    f << scheme.m_colors[i] << ":" << val << ",";
+    for (auto &c : col) c=static_cast<unsigned char>(input->readULong(2)>>8);
+    color=MWAWColor(col[0],col[1],col[2]);
+    f << color << ":" << val << ",";
   }
   f << "],";
   if (m_state->m_idToSchemeMap.find(id)!=m_state->m_idToSchemeMap.end()) {
@@ -2251,7 +2280,7 @@ bool PowerPoint1Parser::sendSlide(PowerPoint1ParserInternal::Slide const &slide,
   for (size_t f=master ? 1 : 0; f<slide.m_framesList[0].size(); ++f)
     sendFrame(slide.m_framesList[0][f], slide.m_textZones[0]);
   if (!slide.m_framesList[1].empty() && !slide.m_textZones[1].empty()) {
-    MWAWPosition pos(MWAWVec2i(0,0), MWAWVec2i(200,200), librevenge::RVNG_POINT);
+    MWAWPosition pos(MWAWVec2f(0,0), MWAWVec2f(200,200), librevenge::RVNG_POINT);
     pos.m_anchorTo = MWAWPosition::Page;
     MWAWSubDocumentPtr doc(new PowerPoint1ParserInternal::SubDocument(*this, getInput(), &slide));
     listener->insertSlideNote(pos, doc);
@@ -2267,8 +2296,7 @@ bool PowerPoint1Parser::sendSlideNote(PowerPoint1ParserInternal::Slide const &sl
     return false;
   }
   // normally, the note rectangles, followed by the note's text
-  for (size_t f=0; f<slide.m_framesList[1].size(); ++f) {
-    PowerPoint1ParserInternal::Frame const &frame=slide.m_framesList[1][f];
+  for (auto const &frame : slide.m_framesList[1]) {
     if (frame.m_type==1) continue;
     if (frame.m_type!=2 && frame.m_type!=3) {
       static bool first=true;
@@ -2333,10 +2361,10 @@ bool PowerPoint1Parser::sendText(PowerPoint1ParserInternal::TextZone const &text
   MWAWInputStreamPtr input = getInput();
   long pos;
   int const vers=version();
-  double const unit=double(m_state->m_unit);
+  auto const unit=double(m_state->m_unit);
   for (int z=tId[0]; z<=tId[1]; ++z) {
     if (z!=tId[0]) listener->insertEOL();
-    PowerPoint1ParserInternal::TextZone::Line const &line=textZone.m_lineList[size_t(z)];
+    auto const &line=textZone.m_lineList[size_t(z)];
     MWAWEntry const &fEntry=line.m_format;
     MWAWEntry const &rEntry=line.m_ruler;
     MWAWEntry const &tEntry=line.m_text;
@@ -2345,7 +2373,7 @@ bool PowerPoint1Parser::sendText(PowerPoint1ParserInternal::TextZone const &text
     para.m_tabs=ruler.m_tabs;
     para.m_justify=line.m_justify;
     if (hasRuler && line.m_outlineLevel>=0 && line.m_outlineLevel<=4) {
-      PowerPoint1ParserInternal::Ruler::Outline const &outline=ruler.m_outlines[line.m_outlineLevel];
+      auto const &outline=ruler.m_outlines[line.m_outlineLevel];
       para.m_marginsUnit=librevenge::RVNG_POINT;
       for (int i=0; i<2; ++i)
         para.m_margins[i]=unit*double(outline.m_margins[i]);
@@ -2366,14 +2394,14 @@ bool PowerPoint1Parser::sendText(PowerPoint1ParserInternal::TextZone const &text
       pos=input->tell();
       f.str("");
       f << "TextZone-F[" << i << "]:";
-      int numC=int(input->readULong(2));
+      auto numC=int(input->readULong(2));
       if (isMacFile) cPos=numC;
       f << "pos=" << cPos << ",";
       MWAWFont font;
       if (!isMacFile)
         font.setId(int(input->readULong(2)));
       font.setSize(float(input->readULong(isMacFile ? 1 : 2)));
-      int flag = int(input->readULong(isMacFile ? 1 : 2));
+      auto flag = int(input->readULong(isMacFile ? 1 : 2));
       uint32_t flags=0;
       if (flag&0x1) flags |= MWAWFont::boldBit;
       if (flag&0x2) flags |= MWAWFont::italicBit;
@@ -2385,7 +2413,7 @@ bool PowerPoint1Parser::sendText(PowerPoint1ParserInternal::TextZone const &text
       if (isMacFile)
         font.setId(int(input->readULong(2)));
       if (dtSz>=8) {
-        int col=int(input->readULong(1));
+        auto col=int(input->readULong(1));
         MWAWColor color;
         if (textZone.m_schemeId>=0 && getColor(col, textZone.m_schemeId, color)) {
           font.setColor(color);
@@ -2394,7 +2422,7 @@ bool PowerPoint1Parser::sendText(PowerPoint1ParserInternal::TextZone const &text
         }
         else
           f << "#col=" << color << ",";
-        int val=int(input->readULong(1)); // 0-255
+        auto val=int(input->readULong(1)); // 0-255
         if (val) f << "f0=" << val << ",";
       }
       if (posToFontMap.find(cPos)!=posToFontMap.end()) {
@@ -2421,13 +2449,13 @@ bool PowerPoint1Parser::sendText(PowerPoint1ParserInternal::TextZone const &text
         pos=input->tell();
         f.str("");
         f << "TextZone-R[" << i << "]:";
-        int numC=int(input->readULong(2));
+        auto numC=int(input->readULong(2));
         f << "pos=" << cPos << ",";
         MWAWParagraph cPara(para);
-        int outlineLevel=int(input->readULong(2));
+        auto outlineLevel=int(input->readULong(2));
         if (hasRuler && outlineLevel>0 && outlineLevel<=4) {
           f << "level=" << outlineLevel << ",";
-          PowerPoint1ParserInternal::Ruler::Outline const &outline=ruler.m_outlines[outlineLevel];
+          auto const &outline=ruler.m_outlines[outlineLevel];
           cPara.m_marginsUnit=librevenge::RVNG_POINT;
           for (int j=0; j<2; ++j)
             cPara.m_margins[j]=unit*double(outline.m_margins[j]);
@@ -2440,7 +2468,7 @@ bool PowerPoint1Parser::sendText(PowerPoint1ParserInternal::TextZone const &text
           MWAW_DEBUG_MSG(("PowerPoint1Parser::sendText: oops, the outline level seems bad\n"));
           f << "###outlineLevel=" << outlineLevel << ",";
         }
-        int adjust=int(input->readULong(2));
+        auto adjust=int(input->readULong(2));
         switch (adjust) {
         case 0: // left
           cPara.m_justify=MWAWParagraph::JustificationLeft;
@@ -2479,7 +2507,7 @@ bool PowerPoint1Parser::sendText(PowerPoint1ParserInternal::TextZone const &text
         listener->setParagraph(posToRulerMap.find(i)->second);
       if (posToFontMap.find(i)!=posToFontMap.end())
         listener->setFont(posToFontMap.find(i)->second);
-      unsigned char c=static_cast<unsigned char>(input->readULong(1));
+      auto c=static_cast<unsigned char>(input->readULong(1));
       switch (c) {
       case 0x9:
         listener->insertTab();
@@ -2537,7 +2565,7 @@ bool PowerPoint1Parser::sendFrame(PowerPoint1ParserInternal::Frame const &frame,
     if (frame.m_type==0)
       shape=MWAWGraphicShape::line(fBox[0], fBox[1]);
     else {
-      if (frame.m_cornerSize >= fBox.size()[0] || frame.m_cornerSize >= fBox.size()[1])
+      if (float(frame.m_cornerSize) >= fBox.size()[0] || float(frame.m_cornerSize) >= fBox.size()[1])
         shape=MWAWGraphicShape::circle(fBox);
       else
         shape=MWAWGraphicShape::rectangle(fBox, MWAWVec2f(float(frame.m_cornerSize)/2.f, float(frame.m_cornerSize)/2.f));
@@ -2587,7 +2615,7 @@ bool PowerPoint1Parser::checkHeader(MWAWHeader *header, bool strict)
   else if (signature!=0xbaddeed)
     return false;
   f << "FileHeader:";
-  int vers=int(input->readLong(4));
+  auto vers=int(input->readLong(4));
   if (vers!=2) return false;
   m_state->m_zoneListBegin=long(input->readULong(4));
   if (m_state->m_zoneListBegin<24 || !input->checkPosition(m_state->m_zoneListBegin))
@@ -2596,7 +2624,7 @@ bool PowerPoint1Parser::checkHeader(MWAWHeader *header, bool strict)
 
   if (strict) {
     input->seek(12, librevenge::RVNG_SEEK_SET);
-    int val=int(input->readULong(2));
+    auto val=int(input->readULong(2));
     if (!input->checkPosition(m_state->m_zoneListBegin+val*8))
       return false;
   }
