@@ -47,7 +47,7 @@
 namespace StarCellAttribute
 {
 //! a character bool attribute
-class StarCAttributeBool : public StarAttributeBool
+class StarCAttributeBool final : public StarAttributeBool
 {
 public:
   //! constructor
@@ -56,22 +56,20 @@ public:
   {
   }
   //! create a new attribute
-  virtual shared_ptr<StarAttribute> create() const
+  std::shared_ptr<StarAttribute> create() const final
   {
-    return shared_ptr<StarAttribute>(new StarCAttributeBool(*this));
+    return std::shared_ptr<StarAttribute>(new StarCAttributeBool(*this));
   }
   //! add to a cell style
-  virtual void addTo(StarState &state, std::set<StarAttribute const *> &/*done*/) const;
+  void addTo(StarState &state, std::set<StarAttribute const *> &/*done*/) const final;
 
 protected:
   //! copy constructor
-  StarCAttributeBool(StarCAttributeBool const &orig) : StarAttributeBool(orig)
-  {
-  }
+  StarCAttributeBool(StarCAttributeBool const &) = default;
 };
 
 //! a character color attribute
-class StarCAttributeColor : public StarAttributeColor
+class StarCAttributeColor final : public StarAttributeColor
 {
 public:
   //! constructor
@@ -79,26 +77,24 @@ public:
   {
   }
   //! destructor
-  ~StarCAttributeColor();
+  ~StarCAttributeColor() final;
   //! create a new attribute
-  virtual shared_ptr<StarAttribute> create() const
+  std::shared_ptr<StarAttribute> create() const final
   {
-    return shared_ptr<StarAttribute>(new StarCAttributeColor(*this));
+    return std::shared_ptr<StarAttribute>(new StarCAttributeColor(*this));
   }
   //! add to a cell style
-  //virtual void addTo(StarState &state, std::set<StarAttribute const *> &/*done*/) const;
+  // void addTo(StarState &state, std::set<StarAttribute const *> &/*done*/) const final;
 protected:
   //! copy constructor
-  StarCAttributeColor(StarCAttributeColor const &orig) : StarAttributeColor(orig)
-  {
-  }
+  StarCAttributeColor(StarCAttributeColor const &) = default;
 };
 
 StarCAttributeColor::~StarCAttributeColor()
 {
 }
 //! a character integer attribute
-class StarCAttributeInt : public StarAttributeInt
+class StarCAttributeInt final : public StarAttributeInt
 {
 public:
   //! constructor
@@ -107,21 +103,19 @@ public:
   {
   }
   //! add to a cell style
-  virtual void addTo(StarState &state, std::set<StarAttribute const *> &/*done*/) const;
+  void addTo(StarState &state, std::set<StarAttribute const *> &/*done*/) const final;
   //! create a new attribute
-  virtual shared_ptr<StarAttribute> create() const
+  std::shared_ptr<StarAttribute> create() const final
   {
-    return shared_ptr<StarAttribute>(new StarCAttributeInt(*this));
+    return std::shared_ptr<StarAttribute>(new StarCAttributeInt(*this));
   }
 protected:
   //! copy constructor
-  StarCAttributeInt(StarCAttributeInt const &orig) : StarAttributeInt(orig)
-  {
-  }
+  StarCAttributeInt(StarCAttributeInt const &) = default;
 };
 
 //! a character unsigned integer attribute
-class StarCAttributeUInt : public StarAttributeUInt
+class StarCAttributeUInt final : public StarAttributeUInt
 {
 public:
   //! constructor
@@ -130,21 +124,19 @@ public:
   {
   }
   //! create a new attribute
-  virtual shared_ptr<StarAttribute> create() const
+  std::shared_ptr<StarAttribute> create() const final
   {
-    return shared_ptr<StarAttribute>(new StarCAttributeUInt(*this));
+    return std::shared_ptr<StarAttribute>(new StarCAttributeUInt(*this));
   }
   //! add to a cell style
-  virtual void addTo(StarState &state, std::set<StarAttribute const *> &/*done*/) const;
+  void addTo(StarState &state, std::set<StarAttribute const *> &/*done*/) const final;
 protected:
   //! copy constructor
-  StarCAttributeUInt(StarCAttributeUInt const &orig) : StarAttributeUInt(orig)
-  {
-  }
+  StarCAttributeUInt(StarCAttributeUInt const &) = default;
 };
 
 //! a void attribute
-class StarCAttributeVoid : public StarAttributeVoid
+class StarCAttributeVoid final : public StarAttributeVoid
 {
 public:
   //! constructor
@@ -152,19 +144,17 @@ public:
   {
   }
   //! destructor
-  ~StarCAttributeVoid();
+  ~StarCAttributeVoid() final;
   //! create a new attribute
-  virtual shared_ptr<StarAttribute> create() const
+  std::shared_ptr<StarAttribute> create() const final
   {
-    return shared_ptr<StarAttribute>(new StarCAttributeVoid(*this));
+    return std::shared_ptr<StarAttribute>(new StarCAttributeVoid(*this));
   }
   //! add to a cell style
-  //virtual void addTo(StarState &state, std::set<StarAttribute const *> &/*done*/) const;
+  // void addTo(StarState &state, std::set<StarAttribute const *> &/*done*/) const final;
 protected:
   //! copy constructor
-  StarCAttributeVoid(StarCAttributeVoid const &orig) : StarAttributeVoid(orig)
-  {
-  }
+  StarCAttributeVoid(StarCAttributeVoid const &) = default;
 };
 
 StarCAttributeVoid::~StarCAttributeVoid()
@@ -274,29 +264,29 @@ void StarCAttributeUInt::addTo(StarState &state, std::set<StarAttribute const *>
 }
 
 //! add a bool attribute
-inline void addAttributeBool(std::map<int, shared_ptr<StarAttribute> > &map, StarAttribute::Type type, std::string const &debugName, bool defValue)
+inline void addAttributeBool(std::map<int, std::shared_ptr<StarAttribute> > &map, StarAttribute::Type type, std::string const &debugName, bool defValue)
 {
-  map[type]=shared_ptr<StarAttribute>(new StarCAttributeBool(type,debugName, defValue));
+  map[type]=std::shared_ptr<StarAttribute>(new StarCAttributeBool(type,debugName, defValue));
 }
 //! add a color attribute
-inline void addAttributeColor(std::map<int, shared_ptr<StarAttribute> > &map, StarAttribute::Type type, std::string const &debugName, STOFFColor const &defValue)
+inline void addAttributeColor(std::map<int, std::shared_ptr<StarAttribute> > &map, StarAttribute::Type type, std::string const &debugName, STOFFColor const &defValue)
 {
-  map[type]=shared_ptr<StarAttribute>(new StarCAttributeColor(type,debugName, defValue));
+  map[type]=std::shared_ptr<StarAttribute>(new StarCAttributeColor(type,debugName, defValue));
 }
 //! add a int attribute
-inline void addAttributeInt(std::map<int, shared_ptr<StarAttribute> > &map, StarAttribute::Type type, std::string const &debugName, int numBytes, int defValue)
+inline void addAttributeInt(std::map<int, std::shared_ptr<StarAttribute> > &map, StarAttribute::Type type, std::string const &debugName, int numBytes, int defValue)
 {
-  map[type]=shared_ptr<StarAttribute>(new StarCAttributeInt(type,debugName, numBytes, defValue));
+  map[type]=std::shared_ptr<StarAttribute>(new StarCAttributeInt(type,debugName, numBytes, defValue));
 }
 //! add a unsigned int attribute
-inline void addAttributeUInt(std::map<int, shared_ptr<StarAttribute> > &map, StarAttribute::Type type, std::string const &debugName, int numBytes, unsigned int defValue)
+inline void addAttributeUInt(std::map<int, std::shared_ptr<StarAttribute> > &map, StarAttribute::Type type, std::string const &debugName, int numBytes, unsigned int defValue)
 {
-  map[type]=shared_ptr<StarAttribute>(new StarCAttributeUInt(type,debugName, numBytes, defValue));
+  map[type]=std::shared_ptr<StarAttribute>(new StarCAttributeUInt(type,debugName, numBytes, defValue));
 }
 //! add a void attribute
-inline void addAttributeVoid(std::map<int, shared_ptr<StarAttribute> > &map, StarAttribute::Type type, std::string const &debugName)
+inline void addAttributeVoid(std::map<int, std::shared_ptr<StarAttribute> > &map, StarAttribute::Type type, std::string const &debugName)
 {
-  map[type]=shared_ptr<StarAttribute>(new StarCAttributeVoid(type,debugName));
+  map[type]=std::shared_ptr<StarAttribute>(new StarCAttributeVoid(type,debugName));
 }
 
 }
@@ -304,25 +294,25 @@ inline void addAttributeVoid(std::map<int, shared_ptr<StarAttribute> > &map, Sta
 namespace StarCellAttribute
 {
 //! a margins attribute
-class StarCAttributeMargins : public StarAttribute
+class StarCAttributeMargins final : public StarAttribute
 {
 public:
   //! constructor
   StarCAttributeMargins(Type type, std::string const &debugName) : StarAttribute(type, debugName)
   {
-    for (int i=0; i<4; ++i) m_margins[i]=20;
+    for (int &margin : m_margins) margin=20;
   }
   //! create a new attribute
-  virtual shared_ptr<StarAttribute> create() const
+  std::shared_ptr<StarAttribute> create() const final
   {
-    return shared_ptr<StarAttribute>(new StarCAttributeMargins(*this));
+    return std::shared_ptr<StarAttribute>(new StarCAttributeMargins(*this));
   }
   //! read a zone
-  virtual bool read(StarZone &zone, int vers, long endPos, StarObject &object);
+  bool read(StarZone &zone, int vers, long endPos, StarObject &object) final;
   //! add to a cell style
-  virtual void addTo(StarState &state, std::set<StarAttribute const *> &/*done*/) const;
+  void addTo(StarState &state, std::set<StarAttribute const *> &/*done*/) const final;
   //! debug function to print the data
-  virtual void printData(libstoff::DebugStream &o) const
+  void printData(libstoff::DebugStream &o) const final
   {
     o << m_debugName << "=[";
     for (int i=0; i<4; ++i) {
@@ -343,7 +333,7 @@ protected:
 };
 
 //! a merge attribute
-class StarCAttributeMerge : public StarAttribute
+class StarCAttributeMerge final : public StarAttribute
 {
 public:
   //! constructor
@@ -351,16 +341,16 @@ public:
   {
   }
   //! create a new attribute
-  virtual shared_ptr<StarAttribute> create() const
+  std::shared_ptr<StarAttribute> create() const final
   {
-    return shared_ptr<StarAttribute>(new StarCAttributeMerge(*this));
+    return std::shared_ptr<StarAttribute>(new StarCAttributeMerge(*this));
   }
   //! read a zone
-  virtual bool read(StarZone &zone, int vers, long endPos, StarObject &object);
+  bool read(StarZone &zone, int vers, long endPos, StarObject &object) final;
   //! add to a cell style
-  virtual void addTo(StarState &state, std::set<StarAttribute const *> &/*done*/) const;
+  void addTo(StarState &state, std::set<StarAttribute const *> &/*done*/) const final;
   //! debug function to print the data
-  virtual void printData(libstoff::DebugStream &o) const
+  void printData(libstoff::DebugStream &o) const final
   {
     o << m_debugName;
     if (m_span!=STOFFVec2i(1,1)) o << "=" << m_span;
@@ -369,15 +359,13 @@ public:
 
 protected:
   //! copy constructor
-  StarCAttributeMerge(StarCAttributeMerge const &orig) : StarAttribute(orig), m_span(orig.m_span)
-  {
-  }
+  StarCAttributeMerge(StarCAttributeMerge const &) = default;
   //! the span
   STOFFVec2i m_span;
 };
 
 //! Pattern attribute of StarAttributeInternal
-class StarCAttributePattern : public StarAttributeItemSet
+class StarCAttributePattern final : public StarAttributeItemSet
 {
 public:
   //! constructor
@@ -387,14 +375,14 @@ public:
     m_limits.push_back(STOFFVec2i(100,148));
   }
   //! destructor
-  ~StarCAttributePattern();
+  ~StarCAttributePattern() final;
   //! create a new attribute
-  virtual shared_ptr<StarAttribute> create() const
+  std::shared_ptr<StarAttribute> create() const final
   {
-    return shared_ptr<StarAttribute>(new StarCAttributePattern(*this));
+    return std::shared_ptr<StarAttribute>(new StarCAttributePattern(*this));
   }
   //! read a zone
-  virtual bool read(StarZone &zone, int /*vers*/, long endPos, StarObject &object)
+  bool read(StarZone &zone, int /*vers*/, long endPos, StarObject &object) final
   {
     STOFFInputStreamPtr input=zone.input();
     long pos=input->tell();
@@ -430,7 +418,7 @@ public:
     return ok && input->tell()<=endPos;
   }
   //! debug function to print the data
-  virtual void print(libstoff::DebugStream &o, std::set<StarAttribute const *> &done) const
+  void print(libstoff::DebugStream &o, std::set<StarAttribute const *> &done) const final
   {
     StarAttributeItemSet::print(o, done);
     if (m_style.empty())
@@ -439,9 +427,7 @@ public:
   }
 protected:
   //! copy constructor
-  StarCAttributePattern(StarCAttributePattern const &orig) : StarAttributeItemSet(orig), m_style(orig.m_style)
-  {
-  }
+  StarCAttributePattern(StarCAttributePattern const &) = default;
   //! the style
   librevenge::RVNGString m_style;
 };
@@ -451,7 +437,7 @@ StarCAttributePattern::~StarCAttributePattern()
 }
 
 //! a protection attribute
-class StarCAttributeProtection : public StarAttribute
+class StarCAttributeProtection final : public StarAttribute
 {
 public:
   //! constructor
@@ -460,16 +446,16 @@ public:
   {
   }
   //! create a new attribute
-  virtual shared_ptr<StarAttribute> create() const
+  std::shared_ptr<StarAttribute> create() const final
   {
-    return shared_ptr<StarAttribute>(new StarCAttributeProtection(*this));
+    return std::shared_ptr<StarAttribute>(new StarCAttributeProtection(*this));
   }
   //! read a zone
-  virtual bool read(StarZone &zone, int vers, long endPos, StarObject &object);
+  bool read(StarZone &zone, int vers, long endPos, StarObject &object) final;
   //! add to a cell style
-  virtual void addTo(StarState &state, std::set<StarAttribute const *> &/*done*/) const;
+  void addTo(StarState &state, std::set<StarAttribute const *> &/*done*/) const final;
   //! debug function to print the data
-  virtual void printData(libstoff::DebugStream &o) const
+  void printData(libstoff::DebugStream &o) const final
   {
     o << m_debugName << "=[";
     if (m_protected) o << "protected,";
@@ -481,9 +467,7 @@ public:
 
 protected:
   //! copy constructor
-  StarCAttributeProtection(StarCAttributeProtection const &orig) : StarAttribute(orig), m_protected(orig.m_protected), m_hiddenFormula(orig.m_hiddenFormula), m_hiddenCell(orig.m_hiddenCell), m_doNotPrint(orig.m_doNotPrint)
-  {
-  }
+  StarCAttributeProtection(StarCAttributeProtection const &) = default;
   //! the cell is protected
   bool m_protected;
   //! the formula is hidden
@@ -541,8 +525,8 @@ bool StarCAttributeMargins::read(StarZone &zone, int /*vers*/, long endPos, Star
   libstoff::DebugFile &ascFile=zone.ascii();
   libstoff::DebugStream f;
   f << "Entries(StarAttribute)[" << zone.getRecordLevel() << "]:";
-  for (int i=0; i<4; ++i)
-    m_margins[i]=int(input->readLong(2));
+  for (int &margin : m_margins)
+    margin=int(input->readLong(2));
   printData(f);
   ascFile.addPos(pos);
   ascFile.addNote(f.str().c_str());
@@ -558,8 +542,8 @@ bool StarCAttributeMerge::read(StarZone &zone, int /*vers*/, long endPos, StarOb
   libstoff::DebugStream f;
   f << "Entries(StarAttribute)[" << zone.getRecordLevel() << "]:";
   int span[2];
-  for (int i=0; i<2; ++i)
-    span[i]=int(input->readLong(2));
+  for (int &i : span)
+    i=int(input->readLong(2));
   m_span=STOFFVec2i(span[0], span[1]);
   printData(f);
   ascFile.addPos(pos);
@@ -588,7 +572,7 @@ bool StarCAttributeProtection::read(StarZone &zone, int /*vers*/, long endPos, S
 
 namespace StarCellAttribute
 {
-void addInitTo(std::map<int, shared_ptr<StarAttribute> > &map)
+void addInitTo(std::map<int, std::shared_ptr<StarAttribute> > &map)
 {
   // --- sc --- sc_docpool.cxx
   addAttributeVoid(map, StarAttribute::ATTR_SC_USERDEF, "sc[userDef]");
@@ -602,10 +586,10 @@ void addInitTo(std::map<int, shared_ptr<StarAttribute> > &map)
   addAttributeUInt(map, StarAttribute::ATTR_SC_ROTATE_MODE,"rotate[mode]",2,0); // normal
   addAttributeUInt(map, StarAttribute::ATTR_SC_WRITINGDIR,"writing[dir]",2,4); // frame environment
 
-  map[StarAttribute::ATTR_SC_PATTERN]=shared_ptr<StarAttribute>(new StarCAttributePattern());
-  map[StarAttribute::ATTR_SC_MARGIN]=shared_ptr<StarAttribute>(new StarCAttributeMargins(StarAttribute::ATTR_SC_MARGIN,"scMargins"));
-  map[StarAttribute::ATTR_SC_MERGE]=shared_ptr<StarAttribute>(new StarCAttributeMerge(StarAttribute::ATTR_SC_MERGE,"scMerge"));
-  map[StarAttribute::ATTR_SC_PROTECTION]=shared_ptr<StarAttribute>(new StarCAttributeProtection(StarAttribute::ATTR_SC_PROTECTION,"scProtection"));
+  map[StarAttribute::ATTR_SC_PATTERN]=std::shared_ptr<StarAttribute>(new StarCAttributePattern());
+  map[StarAttribute::ATTR_SC_MARGIN]=std::shared_ptr<StarAttribute>(new StarCAttributeMargins(StarAttribute::ATTR_SC_MARGIN,"scMargins"));
+  map[StarAttribute::ATTR_SC_MERGE]=std::shared_ptr<StarAttribute>(new StarCAttributeMerge(StarAttribute::ATTR_SC_MERGE,"scMerge"));
+  map[StarAttribute::ATTR_SC_PROTECTION]=std::shared_ptr<StarAttribute>(new StarCAttributeProtection(StarAttribute::ATTR_SC_PROTECTION,"scProtection"));
 
   // TOUSE
   addAttributeBool(map, StarAttribute::ATTR_SC_VERTICAL_ASIAN,"vertical[asian]", false);
