@@ -53,37 +53,45 @@ namespace StarObjectInternal
 //! the state of a StarObject
 struct State {
   //! constructor
-  State() : m_poolList(), m_attributeManager(new StarAttributeManager), m_formatManager(new StarFormatManager)
+  State()
+    : m_poolList()
+    , m_attributeManager(new StarAttributeManager)
+    , m_formatManager(new StarFormatManager)
   {
   }
   //! copy constructor
-  State(State const &orig) : m_poolList(orig.m_poolList), m_attributeManager(orig.m_attributeManager), m_formatManager(orig.m_formatManager)
-  {
-    for (int i=0; i<4; ++i)
-      m_userMetaNames[i]=orig.m_userMetaNames[i];
-  }
+  State(State const &) = default;
   //! the list of pool
-  std::vector<shared_ptr<StarItemPool> > m_poolList;
+  std::vector<std::shared_ptr<StarItemPool> > m_poolList;
   //! the attribute manager
-  shared_ptr<StarAttributeManager> m_attributeManager;
+  std::shared_ptr<StarAttributeManager> m_attributeManager;
   //! the format manager
-  shared_ptr<StarFormatManager> m_formatManager;
+  std::shared_ptr<StarFormatManager> m_formatManager;
   //! the list of user name
   librevenge::RVNGString m_userMetaNames[4];
 private:
-  State operator=(State const &orig);
+  State operator=(State const &) = delete;
 };
 }
 
 ////////////////////////////////////////////////////////////
 // constructor/destructor, ...
 ////////////////////////////////////////////////////////////
-StarObject::StarObject(char const *passwd, shared_ptr<STOFFOLEParser> oleParser, shared_ptr<STOFFOLEParser::OleDirectory> directory) :
-  m_password(passwd), m_oleParser(oleParser), m_directory(directory), m_state(new StarObjectInternal::State()), m_metaData()
+StarObject::StarObject(char const *passwd, std::shared_ptr<STOFFOLEParser> oleParser, std::shared_ptr<STOFFOLEParser::OleDirectory> directory)
+  : m_password(passwd)
+  , m_oleParser(oleParser)
+  , m_directory(directory)
+  , m_state(new StarObjectInternal::State())
+  , m_metaData()
 {
 }
 
-StarObject::StarObject(StarObject const &orig, bool duplicateState) : m_password(orig.m_password), m_oleParser(orig.m_oleParser), m_directory(orig.m_directory), m_state(), m_metaData(orig.m_metaData)
+StarObject::StarObject(StarObject const &orig, bool duplicateState)
+  : m_password(orig.m_password)
+  , m_oleParser(orig.m_oleParser)
+  , m_directory(orig.m_directory)
+  , m_state()
+  , m_metaData(orig.m_metaData)
 {
   if (duplicateState)
     m_state.reset(new StarObjectInternal::State(*orig.m_state));
@@ -97,9 +105,9 @@ StarObject::~StarObject()
 
 void StarObject::cleanPools()
 {
-  for (size_t i=0; i<m_state->m_poolList.size(); ++i) {
-    if (m_state->m_poolList[i])
-      m_state->m_poolList[i]->clean();
+  for (auto &p : m_state->m_poolList) {
+    if (p)
+      p->clean();
   }
   m_state->m_poolList.clear();
 }
@@ -109,12 +117,12 @@ STOFFDocument::Kind StarObject::getDocumentKind() const
   return m_directory ? m_directory->m_kind : STOFFDocument::STOFF_K_UNKNOWN;
 }
 
-shared_ptr<StarAttributeManager> StarObject::getAttributeManager()
+std::shared_ptr<StarAttributeManager> StarObject::getAttributeManager()
 {
   return m_state->m_attributeManager;
 }
 
-shared_ptr<StarFormatManager> StarObject::getFormatManager()
+std::shared_ptr<StarFormatManager> StarObject::getFormatManager()
 {
   return m_state->m_formatManager;
 }
@@ -131,32 +139,32 @@ librevenge::RVNGString StarObject::getUserNameMetaData(int i) const
   return res;
 }
 
-shared_ptr<StarItemPool> StarObject::getNewItemPool(StarItemPool::Type type)
+std::shared_ptr<StarItemPool> StarObject::getNewItemPool(StarItemPool::Type type)
 {
-  shared_ptr<StarItemPool> pool(new StarItemPool(*this, type));
+  std::shared_ptr<StarItemPool> pool(new StarItemPool(*this, type));
   m_state->m_poolList.push_back(pool);
   return pool;
 }
 
-shared_ptr<StarItemPool> StarObject::getCurrentPool(bool onlyInside)
+std::shared_ptr<StarItemPool> StarObject::getCurrentPool(bool onlyInside)
 {
   for (size_t i=m_state->m_poolList.size(); i>0;) {
-    shared_ptr<StarItemPool> pool=m_state->m_poolList[--i];
+    auto pool=m_state->m_poolList[--i];
     if (pool && !pool->isSecondaryPool() && (!onlyInside || pool->isInside()))
       return pool;
   }
-  return shared_ptr<StarItemPool>();
+  return std::shared_ptr<StarItemPool>();
 }
 
-shared_ptr<StarItemPool> StarObject::findItemPool(StarItemPool::Type type, bool isInside)
+std::shared_ptr<StarItemPool> StarObject::findItemPool(StarItemPool::Type type, bool isInside)
 {
   for (size_t i=m_state->m_poolList.size(); i>0;) {
-    shared_ptr<StarItemPool> pool=m_state->m_poolList[--i];
+    auto pool=m_state->m_poolList[--i];
     if (!pool || pool->getType()!=type) continue;
     if (isInside && !pool->isInside()) continue;
     return pool;
   }
-  return shared_ptr<StarItemPool>();
+  return std::shared_ptr<StarItemPool>();
 }
 
 bool StarObject::parse()
@@ -168,11 +176,10 @@ bool StarObject::parse()
   if (!m_directory->m_hasCompObj) {
     STOFF_DEBUG_MSG(("StarObject::parse: called with unknown document\n"));
   }
-  for (size_t i = 0; i < m_directory->m_contentList.size(); ++i) {
-    STOFFOLEParser::OleContent &content=m_directory->m_contentList[i];
+  for (auto &content : m_directory->m_contentList) {
     if (content.isParsed()) continue;
-    std::string name = content.getOleName();
-    std::string const &base = content.getBaseName();
+    auto name = content.getOleName();
+    auto const &base = content.getBaseName();
     STOFFInputStreamPtr ole;
     if (m_directory->m_input)
       ole = m_directory->m_input->getSubStreamByName(name.c_str());
@@ -264,7 +271,7 @@ bool StarObject::readItemSet(StarZone &zone, std::vector<STOFFVec2i> const &/*li
   ascFile.addNote(f.str().c_str());
   for (int i=0; i<int(n); ++i) {
     pos=input->tell();
-    shared_ptr<StarItem> item=pool->readItem(zone, isDirect, lastPos);
+    auto item=pool->readItem(zone, isDirect, lastPos);
     if (item && input->tell()<=lastPos) {
       itemSet.add(item);
       continue;
@@ -344,7 +351,7 @@ bool StarObject::readPersistElements(STOFFInputStreamPtr input, std::string cons
   f.str("");
   f << "Persists-B:";
   int dim[4];
-  for (int i=0; i<4; ++i) dim[i]=int(input->readLong(4));
+  for (int &i : dim) i=int(input->readLong(4));
   f << "dim=" << STOFFBox2i(STOFFVec2i(dim[0],dim[1]), STOFFVec2i(dim[2],dim[3])) << ",";
   val=int(input->readLong(2)); // 0|9
   if (val) f << "f0=" << val << ",";
@@ -605,7 +612,7 @@ bool StarObject::readPersistData(StarZone &zone, long lastPos, bool inPersistEle
 
 bool StarObject::readSfxDocumentInformation(STOFFInputStreamPtr input, std::string const &name)
 {
-  StarZone zone(input, name, "SfxDocInfo", 0); // no password
+  StarZone zone(input, name, "SfxDocInfo", nullptr); // no password
   libstoff::DebugFile &ascii=zone.ascii();
   ascii.open(name);
   input->seek(0, librevenge::RVNG_SEEK_SET);
@@ -640,7 +647,7 @@ bool StarObject::readSfxDocumentInformation(STOFFInputStreamPtr input, std::stri
   if (bPasswd) f << "passwd,"; // the password does not seems to be kept/used in this block
   if (bPGraphic) f << "portableGraphic,";
   if (bQTemplate) f << "queryTemplate,";
-  StarEncoding::Encoding encoding=StarEncoding::getEncodingForId(nUS);
+  auto encoding=StarEncoding::getEncodingForId(nUS);
   ascii.addPos(0);
   ascii.addNote(f.str().c_str());
 
@@ -675,7 +682,7 @@ bool StarObject::readSfxDocumentInformation(STOFFInputStreamPtr input, std::stri
     std::vector<uint32_t> finalString;
     std::vector<size_t> srcPositions;
     if (StarEncoding::convert(string, encoding, finalString, srcPositions)) {
-      librevenge::RVNGString attrib=libstoff::getString(finalString);
+      auto attrib=libstoff::getString(finalString);
       f << attrib.cstr() << ",";
       static char const *(attribNames[]) = {
         "meta:initial-creator", "dc:creator", "", "dc:title", "dc:subject", "dc:description"/*comment*/, "meta:keywords",
@@ -762,12 +769,12 @@ bool StarObject::readSfxStyleSheets(STOFFInputStreamPtr input, std::string const
 
   // sd_sdbinfilter.cxx SdBINFilter::Import: one pool followed by a pool style
   // chart sch_docshell.cxx SchChartDocShell::Load
-  shared_ptr<StarItemPool> pool;
+  std::shared_ptr<StarItemPool> pool;
   if (getDocumentKind()==STOFFDocument::STOFF_K_DRAW) {
     pool=getNewItemPool(StarItemPool::T_XOutdevPool);
     pool->addSecondaryPool(getNewItemPool(StarItemPool::T_EditEnginePool));
   }
-  shared_ptr<StarItemPool>  mainPool=pool;
+  auto mainPool=pool;
   while (!input->isEnd()) {
     // REMOVEME: remove this loop, when creation of secondary pool is checked
     long pos=input->tell();
