@@ -13,16 +13,18 @@
 // All changes made under the Poppler project to this file are licensed
 // under GPL version 2 or later
 //
-// Copyright (C) 2005, 2007, 2011 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2005, 2007, 2011, 2018 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2006 Rainer Keller <class321@gmx.de>
 // Copyright (C) 2008 Timothy Lee <timothy.lee@siriushk.com>
 // Copyright (C) 2008 Vasile Gaburici <gaburici@cs.umd.edu>
 // Copyright (C) 2009 Carlos Garcia Campos <carlosgc@gnome.org>
 // Copyright (C) 2009 William Bader <williambader@hotmail.com>
 // Copyright (C) 2010 Jakob Voss <jakob.voss@gbv.de>
-// Copyright (C) 2012, 2013, 2017 Adrian Johnson <ajohnson@redneon.com>
+// Copyright (C) 2012, 2013, 2017, 2018 Adrian Johnson <ajohnson@redneon.com>
 // Copyright (C) 2013 Thomas Fischer <fischer@unix-ag.uni-kl.de>
 // Copyright (C) 2013 Hib Eris <hib@hiberis.nl>
+// Copyright (C) 2017 Caolán McNamara <caolanm@redhat.com>
+// Copyright (C) 2018 Andreas Gruenbacher <agruenba@redhat.com>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -245,7 +247,9 @@ void ImageOutputDev::listImage(GfxState *state, Object *ref, Stream *str,
     printf("%5.0f ", yppi);
 
   Goffset embedSize = -1;
-  if (!inlineImg)
+  if (inlineImg)
+    embedSize = getInlineImageLength(str, width, height, colorMap);
+  else
     embedSize = str->getBaseStream()->getLength();
 
   long long imageSize = 0;
@@ -289,25 +293,39 @@ void ImageOutputDev::listImage(GfxState *state, Object *ref, Stream *str,
 
   ++imgNum;
 
-  if (inlineImg) {
-    // For inline images we need to advance the stream position to the end of the image
-    // as Gfx needs to continue reading content after the image data.
-    ImageFormat format;
-    if (!colorMap || (colorMap->getNumPixelComps() == 1 && colorMap->getBits() == 1)) {
-      format = imgMonochrome;
-    } else if (colorMap->getColorSpace()->getMode() == csDeviceGray ||
-               colorMap->getColorSpace()->getMode() == csCalGray) {
-      format = imgGray;
-    } else if ((colorMap->getColorSpace()->getMode() == csDeviceRGB ||
-		colorMap->getColorSpace()->getMode() == csCalRGB ||
-		(colorMap->getColorSpace()->getMode() == csICCBased && colorMap->getNumPixelComps() == 3)) &&
-	       colorMap->getBits() > 8) {
-      format = imgRGB48;
-    } else {
-      format = imgRGB;
+}
+
+long ImageOutputDev::getInlineImageLength(Stream *str, int width, int height,
+                                          GfxImageColorMap *colorMap) {
+  long len;
+
+  if (colorMap) {
+    ImageStream *imgStr = new ImageStream(str, width, colorMap->getNumPixelComps(),
+                                          colorMap->getBits());
+    imgStr->reset();
+    for (int y = 0; y < height; y++)
+      imgStr->getLine();
+
+    imgStr->close();
+    delete imgStr;
+  } else {
+    str->reset();
+    for (int y = 0; y < height; y++) {
+      int size = (width + 7)/8;
+      for (int x = 0; x < size; x++)
+        str->getChar();
     }
-    writeImageFile(NULL, format, "", str, width, height, colorMap);
   }
+
+  EmbedStream *embedStr = (EmbedStream *) (str->getBaseStream());
+  embedStr->rewind();
+  len = 0;
+  while (embedStr->getChar() != EOF)
+    len++;
+
+  embedStr->restore();
+
+  return len;
 }
 
 void ImageOutputDev::writeRawImage(Stream *str, const char *ext) {
@@ -336,15 +354,15 @@ void ImageOutputDev::writeRawImage(Stream *str, const char *ext) {
 
 void ImageOutputDev::writeImageFile(ImgWriter *writer, ImageFormat format, const char *ext,
                                     Stream *str, int width, int height, GfxImageColorMap *colorMap) {
-  FILE *f;
-  ImageStream *imgStr = NULL;
+  FILE *f = nullptr; /* squelch bogus compiler warning */
+  ImageStream *imgStr = nullptr;
   unsigned char *row;
   unsigned char *rowp;
   Guchar *p;
   GfxRGB rgb;
   GfxCMYK cmyk;
   GfxGray gray;
-  Guchar zero = 0;
+  Guchar zero[gfxColorMaxComps];
   int invert_bits;
 
   if (writer) {
@@ -383,7 +401,8 @@ void ImageOutputDev::writeImageFile(ImgWriter *writer, ImageFormat format, const
   // the mask we leave the data unchanged.
   invert_bits = 0xff;
   if (colorMap) {
-    colorMap->getGray(&zero, &gray);
+    memset(zero, 0, sizeof(zero));
+    colorMap->getGray(zero, &gray);
     if (colToByte(gray) == 0)
       invert_bits = 0x00;
   }
@@ -496,12 +515,17 @@ void ImageOutputDev::writeImage(GfxState *state, Object *ref, Stream *str,
 				int width, int height,
 				GfxImageColorMap *colorMap, GBool inlineImg) {
   ImageFormat format;
+  EmbedStream *embedStr;
 
-  if (dumpJPEG && str->getKind() == strDCT &&
-      (colorMap->getNumPixelComps() == 1 ||
-       colorMap->getNumPixelComps() == 3) &&
-      !inlineImg) {
+  if (inlineImg) {
+      embedStr = (EmbedStream *) (str->getBaseStream());
+      // Record the stream. This determines the size.
+      getInlineImageLength(str, width, height, colorMap);
+      // Reading the stream again will return EOF at end of recording.
+      embedStr->rewind();
+  }
 
+  if (dumpJPEG && str->getKind() == strDCT) {
     // dump JPEG file
     writeRawImage(str, "jpg");
 
@@ -516,24 +540,24 @@ void ImageOutputDev::writeImage(GfxState *state, Object *ref, Stream *str,
     if (globals->isStream()) {
       FILE *f;
       int c;
-      Stream *str = globals->getStream();
+      Stream *globalsStr = globals->getStream();
 
       setFilename("jb2g");
       if (!(f = fopen(fileName, "wb"))) {
         error(errIO, -1, "Couldn't open image file '{0:s}'", fileName);
         return;
       }
-      str->reset();
-      while ((c = str->getChar()) != EOF)
+      globalsStr->reset();
+      while ((c = globalsStr->getChar()) != EOF)
         fputc(c, f);
-      str->close();
+      globalsStr->close();
       fclose(f);
     }
 
     // dump JBIG2 embedded file
     writeRawImage(str, "jb2e");
 
-  } else if (dumpCCITT && str->getKind() == strCCITTFax && !inlineImg) {
+  } else if (dumpCCITT && str->getKind() == strCCITTFax) {
     // write CCITT parameters
     CCITTFaxStream *ccittStr = static_cast<CCITTFaxStream *>(str);
     FILE *f;
@@ -572,10 +596,9 @@ void ImageOutputDev::writeImage(GfxState *state, Object *ref, Stream *str,
                             (colorMap->getColorSpace()->getMode() == csDeviceCMYK ||
                              (colorMap->getColorSpace()->getMode() == csICCBased &&
                               colorMap->getNumPixelComps() == 4)))) {
-
     // output in PNG format
 
-#if ENABLE_LIBPNG
+#ifdef ENABLE_LIBPNG
     ImgWriter *writer;
 
     if (!colorMap || (colorMap->getNumPixelComps() == 1 && colorMap->getBits() == 1)) {
@@ -597,12 +620,13 @@ void ImageOutputDev::writeImage(GfxState *state, Object *ref, Stream *str,
     }
 
     writeImageFile(writer, format, "png", str, width, height, colorMap);
-#endif
 
+    delete writer;
+#endif
   } else if (outputTiff) {
     // output in TIFF format
 
-#if ENABLE_LIBTIFF
+#ifdef ENABLE_LIBTIFF
     ImgWriter *writer;
 
     if (!colorMap || (colorMap->getNumPixelComps() == 1 && colorMap->getBits() == 1)) {
@@ -628,8 +652,9 @@ void ImageOutputDev::writeImage(GfxState *state, Object *ref, Stream *str,
     }
 
     writeImageFile(writer, format, "tif", str, width, height, colorMap);
-#endif
 
+    delete writer;
+#endif
   } else {
     // output in PPM/PBM format
     ImgWriter *writer;
@@ -648,6 +673,9 @@ void ImageOutputDev::writeImage(GfxState *state, Object *ref, Stream *str,
 
     delete writer;
   }
+
+  if (inlineImg)
+      embedStr->restore();
 }
 
 GBool ImageOutputDev::tilingPatternFill(GfxState *state, Gfx *gfx, Catalog *cat, Object *str,
@@ -663,9 +691,9 @@ void ImageOutputDev::drawImageMask(GfxState *state, Object *ref, Stream *str,
 				   int width, int height, GBool invert,
 				   GBool interpolate, GBool inlineImg) {
   if (listImages)
-    listImage(state, ref, str, width, height, NULL, interpolate, inlineImg, imgStencil);
+    listImage(state, ref, str, width, height, nullptr, interpolate, inlineImg, imgStencil);
   else
-    writeImage(state, ref, str, width, height, NULL, inlineImg);
+    writeImage(state, ref, str, width, height, nullptr, inlineImg);
 }
 
 void ImageOutputDev::drawImage(GfxState *state, Object *ref, Stream *str,
@@ -684,10 +712,10 @@ void ImageOutputDev::drawMaskedImage(
   Stream *maskStr, int maskWidth, int maskHeight, GBool maskInvert, GBool maskInterpolate) {
   if (listImages) {
     listImage(state, ref, str, width, height, colorMap, interpolate, gFalse, imgImage);
-    listImage(state, ref, str, maskWidth, maskHeight, NULL, maskInterpolate, gFalse, imgMask);
+    listImage(state, ref, str, maskWidth, maskHeight, nullptr, maskInterpolate, gFalse, imgMask);
   } else {
     writeImage(state, ref, str, width, height, colorMap, gFalse);
-    writeImage(state, ref, maskStr, maskWidth, maskHeight, NULL, gFalse);
+    writeImage(state, ref, maskStr, maskWidth, maskHeight, nullptr, gFalse);
   }
 }
 
