@@ -7,6 +7,7 @@
 #ifndef XFA_FXFA_CXFA_WIDGETACCITERATOR_H_
 #define XFA_FXFA_CXFA_WIDGETACCITERATOR_H_
 
+#include "core/fxcrt/unowned_ptr.h"
 #include "xfa/fxfa/parser/cxfa_traversestrategy_xfacontainernode.h"
 
 class CXFA_Node;
@@ -22,7 +23,7 @@ class CXFA_WidgetAccIterator {
 
  private:
   CXFA_ContainerIterator m_ContentIterator;
-  CXFA_WidgetAcc* m_pCurWidgetAcc;  // not owned.
+  UnownedPtr<CXFA_WidgetAcc> m_pCurWidgetAcc;
 };
 
 #endif  // XFA_FXFA_CXFA_WIDGETACCITERATOR_H_
